@@ -21,18 +21,20 @@ from pathlib import Path
 import click
 import requests
 import tabulate
-from rq import Connection, Worker
+from rq import Worker
 from sqlalchemy import exc as sa_exc
 
 from dallinger import data, db
 from dallinger.command_line.develop import develop
 from dallinger.command_line.docker import docker
 from dallinger.command_line.docker_ssh import docker_ssh
+from dallinger.command_line.prolific import prolific
 from dallinger.command_line.utils import (
     Output,
     header,
     log,
     require_exp_directory,
+    run_pre_launch_checks,
     verify_id,
     verify_package,
 )
@@ -129,6 +131,19 @@ dallinger.add_command(develop)
 dallinger.add_command(docker)
 dallinger.add_command(docker_ssh)
 
+try:
+    from dallinger.command_line.ec2 import ec2
+
+    dallinger.add_command(ec2)
+except ImportError:
+    log(
+        "Could not import EC2 support. "
+        "Install dallinger with the ec2 extra to use EC2 related commands."
+    )
+    pass
+
+dallinger.add_command(prolific)
+
 
 @dallinger.command()
 def setup():
@@ -227,11 +242,18 @@ def prelaunch_db_bootstrapper(zip_path, log):
     return bootstrap_db
 
 
-def _deploy_in_mode(mode, verbose, log, app=None, archive=None):
+def _deploy_in_mode(mode, verbose, app=None, archive=None):
     if app:
         verify_id(None, None, app)
 
     log(header, chevrons=False)
+
+    config = get_config()
+    config.load()
+    config.extend({"mode": mode, "logfile": "-"})
+
+    run_pre_launch_checks(**locals())
+
     prelaunch = []
     if archive:
         archive_path = os.path.abspath(archive)
@@ -241,10 +263,6 @@ def _deploy_in_mode(mode, verbose, log, app=None, archive=None):
             )
         prelaunch.append(prelaunch_db_bootstrapper(archive_path, log))
 
-    config = get_config()
-    config.load()
-    config.extend({"mode": mode, "logfile": "-"})
-
     return deploy_sandbox_shared_setup(
         log=log, verbose=verbose, app=app, prelaunch_actions=prelaunch
     )
@@ -277,11 +295,9 @@ def fail_on_unsupported_urls(f):
 @require_exp_directory
 @fail_on_unsupported_urls
 @report_idle_after(21600)
-def sandbox(verbose, app, archive):
+def sandbox(**kwargs):
     """Deploy app using Heroku to the MTurk Sandbox."""
-    return _deploy_in_mode(
-        mode="sandbox", verbose=verbose, log=log, app=app, archive=archive
-    )
+    return _deploy_in_mode(mode="sandbox", **kwargs)
 
 
 @dallinger.command()
@@ -291,11 +307,9 @@ def sandbox(verbose, app, archive):
 @require_exp_directory
 @fail_on_unsupported_urls
 @report_idle_after(21600)
-def deploy(verbose, app, archive):
+def deploy(**kwargs):
     """Deploy app using Heroku to MTurk."""
-    return _deploy_in_mode(
-        mode="live", verbose=verbose, log=log, app=app, archive=archive
-    )
+    return _deploy_in_mode(mode="live", **kwargs)
 
 
 @dallinger.command()
@@ -536,7 +550,7 @@ def hits(app, sandbox, recruiter):
     if app is not None:
         verify_id(None, "--app", app)
     prolific_check(recruiter, sandbox)
-    rec = by_name(recruiter, skip_config_validation=True)
+    rec = by_name(recruiter)
     rec.hits(app, sandbox)
 
 
@@ -552,7 +566,7 @@ def hits(app, sandbox, recruiter):
 def hit_details(hit_id, sandbox, recruiter):
     """Print the details of a specific HIT for a recruiter."""
     prolific_check(recruiter, sandbox)
-    rec = by_name(recruiter, skip_config_validation=True)
+    rec = by_name(recruiter)
     details = rec.hit_details(hit_id, sandbox)
     print(json.dumps(details, indent=4, default=str))
 
@@ -570,7 +584,7 @@ def hit_details(hit_id, sandbox, recruiter):
 def copy_qualifications(hit_id, sandbox, recruiter, path):
     """Copy qualifications from an existing HIT and save them to a JSON file."""
     prolific_check(recruiter, sandbox)
-    rec = by_name(recruiter, skip_config_validation=True)
+    rec = by_name(recruiter)
     if path is None:
         path = rec.default_qualification_name
     assert path.endswith(".json"), "Qualification path must be a json file"
@@ -873,10 +887,9 @@ def verify():
 def rq_worker():
     """Start an rq worker in the context of dallinger."""
     setup_experiment(log)
-    with Connection(db.redis_conn):
-        # right now we care about low queue for bots
-        worker = Worker("low")
-        worker.work()
+    # right now we care about low queue for bots
+    worker = Worker("low", connection=db.redis_conn)
+    worker.work()
 
 
 @dallinger.command()
