@@ -1,13 +1,14 @@
 """This module includes utilities to store and retrieve dallinger
 configuration for the command line in the current user directory.
 """
+
 import json
 from pathlib import Path
 from typing import Dict
 
-from dallinger.command_line import appdirs
+import platformdirs
 
-APPDIRS = appdirs.AppDirs("dallinger", "dallinger")
+APPDIRS = platformdirs.PlatformDirs("dallinger", "dallinger")
 
 
 def get_configured_hosts():
