@@ -1,4 +1,3 @@
-import logging
 import subprocess
 
 import click
@@ -14,10 +13,11 @@ from dallinger.command_line.utils import (
 )
 from dallinger.config import get_config
 from dallinger.db import redis_conn
-from dallinger.deployment import DevelopmentDeployment, _handle_launch_data
-from dallinger.utils import develop_target_path, open_browser
+from dallinger.deployment import DevelopmentDeployment, handle_launch_data
+from dallinger.utils import develop_target_path, open_browser, setup_warning_hooks
+
+setup_warning_hooks()
 
-logger = logging.getLogger(__name__)
 BASE_URL = "http://127.0.0.1:{}/"
 
 
@@ -89,7 +89,7 @@ def launch_app_and_open_dashboard(port):
 
 def _launch_app(port):
     url = BASE_URL.format(port) + "launch"
-    _handle_launch_data(url, error=log, delay=1.0)
+    handle_launch_data(url, error=log, delay=1.0, context="local")
 
 
 def _open_dashboard(port):
@@ -114,7 +114,7 @@ def _browser(route=None, port=5000):
     url_factory = valid_routes.get(route)
     if url_factory is not None:
         url = url_factory(config, port)
-        logger.info("Experiment dashboard: %s", url)
+        log(f"Experiment dashboard: {url}")
         open_browser(url)
     else:
         click.echo(
