@@ -21,10 +21,11 @@ from dallinger.command_line.utils import (
     header,
     log,
     require_exp_directory,
+    run_pre_launch_checks,
     verify_id,
 )
 from dallinger.config import get_config
-from dallinger.deployment import _handle_launch_data
+from dallinger.deployment import handle_launch_data
 from dallinger.heroku.tools import HerokuApp
 from dallinger.utils import GitClient, abspath_from_egg, setup_experiment
 
@@ -103,18 +104,18 @@ def remove_services_data():
 @click.option("--verbose", is_flag=True, flag_value=True, help="Verbose mode")
 @click.option("--app", default=None, help="Experiment id")
 @require_exp_directory
-def sandbox(verbose, app):
+def sandbox(**kwargs):
     """Deploy app using Heroku to the MTurk Sandbox."""
-    return _deploy_in_mode(mode="sandbox", verbose=verbose, log=log, app=app)
+    return _deploy_in_mode(mode="sandbox", **kwargs)
 
 
 @docker.command()
 @click.option("--verbose", is_flag=True, flag_value=True, help="Verbose mode")
 @click.option("--app", default=None, help="Experiment id")
 @require_exp_directory
-def deploy(verbose, app):
+def deploy(**kwargs):
     """Deploy app using Heroku to MTurk."""
-    return _deploy_in_mode(mode="live", verbose=verbose, log=log, app=app)
+    return _deploy_in_mode(mode="live", **kwargs)
 
 
 @docker.command()
@@ -139,7 +140,8 @@ def push(use_existing: bool, **kwargs) -> str:
 
     config = get_config()
     config.load()
-    _, tmp = setup_experiment(log=log, debug=True, local_checks=False)
+    app_name = kwargs.get("app_name", None)
+    _, tmp = setup_experiment(log=log, debug=True, local_checks=False, app=app_name)
     image_name_with_tag = build_image(
         tmp,
         config.get("docker_image_base_name"),
@@ -203,7 +205,8 @@ def deploy_image(image_name, mode, config_options):
     """Deploy Heroku app using a docker image and MTurk."""
     config = get_config()
     config.load()
-    dashboard_password = secrets.token_urlsafe(8)
+    dashboard_user = config.get("dashboard_user", "admin")
+    dashboard_password = config.get("dashboard_password", secrets.token_urlsafe(8))
     dallinger_uid = str(uuid.uuid4())
     config_dict = {
         "AWS_ACCESS_KEY_ID": config.get("aws_access_key_id"),
@@ -215,6 +218,7 @@ def deploy_image(image_name, mode, config_options):
         "smtp_password": config.get("smtp_password"),
         "whimsical": config.get("whimsical"),
         "FLASK_SECRET_KEY": secrets.token_urlsafe(16),
+        "dashboard_user": dashboard_user,
         "dashboard_password": dashboard_password,
         "mode": mode,
         "CREATOR": netrc.netrc().hosts["api.heroku.com"][0],
@@ -273,7 +277,7 @@ def deploy_image(image_name, mode, config_options):
             dict(
                 type=type,
                 quantity=config.get(f"num_dynos_{type}", 1),
-                size=config.get("dyno_type", "hobby"),
+                size=config.get("dyno_type", "basic"),
             )
             for type in services
         ]
@@ -286,7 +290,7 @@ def deploy_image(image_name, mode, config_options):
 
     print("Launching experiment")
     app_url = f"https://{app_hostname}"
-    launch_data = _handle_launch_data(f"{app_url}/launch", print)
+    launch_data = handle_launch_data(f"{app_url}/launch", print, context="heroku")
     print(launch_data.get("recruitment_msg"))
 
     print(
@@ -294,15 +298,18 @@ def deploy_image(image_name, mode, config_options):
     )
 
 
-def _deploy_in_mode(mode, verbose, log, app=None):
+def _deploy_in_mode(mode, verbose, app=None):
     if app:
         verify_id(None, None, app)
 
     log(header, chevrons=False)
+
     config = get_config()
     config.load()
     config.extend({"mode": mode, "logfile": "-"})
 
+    run_pre_launch_checks(**locals())
+
     return deploy_heroku_docker(log=log, verbose=verbose, app=app)
 
 
@@ -325,7 +332,7 @@ def deploy_heroku_docker(log, verbose=True, app=None, exp_config=None):
     build_image(tmp, Path(os.getcwd()).name, Output(), force_build=True)
 
     # Push the built image to get the registry sha256
-    image_name = push.callback(use_existing=True)
+    image_name = push.callback(use_existing=True, app_name=app)
 
     # Log in to Heroku if we aren't already.
     log("Making sure that you are logged in to Heroku.")
@@ -349,7 +356,10 @@ def deploy_heroku_docker(log, verbose=True, app=None, exp_config=None):
 
     out = None if verbose else open(os.devnull, "w")
     team = config.get("heroku_team", None)
-    heroku_app = HerokuApp(dallinger_uid=heroku_app_id, output=out, team=team)
+    region = config.get("heroku_region", None)
+    heroku_app = HerokuApp(
+        dallinger_uid=heroku_app_id, output=out, team=team, region=region
+    )
     heroku_app.bootstrap(buildpack=None)
 
     # Set up add-ons and AWS environment variables.
@@ -422,7 +432,7 @@ def deploy_heroku_docker(log, verbose=True, app=None, exp_config=None):
     log("Launching the experiment on the remote server and starting recruitment...")
     launch_url = "{}/launch".format(heroku_app.url)
     log("Calling {}".format(launch_url), chevrons=False)
-    launch_data = _handle_launch_data(launch_url, error=log)
+    launch_data = handle_launch_data(launch_url, error=log, context="heroku")
     result = {
         "app_name": heroku_app.name,
         "app_home": heroku_app.url,
