@@ -1,6 +1,5 @@
 from __future__ import unicode_literals
 
-import distutils.util
 import io
 import json
 import logging
@@ -13,7 +12,7 @@ from pathlib import Path
 import six
 from six.moves import configparser
 
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 marker = object()
 
@@ -25,9 +24,33 @@ def is_valid_json(value):
     json.loads(value)
 
 
+def strtobool(val: str) -> int:
+    """Convert a string representation of truth to true (1) or false (0).
+
+    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
+    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
+    'val' is anything else.
+
+    Notes
+    -----
+    This implementation is derived from the `strtobool` function in setuptools,
+    which itself is based on the original in Python's distutils.
+
+    Source: https://github.com/pypa/setuptools/blob/main/setuptools/dist.py
+
+    Copyright (c) 2016-2024 Python Packaging Authority (PyPA)
+    Licensed under the MIT License.
+    """
+    val = val.lower()
+    if val in ("y", "yes", "t", "true", "on", "1"):
+        return 1
+    if val in ("n", "no", "f", "false", "off", "0"):
+        return 0
+    raise ValueError(f"invalid truth value {val!r}")
+
+
 default_keys = (
-    # These are the keys allowed in a dallinger experiment
-    # config.txt file.
+    # These are the keys allowed in a dallinger experiment config.txt file.
     ("ad_group", six.text_type, []),
     ("approve_requirement", int, []),
     ("assign_qualifications", bool, []),
@@ -53,11 +76,14 @@ default_keys = (
     ("database_url", six.text_type, [], True),
     ("debug_recruiter", six.text_type, []),
     ("description", six.text_type, []),
+    ("disable_browser_autotranslate", bool, []),
     ("disable_when_duration_exceeded", bool, []),
     ("duration", float, []),
     ("dyno_type", six.text_type, []),
     ("dyno_type_web", six.text_type, []),
     ("dyno_type_worker", six.text_type, []),
+    ("ec2_default_pem", six.text_type, []),
+    ("ec2_default_security_group", six.text_type, []),
     ("enable_global_experiment_registry", bool, []),
     ("EXPERIMENT_CLASS_NAME", six.text_type, []),
     ("group_name", six.text_type, []),
@@ -65,14 +91,17 @@ default_keys = (
     ("heroku_auth_token", six.text_type, [], True),
     ("heroku_python_version", six.text_type, []),
     ("heroku_team", six.text_type, ["team"]),
+    ("heroku_region", six.text_type, []),
     ("host", six.text_type, []),
     ("id", six.text_type, []),
     ("infrastructure_debug_details", six.text_type, [], False),
     ("keywords", six.text_type, []),
     ("language", six.text_type, []),
     ("lifetime", int, []),
+    ("lock_table_when_creating_participant", bool, []),
     ("logfile", six.text_type, []),
     ("loglevel", int, []),
+    ("loglevel_worker", int, []),
     ("mode", six.text_type, []),
     ("mturk_qualification_blocklist", six.text_type, ["qualification_blacklist"]),
     ("mturk_qualification_requirements", six.text_type, [], False, [is_valid_json]),
@@ -83,10 +112,13 @@ default_keys = (
     ("prolific_api_token", six.text_type, ["PROLIFIC_RESEARCHER_API_TOKEN"], True),
     ("prolific_api_version", six.text_type, []),
     ("prolific_estimated_completion_minutes", int, []),
+    ("prolific_is_custom_screening", bool, []),
     ("prolific_maximum_allowed_minutes", int, []),
+    ("prolific_project", six.text_type, []),
     ("prolific_recruitment_config", six.text_type, [], False, [is_valid_json]),
-    ("prolific_reward_cents", int, []),
+    ("prolific_workspace", six.text_type, []),
     ("protected_routes", six.text_type, [], False, [is_valid_json]),
+    ("publish_experiment", bool, []),
     ("recruiter", six.text_type, []),
     ("recruiters", six.text_type, []),
     ("redis_size", six.text_type, []),
@@ -106,6 +138,8 @@ default_keys = (
     ("docker_image_base_name", six.text_type, [], ""),
     ("docker_image_name", six.text_type, [], ""),
     ("docker_volumes", six.text_type, [], ""),
+    ("docker_worker_cpu_shares", int, [], ""),
+    ("server_pem", six.text_type, []),
 )
 
 
@@ -140,10 +174,11 @@ class Configuration(object):
         normalized_mapping = {}
         for key, value in mapping.items():
             key = self.synonyms.get(key, key)
+            test_deprecation(key)
             if key not in self.types:
                 # This key hasn't been registered, we ignore it
                 if strict:
-                    raise KeyError("{} is not a valid configuration key".format(key))
+                    raise_invalid_key_error(key)
                 continue
             expected_type = self.types.get(key)
             if cast_types:
@@ -153,8 +188,8 @@ class Configuration(object):
                     with io.open(filename, "rt", encoding="utf-8") as source_file:
                         value = source_file.read()
                 try:
-                    if expected_type == bool:
-                        value = distutils.util.strtobool(value)
+                    if expected_type is bool:
+                        value = strtobool(value)
                     value = expected_type(value)
                 except ValueError:
                     pass
@@ -181,7 +216,17 @@ class Configuration(object):
         yield self
         self.data.popleft()
 
+    changeable_params = ["auto_recruit"]
+
     def get(self, key, default=marker):
+        # For now this is limited to "auto_recruit", but in the future it can be extended
+        # to other parameters as well
+        if key == "auto_recruit":
+            from dallinger.db import redis_conn
+
+            auto_recruit = redis_conn.get("auto_recruit")
+            if auto_recruit is not None:
+                return bool(int(auto_recruit))
         if not self.ready:
             raise RuntimeError("Config not loaded")
         for layer in self.data:
@@ -193,10 +238,10 @@ class Configuration(object):
             except KeyError:
                 continue
         if default is marker:
-            raise KeyError(
-                f"The following config parameter was not set: {key}. Consider setting it in "
-                "config.txt or in ~/.dallingerconfig."
-            )
+            error_text = f"The following config parameter was not set: {key}. Consider setting it in config.txt or in ~/.dallingerconfig."
+            if key == "prolific_project":
+                error_text += " Prolific projects will be created automatically if they don't exist already."
+            raise KeyError(error_text)
         return default
 
     def __getitem__(self, key):
@@ -211,10 +256,10 @@ class Configuration(object):
         except KeyError:
             raise AttributeError
 
-    def as_dict(self):
+    def as_dict(self, include_sensitive=False):
         d = {}
         for key in self.types:
-            if key not in self.sensitive:
+            if key not in self.sensitive or include_sensitive:
                 try:
                     d[key] = self.get(key)
                 except KeyError:
@@ -244,13 +289,13 @@ class Configuration(object):
         if sensitive:
             self.sensitive.add(key)
 
-    def load_from_file(self, filename):
+    def load_from_file(self, filename, strict=True):
         parser = configparser.ConfigParser()
         parser.read(filename)
         data = {}
         for section in parser.sections():
             data.update(dict(parser.items(section)))
-        self.extend(data, cast_types=True, strict=True)
+        self.extend(data, cast_types=True, strict=strict)
 
     def write(self, filter_sensitive=False, directory=None):
         parser = configparser.ConfigParser()
@@ -269,7 +314,7 @@ class Configuration(object):
     def load_from_environment(self):
         self.extend(os.environ, cast_types=True)
 
-    def load_defaults(self):
+    def load_defaults(self, strict=True):
         """Load default configuration values"""
         # Apply extra parameters before loading the configs
         if experiment_available():
@@ -285,18 +330,20 @@ class Configuration(object):
         )
 
         # Load the configuration, with local parameters overriding global ones.
-        for config_file in [global_defaults_file, local_defaults_file, global_config]:
-            self.load_from_file(config_file)
+        for config_file in [global_defaults_file, local_defaults_file]:
+            self.load_from_file(config_file, strict)
 
         if experiment_available():
             self.load_experiment_config_defaults()
 
-    def load(self):
-        self.load_defaults()
+        self.load_from_file(global_config, strict)
+
+    def load(self, strict=True):
+        self.load_defaults(strict)
 
         localConfig = os.path.join(os.getcwd(), LOCAL_CONFIG)
         if os.path.exists(localConfig):
-            self.load_from_file(localConfig)
+            self.load_from_file(localConfig, strict)
 
         self.load_from_environment()
         self.ready = True
@@ -386,3 +433,28 @@ def initialize_experiment_package(path):
 
 def experiment_available():
     return Path("experiment.py").exists()
+
+
+def raise_invalid_key_error(key):
+    error_text = "{} is not a valid configuration key".format(key)
+    if key == "prolific_reward_cents":
+        error_text = (
+            "The 'prolific_reward_cents' config variable has been removed. "
+            + "Use 'base_payment' instead to set base compensation for participants. "
+            + "Note that base_payment is written in terms of the base unit for the currency, "
+            + "not in cents. So, if your prolific_reward_cents was originally set to 50, "
+            + "then you should set your base_payment to 0.5."
+        )
+    raise KeyError(error_text)
+
+
+def test_deprecation(key):
+    if key == "prolific_maximum_allowed_minutes":
+        import warnings
+
+        warnings.simplefilter("always", DeprecationWarning)
+        warnings.warn(
+            "The 'prolific_maximum_allowed_minutes' config variable has no effect "
+            + "as it is currently ignored by the Prolific API.",
+            DeprecationWarning,
+        )
