@@ -23,40 +23,65 @@ from dallinger.utils import (
     bootstrap_development_session,
     get_base_url,
     open_browser,
+    print_bold,
     setup_experiment,
 )
 
-INITIAL_DELAY = 1
+DEFAULT_DELAY = 1
 BACKOFF_FACTOR = 2
 MAX_ATTEMPTS = 6
 
 
-def _handle_launch_data(url, error, delay=INITIAL_DELAY, attempts=MAX_ATTEMPTS):
+def handle_launch_data(
+    url,
+    error,
+    delay=DEFAULT_DELAY,
+    attempts=MAX_ATTEMPTS,
+    dns_host=None,
+    dozzle_password=None,
+    context=None,
+):
+    """Sends a POST request to the given `url`, retrying it with exponential backoff.
+    The passed `error` function is invoked to give feedback as each error occurs,
+    possibly multiple times. If all attempts fail, an exception is raised.
+
+    Args:
+        url: The URL to send the POST request to
+        error: Function to call with error messages
+        delay: Initial delay between retries in seconds
+        attempts: Maximum number of retry attempts
+        dns_host: Hostname for Docker SSH deployments
+        dozzle_password: Password for Dozzle logs in Docker SSH deployments
+        context: Deployment context ('heroku', 'ssh', 'local', etc.)
+    """
     launch_data = None
     launch_request = None
     for remaining_attempt in sorted(range(attempts), reverse=True):  # [3, 2, 1, 0]
-        time.sleep(delay)
         try:
             launch_request = requests.post(url)
             request_happened = True
-        except requests.exceptions.RequestException:
+        except requests.exceptions.RequestException as err:
             request_happened = False
-            error(f"Error accessing {url}")
+            error(f"Error accessing {url}:\n{err}")
 
-        try:
-            if request_happened:
+        if request_happened:
+            try:
                 launch_data = launch_request.json()
-        except ValueError:
-            error(
-                "Error parsing response from {}, "
-                "check web dyno logs for details: {}".format(url, launch_request.text)
-            )
-            raise
-        except json.decoder.JSONDecodeError:
-            # The Heroku backend did not return JSON. It means our dallinger instance
-            # was not (yet) running at the time of the request.
-            # We treat this similarly to a RequestException: we'll try again after waiting.
-            request_happened = False
+            except json.decoder.JSONDecodeError:
+                # The backend did not return JSON. It means our dallinger instance
+                # was not (yet) running at the time of the request.
+                # We treat this similarly to a RequestException: we'll try again after waiting.
+                request_happened = False
+                error(
+                    f"Error parsing response from {url}, "
+                    f"check server logs for details.\n{launch_request.text}"
+                )
+            except ValueError as err:
+                error(
+                    f"Error parsing response from {url}, "
+                    f"check server logs for details.\n{err}\n{launch_request.text}"
+                )
+                raise
 
         # Early return if successful
         if request_happened and launch_request.ok:
@@ -78,13 +103,27 @@ def _handle_launch_data(url, error, delay=INITIAL_DELAY, attempts=MAX_ATTEMPTS):
                     next_attempt_count, attempts, delay
                 )
             )
+        time.sleep(delay)
 
-    error("Experiment launch failed, check web dyno logs for details.")
+    error("Experiment launch failed after multiple attempts.")
     if launch_data and launch_data.get("message"):
         error(launch_data["message"])
+
+    # Show appropriate log location message based on deployment context
+    if context == "heroku":
+        print_bold(
+            "For detailed server logs, visit the Papertrail add-on in your Heroku dashboard"
+        )
+    elif context == "ssh" and dns_host and dozzle_password:
+        print_bold(
+            f"Check the detailed server logs at https://logs.{dns_host} (user = dallinger, password = {dozzle_password})"
+        )
+
     if launch_request is not None:
         launch_request.raise_for_status()
 
+    raise requests.exceptions.ConnectionError
+
 
 def deploy_sandbox_shared_setup(
     log, verbose=True, app=None, exp_config=None, prelaunch_actions=None
@@ -127,7 +166,10 @@ def deploy_sandbox_shared_setup(
     # Initialize the app on Heroku.
     log("Initializing app on Heroku...")
     team = config.get("heroku_team", None)
-    heroku_app = HerokuApp(dallinger_uid=heroku_app_id, output=out, team=team)
+    region = config.get("heroku_region", None)
+    heroku_app = HerokuApp(
+        dallinger_uid=heroku_app_id, output=out, team=team, region=region
+    )
     heroku_app.bootstrap()
     heroku_app.buildpack("https://github.com/stomita/heroku-buildpack-phantomjs")
 
@@ -215,7 +257,7 @@ def deploy_sandbox_shared_setup(
     log("Launching the experiment on the remote server and starting recruitment...")
     launch_url = "{}/launch".format(heroku_app.url)
     log("Calling {}".format(launch_url), chevrons=False)
-    launch_data = _handle_launch_data(launch_url, error=log)
+    launch_data = handle_launch_data(launch_url, error=log)
     result = {
         "app_name": heroku_app.name,
         "app_home": heroku_app.url,
@@ -257,21 +299,14 @@ class DevelopmentDeployment(object):
     def __init__(self, output, exp_config):
         self.out = output
         self.exp_config = exp_config or {}
-        self.exp_config.update({"mode": "debug", "loglevel": 0})
+        self.exp_config.update({"mode": "debug"})
 
     def run(self):
         """Bootstrap the environment and reset the database."""
-        self.out.log("Preparing your pristine development environment...")
         experiment_uid, dst = bootstrap_development_session(
             self.exp_config, os.getcwd(), self.out.log
         )
-        self.out.log("Re-initializing database...")
         db.init_db(drop_all=True)
-        self.out.log(
-            f"Files symlinked in {dst}.\n"
-            "Run './run.sh' in that directory to start Flask, "
-            "plus the worker and clock processes."
-        )
 
 
 class HerokuLocalDeployment(object):
@@ -285,7 +320,7 @@ class HerokuLocalDeployment(object):
     DO_INIT_DB = True
 
     def configure(self):
-        self.exp_config.update({"mode": "debug", "loglevel": 0})
+        self.exp_config.update({"mode": "debug"})
 
     def setup(self):
         self.exp_id, self.tmp_dir = setup_experiment(
@@ -389,7 +424,7 @@ class DebugDeployment(HerokuLocalDeployment):
         self.out.log("Server is running on {}. Press Ctrl+C to exit.".format(base_url))
         self.out.log("Launching the experiment...")
         try:
-            result = _handle_launch_data(
+            result = handle_launch_data(
                 "{}/launch".format(base_url), error=self.out.error, attempts=1
             )
         except Exception:
@@ -544,7 +579,7 @@ class LoaderDeployment(HerokuLocalDeployment):
         if self.exp_config.get("replay"):
             self.out.log("Launching the experiment...")
             time.sleep(4)
-            _handle_launch_data("{}/launch".format(base_url), error=self.out.error)
+            handle_launch_data("{}/launch".format(base_url), error=self.out.error)
             heroku.monitor(listener=self.notify)
 
         # Just run until interrupted:
