@@ -12,7 +12,11 @@ from pip._internal.network.session import PipSession
 from pip._internal.req import parse_requirements
 
 from dallinger.docker.wheel_filename import parse_wheel_filename
-from dallinger.utils import abspath_from_egg, get_editable_dallinger_path
+from dallinger.utils import (
+    JSON_LOGFILE,
+    abspath_from_egg,
+    get_editable_dallinger_path,
+)
 
 docker_compose_template = Template(
     abspath_from_egg("dallinger", "dallinger/docker/docker-compose.yml.j2").read_text()
@@ -57,20 +61,31 @@ class DockerComposeWrapper(object):
         self._record = []
         self.needs_chrome = needs_chrome
 
-    def copy_docker_compse_files(self):
+    def copy_docker_compose_files(self):
         """Prepare a docker-compose.yml file and place it in the experiment tmp dir"""
+        logger_filename = JSON_LOGFILE
         volumes = [
             f"{self.original_dir}:{self.original_dir}",
             f"{self.tmp_dir}:/experiment",
+            f"./{logger_filename}:/experiment/{logger_filename}",
         ]
+        with open(f"{self.tmp_dir}/{logger_filename}", "w") as f:  # noqa: F841
+            pass
+
         editable_dallinger_path = get_editable_dallinger_path()
         if editable_dallinger_path:
             volumes.append(f"{editable_dallinger_path}/dallinger:/dallinger/dallinger")
             volumes.append(
-                f"{editable_dallinger_path}/dallinger:/usr/local/lib/python3.8/dist-packages/dallinger/"
+                f"{editable_dallinger_path}/dallinger:/usr/local/lib/python3.13/dist-packages/dallinger/"
             )
             volumes.append(
-                f"{editable_dallinger_path}/dallinger:/usr/local/lib/python3.9/dist-packages/dallinger/"
+                f"{editable_dallinger_path}/dallinger:/usr/local/lib/python3.12/dist-packages/dallinger/"
+            )
+            volumes.append(
+                f"{editable_dallinger_path}/dallinger:/usr/local/lib/python3.11/dist-packages/dallinger/"
+            )
+            volumes.append(
+                f"{editable_dallinger_path}/dallinger:/usr/local/lib/python3.10/dist-packages/dallinger/"
             )
         tag = get_experiment_image_tag(self.tmp_dir)
         with open(os.path.join(self.tmp_dir, "docker-compose.yml"), "w") as fh:
@@ -115,7 +130,7 @@ class DockerComposeWrapper(object):
         self.out.blather("Postgresql ready\n")
 
     def start(self):
-        self.copy_docker_compse_files()
+        self.copy_docker_compose_files()
         build_image(self.tmp_dir, self.experiment_name, self.out, self.needs_chrome)
         check_output("docker compose up -d".split())
         # Wait for postgres to complete initialization
