@@ -17,15 +17,16 @@ from contextlib import contextmanager
 from functools import wraps
 from importlib import import_module
 from operator import itemgetter
+from typing import List, Optional, Union
 
 import requests
 from cached_property import cached_property
 from flask import Blueprint
-from sqlalchemy import and_, create_engine, distinct, func
-from sqlalchemy.orm import scoped_session, sessionmaker
+from sqlalchemy import Table, and_, create_engine, func
+from sqlalchemy.orm import scoped_session, sessionmaker, undefer
 from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound
 
-from dallinger import models, recruiters
+from dallinger import db, models, recruiters
 from dallinger.config import LOCAL_CONFIG, get_config, initialize_experiment_package
 from dallinger.data import (
     Data,
@@ -35,16 +36,22 @@ from dallinger.data import (
     is_registered,
 )
 from dallinger.data import load as data_load
-from dallinger.db import db_url, init_db
+from dallinger.db import (
+    Base,
+    db_url,
+    get_mapped_class,
+    get_polymorphic_mapping,
+    init_db,
+)
 from dallinger.heroku.tools import HerokuApp
 from dallinger.information import Gene, Meme, State
-from dallinger.models import Info, Network, Node, Participant, Transformation, Vector
+from dallinger.models import Info, Network, Node, Participant, Transformation
 from dallinger.networks import Empty
 from dallinger.nodes import Agent, Environment, Source
 from dallinger.transformations import Compression, Mutation, Replication, Response
-from dallinger.utils import deferred_route_decorator, struct_to_html
+from dallinger.utils import classproperty, deferred_route_decorator, struct_to_html
 
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 def exp_class_working_dir(meth):
@@ -70,13 +77,20 @@ class Experiment(object):
     """Define the structure of an experiment."""
 
     app_id = None
-    # Optional Redis channel to create and subscribe to on launch. Note that if
-    # you define a channel, you probably also want to override the send()
-    # method, since this is where messages from Redis will be sent.
-    channel = None
     exp_config = None
     replay_path = "/"
 
+    #: Optional Redis channel to subscribe to on launch. Note that
+    #: if you set the channel, you will probably also want to override the
+    #: :func:`~dallinger.experiment.Experiment.send` method, since this
+    #: is where messages from Redis will be consumed. Setting a value here
+    #: will also result in the experiment being subscribed to the
+    #: ``dallinger_control`` channel for messages related to
+    #: socket/subscription updates. This is also the default ``channel_name``
+    #: for messages sent using the
+    #: :func:`~dallinger.experiment.Experiment.publish_to_subscribers` method.
+    channel = None
+
     #: Constructor for Participant objects. Callable returning an instance of
     #: :attr:`~dallinger.models.Participant` or a sub-class. Used by
     #: :func:`~dallinger.experiment.Experiment.create_participant`.
@@ -98,7 +112,14 @@ class Experiment(object):
 
     #: Sequence of dashboard route/function names that should be excluded from
     #: rendering as tabs in the dashboard view.
-    hidden_dashboards = ()
+    @classproperty
+    def hidden_dashboards(cls):
+        return []
+
+    @classmethod
+    def organize_dashboard_tabs(cls, tabs):
+        """A hook for custom organization of dashboard tabs in subclasses."""
+        return tabs
 
     def __init__(self, session=None):
         """Create the experiment class. Sets the default value of attributes."""
@@ -173,6 +194,42 @@ class Experiment(object):
         else:
             self.widget = module.ExperimentWidget(self)
 
+    @staticmethod
+    def before_request():
+        return None
+
+    @staticmethod
+    def after_request(request, response):
+        return response
+
+    @staticmethod
+    def gunicorn_when_ready(server):
+        pass
+
+    @staticmethod
+    def gunicorn_on_exit(server):
+        pass
+
+    @staticmethod
+    def gunicorn_worker_exit(server, worker):
+        pass
+
+    @staticmethod
+    def gunicorn_post_worker_init(worker):
+        pass
+
+    @classmethod
+    def get_status(cls):
+        """
+        Return the status of the experiment as a dictionary.
+        """
+        n_working_participants = (
+            db.session.query(func.count(Participant.id))
+            .filter_by(status="working")
+            .scalar()
+        )
+        return {"n_working_participants": n_working_participants}
+
     @classmethod
     def config_class(cls):
         """
@@ -284,27 +341,171 @@ class Experiment(object):
         return waiting_count > self.quorum
 
     def send(self, raw_message):
-        """socket interface implementation, and point of entry for incoming
-        Redis messages.
+        """Async implementation of websocket message processing. Attempts to
+        extract a participant id or node id from the message, and send the
+        message to be processed asynchronously by
+        :func:`~dallinger.experiment.Experiment.receive_message` If it fails to
+        find a participant or node id in the message, then the message is
+        processed synchronously using
+        :func:`~dallinger.experiment.Experiment.receive_message`.
 
-        param raw_message is a string with a channel prefix, for example:
+        ``raw_message`` is a string that includes a channel name prefix, for
+        example a JSON message for a ``shopping`` channel might look like:
 
-            'shopping:{"type":"buy","color":"blue","quantity":"2"}'
+            ``'shopping:{"type":"buy","color":"blue","quantity":"2"}'``
+
+        Control messages about channel subscription and websocket
+        connect/disconnect events use the ``dallinger_control`` channel name.
+
+        Experiments can override this method if they want to process all
+        messages synchronously in a single application instance by default. For
+        example if an experiment retains non-persisted state in an attribute of
+        the experiment class that it uses for message responses then it's best
+        to override this method instead of
+        :func:`~dallinger.experiment.Experiment.receive_message`, and explicitly
+        hand off state synchronization and other database writes to async worker
+        events.
+
+        :param raw_message: a formatted message string ``'$channel_name:$data'``
+        :type raw_message: str
+        """
+        from dallinger.experiment_server.worker_events import worker_function
+
+        receive_time = datetime.datetime.now()
+        channel_name, message_string = raw_message.split(":", 1)
+        try:
+            message = json.loads(message_string)
+        except Exception:
+            # Not JSON we have no information about the participant/node and
+            # will run synchonously
+            with db.sessions_scope():
+                self.receive_message(
+                    message_string, channel_name=channel_name, receive_time=receive_time
+                )
+            return
+
+        participant_id = (
+            message.get("sender")
+            or message.get("participant_id")
+            or message.get("client", {}).get("participant_id")
+        )
+        node_id = message.get("node_id")
+        if not participant_id and not node_id:
+            with db.sessions_scope():
+                self.receive_message(
+                    message_string, channel_name=channel_name, receive_time=receive_time
+                )
+            return
+
+        q = db.get_queue("high")
+        q.enqueue(
+            worker_function,
+            "WebSocketMessage",
+            None,
+            participant_id,
+            node_id=node_id,
+            receive_timestamp=receive_time.timestamp(),
+            details={
+                "message": message_string,
+                "channel_name": channel_name,
+            },
+            queue_name="high",
+        )
+
+    @classmethod
+    def handle_recruitment_error(cls, error, **kwargs):
+        """Handle errors that occur during recruitment.
+
+        This method provides a hook which is called upon a recruitment error.
+
+        :param error: The exception or error object that occurred.
+        :type error: Exception
+        :param kwargs: Additional context or metadata about the error.
         """
         pass
 
+    def receive_message(
+        self, message, channel_name=None, participant=None, node=None, receive_time=None
+    ):
+        """Stub implementation of a websocket message processor. Messages
+        are are queued to be processed asynchronously by
+        :func:`~dallinger.experiment.Experiment.send` and the worker runs this
+        method to process those messages. Sub-classes that wish to handle
+        incoming messages asynchronously should override this method. Generally
+        this method should always be overridden whenever the ``Experiment``
+        :attr:`~dallinger.experiment.Experiment.channel` attribute is set.
+
+        ``message`` is a string, e.g. containing JSON formatted data.
+
+        Control messages about channel subscription and websocket
+        connect/disconnect events are sent over the ``"dallinger_control"``
+        channel.
+
+        This method is called synchronously when no participant or node
+        id can be determined from the message.
+
+        :param message: a websocket message
+        :type message: str
+
+        :param channel_name: The name of the channel the message was received on.
+        :type channel_name: str
+
+        :param participant: the experiment participant object responsible for the message
+        :type participant: :attr:`~dallinger.models.Participant` instance
+
+        :param node: the experiment node the message corresponds to
+        :type node: :attr:`~dallinger.models.Node` instance
+
+        :param receive_time: The time the message was received by the experiment
+        :type receive_time: datetime.datetime
+        """
+        pass
+
+    def publish_to_subscribers(self, data, channel_name=None):
+        """Publish data to the given channel_name. Data will be sent to all
+        channel subscribers, potentially including the experiment instance
+        itself. If no ``channel_name`` is specified, then the ``Experiment``
+        :attr:`~dallinger.experiment.Experiment.channel` value will be used
+        (and the data will automatically be consumed by
+        :func:`~dallinger.experiment.Experiment.send`). The ``data`` must be
+        a string, it typically contains JSON.
+
+        :param data: the message data to be send
+        :type data: str
+        :param channel_name: the name of the channel to publish the data to
+        :type channel_name: str
+        """
+        if channel_name is None:
+            channel_name = self.channel
+        db.redis_conn.publish(channel_name, data)
+
+    def client_info(self):
+        """Returns a JSON compatible dictionary with data about this client to
+        be included in control channel messages.
+        """
+        return {
+            "class": self.__class__.__module__ + "." + self.__class__.__name__,
+        }
+
     def setup(self):
         """Create the networks if they don't already exist."""
+        # XXX: This is typically called from the constructor, conditional on the
+        # `session` argument. We should probably do this initialization more
+        # explicitly elsewhere, especially since the default constructor doesn't
+        # call it.
         if not self.networks():
-            for _ in range(self.practice_repeats):
-                network = self.create_network()
-                network.role = "practice"
-                self.session.add(network)
-            for _ in range(self.experiment_repeats):
-                network = self.create_network()
-                network.role = "experiment"
-                self.session.add(network)
-            self.session.commit()
+            # This should generally be called from a flask route, but we use a
+            # session contextmanager to be safe.
+            with db.sessions_scope() as session:
+                for _ in range(self.practice_repeats):
+                    network = self.create_network()
+                    network.role = "practice"
+                    session.add(network)
+                for _ in range(self.experiment_repeats):
+                    network = self.create_network()
+                    network.role = "experiment"
+                    session.add(network)
+                session.commit()
 
     def create_network(self):
         """Return a new network."""
@@ -319,16 +520,18 @@ class Experiment(object):
 
         if full == "all":
             if role == "all":
-                return Network.query.all()
+                return self.session.query(Network).all()
             else:
-                return Network.query.filter_by(role=role).all()
+                return self.session.query(Network).filter_by(role=role).all()
         else:
             if role == "all":
-                return Network.query.filter_by(full=full).all()
+                return self.session.query(Network).filter_by(full=full).all()
             else:
-                return Network.query.filter(
-                    and_(Network.role == role, Network.full == full)
-                ).all()
+                return (
+                    self.session.query(Network)
+                    .filter(and_(Network.role == role, Network.full == full))
+                    .all()
+                )
 
     def get_network_for_participant(self, participant):
         """Find a network for a participant.
@@ -341,11 +544,11 @@ class Experiment(object):
         """
         key = participant.id
         networks_with_space = (
-            Network.query.filter_by(full=False).order_by(Network.id).all()
+            self.session.query(Network).filter_by(full=False).order_by(Network.id).all()
         )
         networks_participated_in = [
             node.network_id
-            for node in Node.query.with_entities(Node.network_id)
+            for node in self.session.query(Node.network_id)
             .filter_by(participant_id=participant.id)
             .all()
         ]
@@ -480,7 +683,11 @@ class Experiment(object):
                   single matching participant.
         """
         try:
-            return Participant.query.filter_by(assignment_id=assignment_id).one()
+            return (
+                self.session.query(Participant)
+                .filter_by(assignment_id=assignment_id)
+                .one()
+            )
         except (NoResultFound, MultipleResultsFound):
             return None
 
@@ -549,9 +756,88 @@ class Experiment(object):
         """
         return True
 
+    def on_recruiter_submission_complete(self, participant, event):
+        """Called after assignment submission has been processed by
+        the recruitment platform (may be Dallinger itself).
+
+        :param participant (Participant): the ``Participant`` who has
+        submitted an assignment via their recruiter
+        :param event: (dict): Info about the triggering event
+        """
+        # Check that the participant has completed task submission with
+        # their recruiter:
+        if participant.status != "submitted":
+            logger.warning(
+                "Called with unexpected participant status! "
+                "participant ID: {}, status: {}, recruiter: {}".format(
+                    participant.id, participant.status, participant.recruiter.nickname
+                )
+            )
+            return
+
+        config = get_config()
+        min_real_bonus = 0.01
+
+        # Usually, end_time will be set when the participant first exits
+        # the experiment via /worker_complete, but in case that hasn't
+        # happened, we set it here:
+        if participant.end_time is None:
+            participant.end_time = event["timestamp"]
+        participant.base_pay = config.get("base_payment")
+        participant.recruiter.approve_hit(participant.assignment_id)
+
+        # Data Check
+        if not self.data_check(participant=participant):
+            participant.status = "bad_data"
+            self.data_check_failed(participant=participant)
+            # NB: if MultiRecruiter is in use, this may not be the same recruiter as
+            # provided the participant we're replacing
+            self.recruiter.recruit(n=1)
+
+            # NOTE EARLY RETURN!!
+            return
+
+        # If they pass the data check, we might pay a bonus
+        bonus = self.bonus(participant=participant)
+        has_already_received_bonus = participant.bonus is not None
+        if has_already_received_bonus:
+            self.log(
+                "Bonus of {} will NOT be paid, since participant {} "
+                "has already received a bonus of {}".format(
+                    bonus, participant.id, participant.bonus
+                )
+            )
+        elif bonus < min_real_bonus:
+            self.log(
+                "Bonus of {} will NOT be paid to participant {} as it is "
+                "less than {}.".format(bonus, participant.id, min_real_bonus)
+            )
+        else:
+            self.log("Paying bonus of {} to {}".format(bonus, participant.id))
+            participant.recruiter.reward_bonus(
+                participant,
+                bonus,
+                self.bonus_reason(),
+            )
+            participant.bonus = bonus
+
+        # Attention Check
+        if self.attention_check(participant=participant):
+            self.log("Attention checks passed.")
+            participant.status = "approved"
+            self.submission_successful(participant=participant)
+            self.recruit()
+        else:
+            self.log("Attention checks failed.")
+            participant.status = "did_not_attend"
+            self.attention_check_failed(participant=participant)
+            # NB: if MultiRecruiter is in use, this may not be the same recruiter
+            # that provided the participant we're replacing:
+            self.recruiter.recruit(n=1)
+
     def participant_task_completed(self, participant):
-        """Called when an experiment task is finished and submitted, and prior
-        to data and attendance checks.
+        """Called when an experiment task is first finished, and prior
+        to recruiter submission, data and attendance checks.
 
         Assigns the qualifications to the Participant, via their recruiter.
         These will include one Qualification for the experiment
@@ -569,6 +855,11 @@ class Experiment(object):
             return
 
         if participant.status == "overrecruited":
+            logger.info(
+                "Skipping qualification assignment for overrecruited participant {}.".format(
+                    participant.id
+                )
+            )
             return
 
         quals = self.calculate_qualifications(participant)
@@ -591,7 +882,6 @@ class Experiment(object):
         experiment (participants who fail to finish successfully are
         automatically replaced). By default it recruits 1 participant at a time
         until all networks are full.
-
         """
         if not self.networks(full=False):
             self.log("All networks full: closing recruitment", "-----")
@@ -605,7 +895,7 @@ class Experiment(object):
 
     def log_summary(self):
         """Log a summary of all the participants' status codes."""
-        participants = Participant.query.with_entities(Participant.status).all()
+        participants = self.session.query(Participant.status).all()
         counts = Counter([p.status for p in participants])
         sorted_counts = sorted(counts.items(), key=itemgetter(0))
         self.log("Status summary: {}".format(str(sorted_counts)))
@@ -615,7 +905,6 @@ class Experiment(object):
         """Add all the objects to the session and commit them.
 
         This only needs to be done for networks and participants.
-
         """
         if len(objects) > 0:
             self.session.add_all(objects)
@@ -663,9 +952,11 @@ class Experiment(object):
 
     def fail_participant(self, participant):
         """Fail all the nodes of a participant."""
-        participant_nodes = Node.query.filter_by(
-            participant_id=participant.id, failed=False
-        ).all()
+        participant_nodes = (
+            self.session.query(Node)
+            .filter_by(participant_id=participant.id, failed=False)
+            .all()
+        )
 
         for node in participant_nodes:
             node.fail()
@@ -906,7 +1197,7 @@ class Experiment(object):
     def monitoring_panels(self, **kw):
         """Provides monitoring dashboard sidebar panels.
 
-        :param \**kw: arguments passed in from the request
+        :param \\**kw: arguments passed in from the request
         :returns: An ``OrderedDict()`` mapping panel titles to HTML strings
                   to render in the dashboard sidebar.
         """  # noqa
@@ -919,22 +1210,19 @@ class Experiment(object):
     def monitoring_statistics(self, **kw):
         """The default data used for the monitoring panels
 
-        :param \**kw: arguments passed in from the request
+        :param \\**kw: arguments passed in from the request
         :returns: An ``OrderedDict()`` mapping panel titles to data structures
                   describing the experiment state.
         """  # noqa
-        participants = Participant.query
-        nodes = Node.query
-        infos = Info.query
+        participants = self.session.query(Participant)
+        nodes = self.session.query(Node)
+        infos = self.session.query(Info)
 
+        unique_statuses = set(participant.status for participant in participants.all())
         stats = OrderedDict()
-        stats["Participants"] = OrderedDict(
-            (
-                ("working", participants.filter_by(status="working").count()),
-                ("abandoned", participants.filter_by(status="abandoned").count()),
-                ("returned", participants.filter_by(status="returned").count()),
-                ("approved", participants.filter_by(status="approved").count()),
-            )
+        stats["Participants"] = dict(
+            (status, participants.filter_by(status=status).count())
+            for status in sorted(unique_statuses)
         )
 
         # Count up our networks by role
@@ -977,118 +1265,143 @@ class Experiment(object):
 
         return stats
 
-    @staticmethod
-    def count_active_complete_failed(entities):
-        n_pending = 0
-        n_completed = 0
-        n_failed = 0
-        for entity in entities:
-            if entity.failed is False:
-                is_pending = (
-                    issubclass(entity.__class__, Info) and entity.contents is None
-                )
-                if is_pending:
-                    n_pending += 1
-                else:
-                    n_completed += 1
-            else:
-                n_failed += 1
-        return n_pending, n_completed, n_failed
+    def network_structure(
+        self,
+        network_roles=None,
+        network_ids=None,
+        collapsed=False,
+        transformations=False,
+    ):
+        networks = self.summarize_table("network", network_roles, network_ids)
 
-    def network_structure(self, **kw):
-        network_ids = {i[0] for i in self.session.query(distinct(Network.id)).all()}
-        if "network_roles" in kw:
-            network_ids = {
-                i[0]
-                for i in self.session.query(distinct(Network.id)).filter(
-                    Network.role.in_(kw["network_roles"])
-                )
-            }
-        if "network_ids" in kw:
-            network_ids = network_ids.intersection(int(v) for v in kw["network_ids"])
+        nodes = self.summarize_table(
+            "node",
+            network_roles,
+            network_ids,
+            cls_filter=(lambda cls: issubclass(cls, Source)) if collapsed else None,
+        )
 
-        network_ids = sorted(network_ids)  # default sorting is by id
-
-        jnetworks = []
-        for network in Network.query.filter(Network.id.in_(network_ids)).all():
-            (
-                n_pending_infos,
-                n_completed_infos,
-                n_failed_infos,
-            ) = self.count_active_complete_failed(network.all_infos)
-            _, n_completed_nodes, n_failed_nodes = self.count_active_complete_failed(
-                network.all_nodes
-            )
-
-            jnetworks.append(
-                {
-                    **network.__json__(),
-                    "n_pending_infos": n_pending_infos,
-                    "n_completed_infos": n_completed_infos,
-                    "n_failed_infos": n_failed_infos,
-                    "n_completed_nodes": n_completed_nodes,
-                    "n_failed_nodes": n_failed_nodes,
-                }
-            )
-
-        if "collapsed" in kw:
-            # Collapsed view shows Source nodes only
-            jnodes = [
-                n.__json__()
-                for n in Source.query.filter(Node.network_id.in_(network_ids))
-                .order_by(Source.id)
-                .all()
-            ]
-            jinfos = jparticipants = jtransformations = jvectors = []
+        if collapsed:
+            vectors = []
+            infos = []
+            participants = []
+            trans = []
         else:
-            jnodes = [
-                n.__json__()
-                for n in Node.query.filter(Node.network_id.in_(network_ids))
-                .order_by(Node.id)
-                .all()
-            ]
-            jinfos = [
-                n.__json__()
-                for n in Info.query.filter(Info.network_id.in_(network_ids))
-                .order_by(Info.id)
-                .all()
-            ]
-            # We don't filter participants because they aren't directly connected to specific networks
-            jparticipants = [
-                n.__json__() for n in Participant.query.order_by(Participant.id).all()
-            ]
-            jtransformations = []
-            if kw.get("transformations"):
-                jtransformations = [
-                    n.__json__()
-                    for n in Transformation.query.filter(
-                        Transformation.network_id.in_(network_ids)
-                    )
-                    .order_by(Transformation.id)
-                    .all()
-                ]
+            vectors = self.summarize_table("vector", network_roles, network_ids)
+            infos = self.summarize_table("info", network_roles, network_ids)
+            participants = self.summarize_table("participant")
 
-            jvectors = [
-                {
-                    "origin_id": v.origin_id,
-                    "destination_id": v.destination_id,
-                    "id": v.id,
-                    "failed": v.failed,
-                }
-                for v in Vector.query.filter(Vector.network_id.in_(network_ids))
-                .order_by(Vector.id)
-                .all()
-            ]
+            if transformations:
+                trans = self.summarize_table(
+                    "transformation", network_roles, network_ids
+                )
+            else:
+                trans = []
 
         return {
-            "networks": jnetworks,
-            "nodes": jnodes,
-            "vectors": jvectors,
-            "infos": jinfos,
-            "participants": jparticipants,
-            "trans": jtransformations,
+            "networks": networks,
+            "nodes": nodes,
+            "vectors": vectors,
+            "infos": infos,
+            "participants": participants,
+            "trans": trans,
         }
 
+    def summarize_table(
+        self,
+        table: Union[Table, str],
+        network_roles: Optional[List] = None,
+        network_ids: Optional[List] = None,
+        cls_filter: Optional[callable] = None,
+    ):
+        """
+        Summarizes a given database table.
+
+        :param table: Table to be summarized
+        :param network_roles: Optionally restrict output to objects from networks with these roles
+        :param network_ids: Optionally restrict output to objects from networks with these IDs
+        :param cls_filter: Optional lambda function that returns ``False`` for classes that should be excluded
+
+        Returns a list of JSON-style dictionaries produced by calling ``.__json__()`` on every object
+        retrieved from the table.
+        """
+        objects = self.pull_table(
+            table=table,
+            polymorphic_identity=None,
+            network_roles=network_roles,
+            network_ids=network_ids,
+            cls_filter=cls_filter,
+        )
+        return [obj.__json__() for obj in objects]
+
+    def pull_table(
+        self,
+        table: Union[Table, str],
+        polymorphic_identity: Optional[str] = None,
+        network_roles: Optional[List] = None,
+        network_ids: Optional[List] = None,
+        cls_filter: Optional[callable] = None,
+    ):
+        """
+        Downloads every object in the specified table.
+        For efficiency, the SQL queries are batched by the values of the polymorphic identity column ``type``
+        if it is present.
+
+        :param table: Table to be summarized
+        :param polymorphic_identity: Optionally restrict output to a given polymorphic identity (i.e. ``type`` value)
+        :param network_roles: Optionally restrict output to objects from networks with these roles
+        :param network_ids: Optionally restrict output to objects from networks with these IDs
+        :param cls_filter: Optional lambda function that returns ``False`` for classes that should be excluded
+
+        Returns a list of database-mapped objects.
+        """
+        if isinstance(table, str):
+            table = Base.metadata.tables[table]
+
+        if polymorphic_identity is None and "type" in table.columns:
+            observed_types = [
+                r.type for r in self.session.query(table.columns.type).distinct().all()
+            ]
+            obj_by_type = [
+                self.pull_table(
+                    table,
+                    polymorphic_identity=_type,
+                    network_roles=network_roles,
+                    network_ids=network_ids,
+                    cls_filter=cls_filter,
+                )
+                for _type in observed_types
+            ]
+            return [obj for sublist in obj_by_type for obj in sublist]
+
+        if polymorphic_identity is None:
+            cls = get_mapped_class(table)
+        else:
+            assert "type" in table.columns
+            cls = get_polymorphic_mapping(table)[polymorphic_identity]
+
+        if cls_filter is not None and not cls_filter(cls):
+            return
+
+        query = cls.query
+
+        if polymorphic_identity is not None:
+            query = query.filter(cls.type == polymorphic_identity)
+
+        if network_roles is not None:
+            query = query.filter(Network.role.in_(network_roles))
+
+        if network_ids is not None:
+            query = query.filter(Network.id.in_(network_ids))
+
+        if network_roles is not None or network_ids is not None:
+            if "network_id" in table.columns:
+                query = query.join(Network, cls.network_id == Network.id)
+
+        primary_keys = [c.name for c in table.primary_key.columns]
+
+        return query.order_by(*primary_keys).options(undefer("*")).all()
+
     def node_visualization_options(self):
         """Provides custom vis.js configuration options for the
         Network Monitoring Dashboard.
@@ -1116,15 +1429,18 @@ class Experiment(object):
                 return obj.visualization_html
         return ""
 
-    def table_data(self, **kw):
+    def table_data(
+        self, table: str = "participant", polymorphic_identity: Optional[str] = None
+    ):
         """Generates DataTablesJS data and configuration for the experiment. The data
         is compiled from the models' ``__json__`` methods, and can be customized by either
         overriding this method or using the ``json_data`` method on the model to return
         additional serializable data.
 
-        :param \**kw: arguments passed in from the request. The ``model_type`` parameter
-                      takes a ``str`` or iterable and queries all objects of those types,
-                      ordered by ``id``.
+        :param table: table to query
+
+        :param polymorphic_identity: optional polymorphic identity (corresponds to the ``type`` column)
+
         :returns: Returns a ``dict`` with DataTablesJS data and configuration, filters using
                   arbitrary keyword arguments. Should contain ``data`` and ``columns`` keys
                   at least, with ``columns`` containing data for all fields on all returned
@@ -1133,23 +1449,25 @@ class Experiment(object):
         rows = []
         found_columns = set()
         columns = []
-        model_types = kw.get("model_type", ["Participant"])
-        if hasattr(model_types, "strip"):
-            model_types = [model_types]
 
-        for model_type in model_types:
-            model = getattr(models, model_type, None)
-            for obj in model.query.order_by(model.id).all():
-                data = obj.__json__()
-                # Add participant worker_id to data, we normally leave it out of
-                # JSON renderings
-                if model_type == "Participant":
-                    data["worker_id"] = obj.worker_id
-                rows.append(data)
-                for key in data:
-                    if key not in found_columns:
-                        columns.append({"name": key, "data": key})
-                        found_columns.add(key)
+        table = Base.metadata.tables[table]
+
+        if polymorphic_identity == "None":
+            polymorphic_identity = None
+
+        objects = self.pull_table(table, polymorphic_identity=polymorphic_identity)
+
+        for obj in objects:
+            data = obj.__json__()
+            # Add participant worker_id to data, we normally leave it out of
+            # JSON renderings
+            if table.name == "participant":
+                data["worker_id"] = obj.worker_id
+            rows.append(data)
+            for key in data:
+                if key not in found_columns:
+                    columns.append({"name": key, "data": key})
+                    found_columns.add(key)
 
         # Make sure every row has an entry for every column
         for col in found_columns:
@@ -1567,7 +1885,7 @@ def scheduled_task(trigger, **kwargs):
 
     :param trigger: an ``apscheduler`` trigger type. One of "interval", "cron",
                     or "date"
-    :param \**kwargs: other arguments for `apscheduler.schedulers.base.BaseSchedule.scheduled_job`
+    :param \\**kwargs: other arguments for `apscheduler.schedulers.base.BaseSchedule.scheduled_job`
                       generally used for trigger arguments to determine
                       the run interval.
 
