@@ -2,10 +2,13 @@ import json
 import logging
 from copy import deepcopy
 from datetime import datetime, timedelta
+from typing import Optional, Union
 from xml.sax.saxutils import escape
 
+import bs4
 import six
 import timeago
+from ansi2html import Ansi2HTMLConverter
 from flask import (
     Blueprint,
     abort,
@@ -20,6 +23,7 @@ from flask.wrappers import Response
 from flask_login import UserMixin, current_user, login_required, login_user, logout_user
 from flask_login.utils import login_url as make_login_url
 from flask_wtf import FlaskForm
+from pygtail import Pygtail
 from six.moves.urllib.parse import urlencode
 from tzlocal import get_localzone
 from wtforms import BooleanField, HiddenField, PasswordField, StringField, SubmitField
@@ -28,8 +32,12 @@ from wtforms.validators import DataRequired, ValidationError
 import dallinger.db
 from dallinger import recruiters
 from dallinger.config import get_config
+from dallinger.db import get_all_mapped_classes
 from dallinger.heroku.tools import HerokuApp
-from dallinger.utils import deferred_route_decorator
+from dallinger.utils import (
+    JSON_LOGFILE,
+    deferred_route_decorator,
+)
 
 from .utils import date_handler, error_response, success_response
 
@@ -144,7 +152,13 @@ class DashboardTabs(object):
         :raises ValueError: When ``before_route`` is not found in registered tabs
 
         """
-        before_check = frozenset((before_route, "dashboard." + before_route))
+        before_check = frozenset(
+            (
+                before_route,
+                "dashboard." + before_route,
+                "dashboard.dashboard_" + before_route,
+            )
+        )
         for i, cur_tab in enumerate(self.tabs):
             if cur_tab.route_name in before_check:
                 position = i
@@ -178,7 +192,13 @@ class DashboardTabs(object):
         :raises ValueError: When ``after_route`` is not found in registered tabs
 
         """
-        after_check = frozenset((after_route, "dashboard." + after_route))
+        after_check = frozenset(
+            (
+                after_route,
+                "dashboard." + after_route,
+                "dashboard.dashboard_" + after_route,
+            )
+        )
         for i, cur_tab in enumerate(self.tabs):
             if cur_tab.route_name in after_check:
                 position = i + 1
@@ -201,34 +221,31 @@ class DashboardTabs(object):
         return iter(self.tabs)
 
 
-BROWSEABLE_MODELS = [
-    "Info",
-    "Network",
-    "Node",
-    "Participant",
-    "Question",
-    "Transformation",
-    "Transmission",
-    "Notification",
-]
-
-
 def database_children():
-    for model_type in BROWSEABLE_MODELS:
+    mapped_classes = list(get_all_mapped_classes().items())
+    mapped_classes.sort(key=lambda x: x[0])
+    for cls_name, cls_info in mapped_classes:
         yield DashboardTab(
-            model_type + "s", "dashboard.database", None, {"model_type": model_type}
+            cls_name,
+            "dashboard.dashboard_database",
+            None,
+            {
+                "table": cls_info["table"],
+                "polymorphic_identity": cls_info["polymorphic_identity"],
+            },
         )
 
 
 dashboard_tabs = DashboardTabs(
     [
-        DashboardTab("Home", "dashboard.index"),
-        DashboardTab("Heroku", "dashboard.heroku"),
-        DashboardTab("MTurk", "dashboard.mturk"),
-        DashboardTab("Monitoring", "dashboard.monitoring"),
-        DashboardTab("Lifecycle", "dashboard.lifecycle"),
-        DashboardTab("Database", "dashboard.database", database_children),
-        DashboardTab("Development", "dashboard.develop"),
+        DashboardTab("Config", "dashboard.dashboard_index"),
+        DashboardTab("Heroku", "dashboard.dashboard_heroku"),
+        DashboardTab("MTurk", "dashboard.dashboard_mturk"),
+        DashboardTab("Monitoring", "dashboard.dashboard_monitoring"),
+        DashboardTab("Lifecycle", "dashboard.dashboard_lifecycle"),
+        DashboardTab("Database", "dashboard.dashboard_database", database_children),
+        DashboardTab("Logger", "dashboard.dashboard_logger"),
+        DashboardTab("Development", "dashboard.dashboard_develop"),
     ]
 )
 
@@ -299,7 +316,9 @@ def is_safe_url(url):
 def login():
     next_url = request.form.get("next", request.args.get("next"))
     next_url = (
-        next_url if next_url and is_safe_url(next_url) else url_for("dashboard.index")
+        next_url
+        if next_url and is_safe_url(next_url)
+        else url_for("dashboard.dashboard_index")
     )
     if current_user.is_authenticated:
         return redirect(next_url)
@@ -323,23 +342,30 @@ def login():
 @dashboard.route("/logout")
 def logout():
     logout_user()
-    return redirect(url_for("dashboard.index"))
+    return redirect(url_for("dashboard.dashboard_index"))
 
 
 @dashboard.route("/")
 @dashboard.route("/index")
 @login_required
-def index():
+def dashboard_index():
     """Displays active experiment configuation"""
-    config = sorted(get_config().as_dict().items())
+    config = get_config()
+    config.load()
+    config_dict = config.as_dict()
+    config_list = sorted(config_dict.items())
     return render_template(
-        "dashboard_home.html", title="Dashboard Home", configuration=config
+        "dashboard_home.html",
+        title="Config",
+        configuration=config_list,
+        configuration_dictionary=config_dict,
+        changeable_params=config.changeable_params,
     )
 
 
 @dashboard.route("/heroku")
 @login_required
-def heroku():
+def dashboard_heroku():
     """Assemble links from Heroku add-on info, stored in config, plus some
     standard dashboard links.
     """
@@ -537,7 +563,7 @@ def mturk_data_source(config):
 
 @dashboard.route("/mturk")
 @login_required
-def mturk():
+def dashboard_mturk():
     config = get_config()
     try:
         data_source = mturk_data_source(config)
@@ -564,9 +590,20 @@ def mturk():
     return render_template("dashboard_mturk.html", title="MTurk Dashboard", data=data)
 
 
+@dashboard.route("/auto_recruit/<bool_val>", methods=["POST"])
+@login_required
+def auto_recruit(bool_val):
+    from dallinger.db import redis_conn
+
+    num_val = int(bool_val)
+    assert num_val in [0, 1]
+    redis_conn.set("auto_recruit", num_val)
+    return success_response()
+
+
 @dashboard.route("/monitoring")
 @login_required
-def monitoring():
+def dashboard_monitoring():
     from sqlalchemy import distinct, func
 
     from dallinger.experiment_server.experiment_server import Experiment, session
@@ -615,7 +652,7 @@ def init_db():
 
 @dashboard.route("/lifecycle")
 @login_required
-def lifecycle():
+def dashboard_lifecycle():
     config = get_config()
 
     try:
@@ -635,6 +672,301 @@ def lifecycle():
     )
 
 
+def clean_http_request(msg: str) -> str:
+    """Remove redundant prefix from HTTP request log messages.
+
+    Parameters
+    ----------
+    msg : str
+        The log message to clean
+
+    Returns
+    -------
+    str
+        The cleaned message with redundant prefix removed
+
+    Examples
+    --------
+    >>> msg = '127.0.0.1 - - [20/Mar/2024:10:00:00] "GET /dashboard HTTP/1.1" 200 -'
+    >>> clean_http_request(msg)
+    'GET /dashboard HTTP/1.1" 200 -'
+
+    >>> msg = 'Regular log message without HTTP request'
+    >>> clean_http_request(msg)
+    'Regular log message without HTTP request'
+    """
+    has_http_method = any(f" {method} " in msg for method in ("GET", "POST"))
+    if has_http_method and msg.endswith(("-", '"')):
+        return '"'.join(msg.split('"')[1:])
+    return msg
+
+
+def convert_ansi_to_html(msg: str) -> str:
+    """Convert ANSI terminal colors to HTML formatted text.
+
+    Parameters
+    ----------
+    msg : str
+        Message potentially containing ANSI color codes
+
+    Returns
+    -------
+    str
+        HTML-formatted message with ANSI codes converted to CSS
+
+    Examples
+    --------
+    >>> msg = '\033[31mError:\033[0m Database connection failed'
+    >>> convert_ansi_to_html(msg)
+    '<span style="color: red">Error:</span> Database connection failed'
+
+    >>> msg = 'Plain text without ANSI codes'
+    >>> convert_ansi_to_html(msg)
+    'Plain text without ANSI codes'
+    """
+    return Ansi2HTMLConverter().convert(msg)
+
+
+def ensure_html_structure(html: str) -> str:
+    """Ensure HTML has proper structure with head and body tags.
+
+    Parameters
+    ----------
+    html : str
+        HTML content to structure
+
+    Returns
+    -------
+    str
+        Properly structured HTML document
+
+    Examples
+    --------
+    >>> html = '<span style="color: red">Error</span>'
+    >>> ensure_html_structure(html)
+    '<html><head></head><body><span style="color: red">Error</span></body></html>'
+
+    >>> html = '<head><style>...</style></head><body>Content</body>'
+    >>> ensure_html_structure(html)
+    '<html><head><style>...</style></head><body>Content</body></html>'
+    """
+    soup = bs4.BeautifulSoup(html, "html.parser")
+    return f"<html>{soup.head or ''}{soup.body or ''}</html>"
+
+
+def clean_line_info(line_info: dict, log_line_number: Optional[int] = None) -> dict:
+    """Clean and format a log message dictionary for dashboard display.
+
+    Parameters
+    ----------
+    line_info : dict
+        Dictionary containing log message data with at least a 'message' key
+    log_line_number : int, optional
+        Line number in the log file
+
+    Returns
+    -------
+    dict
+        Cleaned dictionary with HTML-formatted message
+
+    Examples
+    --------
+    >>> line_info = {
+    ...     'message': '\033[31mError:\033[0m GET /dashboard failed',
+    ...     'level': 'ERROR'
+    ... }
+    >>> cleaned = clean_line_info(line_info, log_line_number=42)
+    >>> cleaned
+    {
+        'message': '<html><head>...</head><body><span style="color: red">Error:</span> GET /dashboard failed</body></html>',
+        'level': 'ERROR',
+        'log_line_number': 42
+    }
+    """
+    msg = line_info["message"]
+
+    # Clean and format the message
+    msg = clean_http_request(msg)
+    msg = convert_ansi_to_html(msg)
+    msg = ensure_html_structure(msg)
+
+    # Finalize the dictionary
+    line_info["message"] = msg
+    if log_line_number is not None:
+        line_info["log_line_number"] = log_line_number
+
+    return line_info
+
+
+def log_read_lines(line_start: int, line_end: int) -> tuple[list[dict], bool, int]:
+    """
+    Read the log file and return the lines in the specified range.
+
+    :param line_start: The line number to start reading from
+    :type line_start: int
+
+    :param line_end: The line number to stop reading at
+    :type line_end: int
+
+    :return: A tuple containing the lines, a boolean indicating if the end of the file was reached, and the last line number
+
+    :Note: The line numbers are 1-based
+
+    """
+    lines = []
+    line_range = range(line_start, line_end)
+    early_stop = False
+    with open(JSON_LOGFILE) as f:
+        for i, line in enumerate(f):
+            number = i + 1
+            if number in line_range:
+                line_info = clean_line_info(json.loads(line), number)
+                lines.append(line_info)
+            if number > line_end:
+                early_stop = True
+                break
+    return lines, early_stop, i
+
+
+def find_log_line_number(substring) -> Union[int, None]:
+    """
+    Find the line number in the log file that contains a substring.
+
+    :param substring: The substring to search for
+    :type substring: str
+
+    :return: The line number (1-based) or None if the substring was not found
+    """
+    with open(JSON_LOGFILE) as f:
+        for number, line in enumerate(f):
+            if substring in line:
+                return number + 1
+    return None
+
+
+def log_search_substring(substring: str):
+    """
+    Search the log file for a substring and return the matching lines.
+
+    :param substring: The substring to search for
+    :type substring: str
+
+    :return: A generator that yields the matching lines in the right format (i.e. f"data:{json.dumps(obj)}\n\n")
+
+    :Note: The line numbers are 1-based
+    :Note: The generator will yield a 'stop' message when the end of the file is reached
+    """
+    with open(JSON_LOGFILE) as f:
+        for number, line in enumerate(f):
+            if substring in line:
+                line_info = clean_line_info(json.loads(line), number)
+                yield f"data:{json.dumps(line_info)}\n\n"
+    yield f"data:{json.dumps({'stop': True})}\n\n"
+
+
+@dashboard.route("/logs/live", methods=["GET"])
+@login_required
+def logs_live():
+    """Stream log file updates to the dashboard using Server-Sent Events (SSE).
+
+    Creates a streaming response that pushes new log entries to connected clients
+    in real-time. Each log entry is:
+    - Read from the log file using Pygtail
+    - Parsed from JSON
+    - Cleaned and formatted for display
+    - Sent as an SSE message
+
+    Returns
+    -------
+    Response
+        A Flask response object with mimetype 'text/event-stream' containing
+        the event stream of log updates. Each event contains:
+        - message : str
+            The formatted log message with ANSI codes converted to HTML
+        - original_line : str
+            The raw log line before processing
+        - log_line_number : int, optional
+            The line number in the log file
+
+    Notes
+    -----
+
+    The client should connect to this endpoint using an EventSource object
+    to receive the updates.
+    """
+
+    def generate():
+        for line in Pygtail(JSON_LOGFILE, read_from_end=True, every_n=1):
+            try:
+                line_info = clean_line_info(json.loads(line))
+                line_info["original_line"] = line
+                yield f"data:{json.dumps(line_info)}\n\n"
+            except json.decoder.JSONDecodeError:
+                yield f"data:{json.dumps({'message': line})}\n\n"
+
+    return Response(generate(), mimetype="text/event-stream")
+
+
+@dashboard.route("/logs/range", methods=["GET"])
+@login_required
+def logs_range():
+    """
+    Return a range of log lines from the log file.
+    """
+    params = request.args
+    start, end = params.get("start", None), params.get("end", None)
+    if start is None or end is None:
+        return json_error_response("Both 'start' and 'end' parameters are required.")
+    start, end = int(start), int(end)
+    if start < 1:
+        return json_error_response("'start' must be > 0.")
+
+    if start > end:
+        return json_error_response("'start' must be <= 'end'.")
+
+    lines, early_stop, last_line = log_read_lines(start, end + 1)
+    if not early_stop and start > last_line:
+        return json_error_response(
+            f"'start' must be less than the current line number {last_line}."
+        )
+
+    return lines
+
+
+def json_error_response(message, status_code=400):
+    return {"msg": message}, status_code
+
+
+@dashboard.route("/logs/find_lines", methods=["GET"])
+@login_required
+def logs_find_lines():
+    """
+    Find lines in the log file that contain a substring (GET parameter 'query').
+    """
+    params = request.args
+    query = params.get("query", None)
+    if query is None:
+        return json_error_response("No query provided.")
+    return Response(log_search_substring(query), mimetype="text/event-stream")
+
+
+@dashboard.route("/logs/find_line_number", methods=["POST"])
+@login_required
+def logs_find_line_number():
+    """
+    Find the line number in the log file that contains a substring (POST parameter 'query').
+    """
+    params = request.json
+    query = params.get("query", None)
+    if query is None:
+        return json_error_response("No query provided.")
+
+    line_number = find_log_line_number(query)
+    if line_number is not None:
+        return {"line_number": line_number}
+    return json_error_response("No line found.", 404)
+
+
 TABLE_DEFAULTS = {
     "dom": "frtilBpP",
     "ordering": True,
@@ -708,16 +1040,31 @@ def prep_datatables_options(table_data):
 
 @dashboard.route("/database")
 @login_required
-def database():
+def dashboard_database():
+    from dallinger.db import get_polymorphic_mapping
     from dallinger.experiment_server.experiment_server import Experiment, session
 
-    title = "Database View"
     exp = Experiment(session)
-    model_type = request.args.get("model_type")
-    if model_type:
-        title = "Database View: {}s".format(model_type)
+
+    table = request.args.get("table", None)
+    polymorphic_identity = request.args.get("polymorphic_identity", None)
+
+    if polymorphic_identity == "None":
+        polymorphic_identity = None
+
+    if table is None and polymorphic_identity is None:
+        table = "participant"
+
+    if polymorphic_identity is not None:
+        assert table is not None
+        cls = get_polymorphic_mapping(table)[polymorphic_identity]
+        label = cls.__name__
+    else:
+        label = table.capitalize()
+
+    title = "Database View: {}".format(label)
     datatables_options = prep_datatables_options(
-        exp.table_data(**request.args.to_dict(flat=False))
+        exp.table_data(**request.args.to_dict())
     )
     columns = [
         c.get("name") or c["data"]
@@ -763,12 +1110,24 @@ def database():
     )
 
 
+@dashboard.route("/logger")
+@login_required
+def dashboard_logger():
+    """
+    Streams the tail of the log file to the dashboard or shows selected lines
+    """
+    return render_template("dashboard_logger.html")
+
+
 @dashboard.route("/develop", methods=["GET", "POST"])
 @login_required
-def develop():
+def dashboard_develop():
     """Dashboard for working with ``dallinger develop`` Flask server."""
-
-    return render_template("dashboard_develop.html")
+    return render_template(
+        "dashboard_develop.html",
+        mode=get_config().get("mode"),
+        recruiter=recruiters.from_config(get_config()).nickname,
+    )
 
 
 @dashboard.route("/database/action/<route_name>", methods=["POST"])
@@ -836,4 +1195,10 @@ def dashboard_tab(title, **kwargs):
         "tab": full_tab,
     }
 
-    return deferred_route_decorator(route, registered_routes)
+    def route_name_from_func_name(func_name: str) -> str:
+        key = "dashboard_"
+        if func_name.startswith(key):
+            return func_name[len(key) :]
+        return func_name
+
+    return deferred_route_decorator(route, registered_routes, route_name_from_func_name)
