@@ -1,5 +1,6 @@
-""" This module provides the backend Flask server that serves an experiment. """
+"""This module provides the backend Flask server that serves an experiment."""
 
+import logging
 import os
 import re
 from datetime import datetime
@@ -27,7 +28,13 @@ from sqlalchemy.sql.expression import true
 from dallinger import db, experiment, models, recruiters
 from dallinger.config import get_config
 from dallinger.notifications import MessengerError, admin_notifier
-from dallinger.utils import generate_random_id
+from dallinger.recruiters import ProlificRecruiter
+from dallinger.utils import (
+    attach_json_logger,
+    generate_random_id,
+    get_from_config,
+    setup_warning_hooks,
+)
 
 from . import dashboard
 from .replay import ReplayBackend
@@ -42,17 +49,24 @@ from .utils import (
 )
 from .worker_events import worker_function
 
+setup_warning_hooks()
+
 # Initialize the Dallinger database.
 session = db.session
 redis_conn = db.redis_conn
 
 # Connect to the Redis queue for notifications.
-q = Queue(connection=redis_conn)
+q = Queue("default", connection=redis_conn)
 WAITING_ROOM_CHANNEL = "quorum"
 
 app = Flask("Experiment_Server")
 
 
+@app.before_request
+def _load_config():
+    _config()
+
+
 @app.before_request
 def check_for_protected_routes():
     if current_user.is_authenticated:
@@ -70,7 +84,6 @@ def check_for_protected_routes():
         )
 
 
-@app.before_first_request
 def _config():
     app.secret_key = app.config["SECRET_KEY"] = os.environ.get("FLASK_SECRET_KEY")
     config = get_config()
@@ -86,10 +99,19 @@ def _config():
 
 
 def Experiment(args):
+    _config()
     klass = experiment.load()
     return klass(args)
 
 
+log = logging.getLogger()
+DATE_FORMAT = "%Y-%m-%d %H:%M:%S"
+logging.basicConfig(
+    level=logging.INFO,
+    datefmt=DATE_FORMAT,
+)
+attach_json_logger(log)
+
 # Load the experiment's extra routes, if any.
 try:
     from dallinger_experiment.experiment import extra_routes
@@ -98,6 +120,12 @@ except ImportError:
 else:
     app.register_blueprint(extra_routes)
 
+# Enable the websocket route. This needs to be imported after app is defined to
+# avoid an import loop.
+# XXX: It would be nice to not do this at import time and
+# avoid this circularity, but doing so seems to cause tests in `test_deployment`
+# to deadlock.
+from dallinger.experiment_server import sockets  # noqa: E402
 
 # Skipping coverage testing on this for now because it only runs at import time
 # and cannot be exercised within tests
@@ -106,6 +134,20 @@ try:
 except ImportError:
     exp_klass = None  # pragma: no cover
 
+
+@app.before_request
+def before_request():
+    if exp_klass is not None:
+        return exp_klass.before_request()
+
+
+@app.after_request
+def after_request(response):
+    if exp_klass is not None:
+        return exp_klass.after_request(request, response)
+    return response
+
+
 if exp_klass is not None:  # pragma: no cover
     bp = exp_klass.experiment_routes
     routes = experiment.EXPERIMENT_ROUTE_REGISTRATIONS
@@ -127,10 +169,11 @@ if exp_klass is not None:  # pragma: no cover
         if route_func is not None:
             # All dashboard routes require login
             route_func = login_required(route_func)
-            route_name = route["func_name"]
+            route_name = route["name"]
+            endpoint = route["func_name"]
             dashboard.dashboard.add_url_rule(
                 "/" + route_name,
-                endpoint=route_name,
+                endpoint=endpoint,
                 view_func=route_func,
                 **dict(route["kwargs"]),
             )
@@ -140,24 +183,25 @@ if exp_klass is not None:  # pragma: no cover
                 tabs.insert_tab_before_route(full_tab, route["before_route"])
             elif route.get("before_route"):
                 tabs.insert_before_route(
-                    route["title"], route_name, route["before_route"]
+                    route["title"], endpoint, route["before_route"]
                 )
             elif route.get("after_route") and full_tab:
                 tabs.insert_tab_after_route(full_tab, route["after_route"])
             elif route.get("after_route"):
-                tabs.insert_after_route(
-                    route["title"], route_name, route["after_route"]
-                )
+                tabs.insert_after_route(route["title"], endpoint, route["after_route"])
             elif full_tab:
                 tabs.insert(full_tab)
             else:
-                tabs.insert(route["title"], route_name)
+                tabs.insert(route["title"], endpoint)
 
     # This hides dashboard tabs from view, but doesn't prevent the routes from
     # being registered
     hidden_dashboards = getattr(exp_klass, "hidden_dashboards", ())
     for route_name in hidden_dashboards:
         dashboard.dashboard_tabs.remove(route_name)
+    dashboard.dashboard_tabs = exp_klass.organize_dashboard_tabs(
+        dashboard.dashboard_tabs
+    )
 
 
 # Ideally, we'd only load recruiter routes if the recruiter is active, but
@@ -175,6 +219,8 @@ login.user_loader(dashboard.load_user)
 login.unauthorized_handler(dashboard.unauthorized)
 app.config["dashboard_tabs"] = dashboard.dashboard_tabs
 
+app.jinja_env.globals.update(get_from_config=get_from_config)
+
 """Basic routes."""
 
 
@@ -184,7 +230,7 @@ def index():
     html = (
         "<html><head></head><body><h1>Dallinger Experiment in progress</h1>"
         "<p><a href={}>Dashboard</a></p></body></html>".format(
-            url_for("dashboard.index")
+            url_for("dashboard.dashboard_index")
         )
     )
 
@@ -215,10 +261,24 @@ def handle_exp_error(exception):
 
 
 @app.teardown_request
-def shutdown_session(_=None):
-    """Rollback and close session at end of a request."""
+def shutdown_session(exception=None):
+    """Flask teardown handler to manage SQLAlchemy session lifecycle.
+
+    This function is called automatically by Flask after every request, whether
+    the request completed successfully or with an error.
+
+    - If an exception occurred during request processing, any uncommitted
+      changes in the current database session are rolled back to prevent partial
+      or inconsistent writes.
+    - The session is always removed at the end of the request to ensure that
+      database connections are closed and that no session state leaks
+      between requests.
+
+    This pattern is recommended when using SQLAlchemy's scoped_session.
+    """
+    if exception is not None:
+        session.rollback()
     session.remove()
-    db.logger.debug("Closing Dallinger DB session at flask request end")
 
 
 @app.context_processor
@@ -234,7 +294,7 @@ def render_error():
     participant_id = request.form.get("participant_id")
     participant = None
     if participant_id:
-        participant = models.Participant.query.get(participant_id)
+        participant = session.query(models.Participant).get(participant_id)
     return error_page(participant=participant, request_data=request_data)
 
 
@@ -375,14 +435,6 @@ def launch():
             status=500,
             simple=True,
         )
-    try:
-        exp.log("Launching experiment...", "-----")
-    except IOError as ex:
-        return error_response(
-            error_text="IOError writing to experiment log: {}".format(str(ex)),
-            status=500,
-            simple=True,
-        )
 
     try:
         exp.on_launch()
@@ -394,6 +446,10 @@ def launch():
             simple=True,
         )
 
+    recruitment_details = None
+    config = _config()
+    loglevel = config.get("loglevel", 0)
+    recruiters.set_log_level(loglevel)
     try:
         recruitment_details = exp.recruiter.open_recruitment(
             n=exp.initial_recruitment_size
@@ -418,7 +474,7 @@ def launch():
                 simple=True,
             )
 
-    if _config().get("replay", False):
+    if config.get("replay", False):
         try:
             task = ReplayBackend(exp)
             gevent.spawn(task)
@@ -434,9 +490,11 @@ def launch():
     # redis communication channel:
     if exp.channel is not None:
         try:
-            from dallinger.experiment_server.sockets import chat_backend
-
-            chat_backend.subscribe(exp, exp.channel)
+            sockets.chat_backend.subscribe(exp, exp.channel)
+            # Additionally subscribe the experiment to the Dallinger Control
+            # channel for messages about websocket
+            # connect/disconnect/subscribe/unsubscribe events
+            sockets.chat_backend.subscribe(exp, sockets.CONTROL_CHANNEL)
         except Exception:
             return error_response(
                 error_text="Failed to subscribe to chat for channel on launch "
@@ -446,15 +504,17 @@ def launch():
                 simple=True,
             )
 
-    message = "\n".join(
-        (
-            "Initial recruitment list:\n{}".format(
-                "\n".join(recruitment_details["items"])
-            ),
-            "Additional details:\n{}".format(recruitment_details["message"]),
+    if recruitment_details is not None:
+        message = "\n".join(
+            (
+                "Initial recruitment list:\n{}".format(
+                    "\n".join(recruitment_details["items"])
+                ),
+                "Additional details:\n{}".format(recruitment_details["message"]),
+            )
         )
-    )
-
+    else:
+        message = "Recruitment hasn't been started yet. Please, initialize recruitment manually!"
     return success_response(recruitment_msg=message)
 
 
@@ -468,12 +528,23 @@ def prepare_advertisement():
     if not browser.is_supported(request.user_agent.string):
         raise ExperimentError("browser_type_not_allowed")
 
+    recruiter = recruiters.from_config(config)
+    recruiter_name = request.args.get("recruiter")
+    if not recruiter_name:
+        recruiter_name = recruiter.nickname
+
     entry_information = request.args.to_dict()
 
     if entry_information.get("generate_tokens", None) in ("1", "true", "yes"):
         redirect_params = entry_information.copy()
         del redirect_params["generate_tokens"]
-        for entry_param in ("hitId", "assignmentId", "workerId"):
+
+        if isinstance(recruiter, ProlificRecruiter):
+            entry_params = ("PROLIFIC_PID", "STUDY_ID", "SESSION_ID")
+        else:
+            entry_params = ("hitId", "assignmentId", "workerId")
+
+        for entry_param in entry_params:
             if not redirect_params.get(entry_param):
                 redirect_params[entry_param] = generate_random_id()
         return True, {"redirect": redirect(url_for("advertisement", **redirect_params))}
@@ -492,20 +563,15 @@ def prepare_advertisement():
     if worker_id is not None:
         # Check if this workerId has completed the task before
         already_participated = (
-            models.Participant.query.filter(
-                models.Participant.worker_id == worker_id
-            ).first()
+            session.query(models.Participant)
+            .filter(models.Participant.worker_id == worker_id)
+            .first()
             is not None
         )
 
         if already_participated:
             raise ExperimentError("already_did_exp_hit")
 
-    recruiter_name = request.args.get("recruiter")
-    if not recruiter_name:
-        recruiter = recruiters.from_config(config)
-        recruiter_name = recruiter.nickname
-
     kwargs = {
         "recruiter": recruiter_name,
         "hitid": hit_id,
@@ -547,7 +613,7 @@ def advertisement():
 
 @app.route("/recruiter-exit", methods=["GET"])
 @nocache
-def recriter_exit():
+def recruiter_exit():
     """Display an exit page defined by the Participant's Recruiter.
     The Recruiter may in turn delegate to the Experiment for additional
     values to display.
@@ -558,7 +624,8 @@ def recriter_exit():
             error_type="/recruiter-exit GET: param participant_id is required",
             status=400,
         )
-    participant = models.Participant.query.get(participant_id)
+    exp = Experiment(session)
+    participant = session.query(models.Participant).get(participant_id)
     if participant is None:
         return error_response(
             error_type="/recruiter-exit GET: no participant found for ID {}".format(
@@ -570,7 +637,6 @@ def recriter_exit():
     # Get the recruiter from the participant rather than config, to support
     # MultiRecruiter experiments
     recruiter = recruiters.by_name(participant.recruiter_id)
-    exp = Experiment(session)
 
     return recruiter.exit_response(experiment=exp, participant=participant)
 
@@ -585,12 +651,14 @@ def summary():
         "completed": exp.is_complete(),
     }
     unfilled_nets = (
-        models.Network.query.filter(models.Network.full != true())
+        session.query(models.Network)
+        .filter(models.Network.full != true())
         .with_entities(models.Network.id, models.Network.max_size)
         .all()
     )
     working = (
-        models.Participant.query.filter_by(status="working")
+        session.query(models.Participant)
+        .filter_by(status="working")
         .with_entities(func.count(models.Participant.id))
         .scalar()
     )
@@ -603,7 +671,8 @@ def summary():
     else:
         for net in unfilled_nets:
             node_count = (
-                models.Node.query.filter_by(network_id=net.id, failed=False)
+                session.query(models.Node)
+                .filter_by(network_id=net.id, failed=False)
                 .with_entities(func.count(models.Node.id))
                 .scalar()
             )
@@ -618,13 +687,17 @@ def summary():
 
     # Regenerate a waiting room message when checking status
     # to counter missed messages at the end of the waiting room
-    nonfailed_count = models.Participant.query.filter(
-        (models.Participant.status == "working")
-        | (models.Participant.status == "overrecruited")
-        | (models.Participant.status == "submitted")
-        | (models.Participant.status == "approved")
-    ).count()
-    exp = Experiment(session)
+    nonfailed_count = (
+        session.query(models.Participant)
+        .filter(
+            (models.Participant.status == "working")
+            | (models.Participant.status == "recruiter_submission_started")
+            | (models.Participant.status == "overrecruited")
+            | (models.Participant.status == "submitted")
+            | (models.Participant.status == "approved")
+        )
+        .count()
+    )
     overrecruited = exp.is_overrecruited(nonfailed_count)
     if exp.quorum:
         quorum = {"q": exp.quorum, "n": nonfailed_count, "overrecruited": overrecruited}
@@ -696,8 +769,6 @@ def request_parameter(parameter, parameter_type=None, default=None, optional=Fal
     or if the parameter is found but is of the wrong type
     then a Response object is returned
     """
-    exp = Experiment(session)
-
     # get the parameter
     try:
         value = request.values[parameter]
@@ -730,6 +801,7 @@ def request_parameter(parameter, parameter_type=None, default=None, optional=Fal
     elif parameter_type == "known_class":
         # if its a known class check against the known classes
         try:
+            exp = Experiment(session)
             value = exp.known_classes[value]
             return value
         except KeyError:
@@ -782,41 +854,63 @@ def create_participant(worker_id, hit_id, assignment_id, mode, entry_information
     defined in reference to the participant object. You must specify the
     worker_id, hit_id, assignment_id, and mode in the url.
     """
-    # Lock the table, triggering multiple simultaneous accesses to fail
-    try:
-        session.connection().execute("LOCK TABLE participant IN EXCLUSIVE MODE NOWAIT")
-    except exc.OperationalError as e:
-        e.orig = TransactionRollbackError()
-        raise e
+    config = get_config()
+    if not config.ready:
+        config.load()
+
+    recruiter_name = request.args.get("recruiter")
+    fingerprint_hash = request.args.get("fingerprint_hash") or request.form.get(
+        "fingerprint_hash"
+    )
+
+    if config.get("lock_table_when_creating_participant"):
+        # Historically we have locked the participant table when creating participants
+        # to avoid database inconsistency problems. However some experimenters have experienced
+        # some deadlocking problems associated with this locking, so we have made
+        # it an opt-out behavior.
+        try:
+            session.connection().execute(
+                "LOCK TABLE participant IN EXCLUSIVE MODE NOWAIT"
+            )
+        except exc.OperationalError as e:
+            e.orig = TransactionRollbackError()
+            raise e
 
     missing = [p for p in (worker_id, hit_id, assignment_id) if p == "undefined"]
     if missing:
         msg = "/participant POST: required values were 'undefined'"
         return error_response(error_type=msg, status=403)
 
-    fingerprint_hash = request.args.get("fingerprint_hash") or request.form.get(
-        "fingerprint_hash"
-    )
+    exp = Experiment(session)
+
     fingerprint_found = False
     if fingerprint_hash:
         try:
-            fingerprint_found = models.Participant.query.filter_by(
-                fingerprint_hash=fingerprint_hash
-            ).one_or_none()
+            fingerprint_found = (
+                session.query(models.Participant)
+                .filter_by(fingerprint_hash=fingerprint_hash)
+                .one_or_none()
+            )
         except MultipleResultsFound:
             fingerprint_found = True
 
     if fingerprint_hash and fingerprint_found:
         db.logger.warning("Same browser fingerprint detected.")
 
-        if mode == "live":
-            return error_response(
-                error_type="/participant POST: Same participant dectected.", status=403
-            )
+        # if mode == "live":
+        #     return error_response(
+        #         error_type="/participant POST: Same participant dectected.", status=403
+        #     )
+        # We previously used to throw an error in such cases in a deployed experiment,
+        # but we found this behaviour problematic in HotAirRecruiter (when we might have participants
+        # legitimately taking the same experiment twice) and also sometimes problematic in other
+        # recruiters (e.g. it is possible to configure a Prolific study to allow repeat participation).
+        # If this proves to be a problem, we can make this configurable via a config parameter in the future.
+        # For now we just log a warning.
 
-    already_participated = models.Participant.query.filter_by(
-        worker_id=worker_id
-    ).one_or_none()
+    already_participated = (
+        session.query(models.Participant).filter_by(worker_id=worker_id).one_or_none()
+    )
 
     if already_participated:
         db.logger.warning("Worker has already participated.")
@@ -824,33 +918,35 @@ def create_participant(worker_id, hit_id, assignment_id, mode, entry_information
             error_type="/participant POST: worker has already participated.", status=403
         )
 
-    duplicate = models.Participant.query.filter_by(
-        assignment_id=assignment_id, status="working"
-    ).one_or_none()
+    duplicate = (
+        session.query(models.Participant)
+        .filter_by(assignment_id=assignment_id, status="working")
+        .one_or_none()
+    )
 
     if duplicate:
         msg = """
-            AWS has reused assignment_id while existing participant is
+            {} has reused assignment_id while existing participant is
             working. Replacing older participant {}.
         """
-        app.logger.warning(msg.format(duplicate.id))
+        app.logger.warning(msg.format(recruiter_name, duplicate.id))
         q.enqueue(worker_function, "AssignmentReassigned", None, duplicate.id)
 
     # Count working or beyond participants.
     nonfailed_count = (
-        models.Participant.query.filter(
+        session.query(models.Participant)
+        .filter(
             (models.Participant.status == "working")
+            | (models.Participant.status == "recruiter_submission_started")
             | (models.Participant.status == "overrecruited")
             | (models.Participant.status == "submitted")
             | (models.Participant.status == "approved")
-        ).count()
+        )
+        .count()
         + 1
     )
 
-    recruiter_name = request.args.get("recruiter")
-
     # Create the new participant.
-    exp = Experiment(session)
     participant_vals = {
         "worker_id": worker_id,
         "hit_id": hit_id,
@@ -874,7 +970,14 @@ def create_participant(worker_id, hit_id, assignment_id, mode, entry_information
     if overrecruited:
         participant.status = "overrecruited"
 
-    result = {"participant": participant.__json__()}
+    result = {
+        "participant": {
+            **participant.__json__(),
+            # Add some extra information that is useful for initializing dallinger.identity
+            "unique_id": participant.unique_id,
+            "worker_id": participant.worker_id,
+        }
+    }
 
     # Queue notification to others in waiting room
     if exp.quorum:
@@ -907,7 +1010,7 @@ def post_participant():
 def get_participant(participant_id):
     """Get the participant with the given id."""
     try:
-        ppt = models.Participant.query.filter_by(id=participant_id).one()
+        ppt = session.query(models.Participant).filter_by(id=participant_id).one()
     except NoResultFound:
         return error_response(
             error_type="/participant GET: no participant found", status=403
@@ -945,7 +1048,7 @@ def load_participant():
 def get_network(network_id):
     """Get the network with the given id."""
     try:
-        net = models.Network.query.filter_by(id=network_id).one()
+        net = session.query(models.Network).filter_by(id=network_id).one()
     except NoResultFound:
         return error_response(error_type="/network GET: no network found", status=403)
 
@@ -964,7 +1067,7 @@ def create_question(participant_id):
     """
     # Get the participant.
     try:
-        ppt = models.Participant.query.filter_by(id=participant_id).one()
+        ppt = session.query(models.Participant).filter_by(id=participant_id).one()
     except NoResultFound:
         return error_response(
             error_type="/question POST no participant found", status=403
@@ -996,9 +1099,10 @@ def create_question(participant_id):
 
     try:
         # execute the request
-        models.Question(
+        question = models.Question(
             participant=ppt, question=question, response=response, number=number
         )
+        session.add(question)
         session.commit()
     except Exception:
         return error_response(error_type="/question POST server error", status=403)
@@ -1029,11 +1133,11 @@ def node_neighbors(node_id):
     connection = request_parameter(parameter="connection", default="to")
     failed = request_parameter(parameter="failed", parameter_type="bool", optional=True)
     for x in [node_type, connection]:
-        if type(x) == Response:
+        if isinstance(x, Response):
             return x
 
     # make sure the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(
             error_type="/node/neighbors, node does not exist",
@@ -1076,7 +1180,9 @@ def create_node(participant_id):
 
     # Get the participant.
     try:
-        participant = models.Participant.query.filter_by(id=participant_id).one()
+        participant = (
+            session.query(models.Participant).filter_by(id=participant_id).one()
+        )
     except NoResultFound:
         return error_response(error_type="/node POST no participant found", status=403)
 
@@ -1114,11 +1220,11 @@ def node_vectors(node_id):
     direction = request_parameter(parameter="direction", default="all")
     failed = request_parameter(parameter="failed", parameter_type="bool", default=False)
     for x in [direction, failed]:
-        if type(x) == Response:
+        if isinstance(x, Response):
             return x
 
     # execute the request
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/node/vectors, node does not exist")
 
@@ -1152,11 +1258,11 @@ def connect(node_id, other_node_id):
         return direction
 
     # check the nodes exist
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/node/connect, node does not exist")
 
-    other_node = models.Node.query.get(other_node_id)
+    other_node = session.query(models.Node).get(other_node_id)
     if other_node is None:
         return error_response(
             error_type="/node/connect, other node does not exist",
@@ -1192,7 +1298,7 @@ def get_info(node_id, info_id):
     exp = Experiment(session)
 
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/info, node does not exist")
 
@@ -1239,11 +1345,11 @@ def node_infos(node_id):
     info_type = request_parameter(
         parameter="info_type", parameter_type="known_class", default=models.Info
     )
-    if type(info_type) == Response:
+    if isinstance(info_type, Response):
         return info_type
 
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/node/infos, node does not exist")
 
@@ -1278,11 +1384,11 @@ def node_received_infos(node_id):
     info_type = request_parameter(
         parameter="info_type", parameter_type="known_class", default=models.Info
     )
-    if type(info_type) == Response:
+    if isinstance(info_type, Response):
         return info_type
 
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(
             error_type="/node/infos, node {} does not exist".format(node_id)
@@ -1315,7 +1421,7 @@ def tracking_event_post(node_id):
         details = loads(details)
 
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/info POST, node does not exist")
 
@@ -1351,14 +1457,15 @@ def info_post(node_id):
     failed = request_parameter(parameter="failed", parameter_type="bool", default=False)
 
     for x in [contents, info_type]:
-        if type(x) == Response:
+        if isinstance(x, Response):
             return x
+
+    exp = Experiment(session)
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/info POST, node does not exist")
 
-    exp = Experiment(session)
     try:
         # execute the request
         additional_params = {}
@@ -1396,11 +1503,11 @@ def node_transmissions(node_id):
     direction = request_parameter(parameter="direction", default="incoming")
     status = request_parameter(parameter="status", default="all")
     for x in [direction, status]:
-        if type(x) == Response:
+        if isinstance(x, Response):
             return x
 
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/node/transmissions, node does not exist")
 
@@ -1458,7 +1565,7 @@ def node_transmit(node_id):
     to_whom = request_parameter(parameter="to_whom", optional=True)
 
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(error_type="/node/transmit, node does not exist")
     # create what
@@ -1484,7 +1591,7 @@ def node_transmit(node_id):
     if to_whom is not None:
         try:
             to_whom = int(to_whom)
-            to_whom = models.Node.query.get(to_whom)
+            to_whom = session.query(models.Node).get(to_whom)
             if to_whom is None:
                 return error_response(
                     error_type="/node/transmit POST, recipient Node does not exist",
@@ -1533,11 +1640,11 @@ def transformation_get(node_id):
         parameter_type="known_class",
         default=models.Transformation,
     )
-    if type(transformation_type) == Response:
+    if isinstance(transformation_type, Response):
         return transformation_type
 
     # check the node exists
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(
             error_type="/node/transformations, "
@@ -1576,11 +1683,11 @@ def transformation_post(node_id, info_in_id, info_out_id):
         parameter_type="known_class",
         default=models.Transformation,
     )
-    if type(transformation_type) == Response:
+    if isinstance(transformation_type, Response):
         return transformation_type
 
     # Check that the node etc. exists.
-    node = models.Node.query.get(node_id)
+    node = session.query(models.Node).get(node_id)
     if node is None:
         return error_response(
             error_type="/transformation POST, " "node {} does not exist".format(node_id)
@@ -1645,9 +1752,11 @@ def check_for_duplicate_assignments(participant):
 
     If it isnt the older participants will be failed.
     """
-    participants = models.Participant.query.filter_by(
-        assignment_id=participant.assignment_id
-    ).all()
+    participants = (
+        session.query(models.Participant)
+        .filter_by(assignment_id=participant.assignment_id)
+        .all()
+    )
     duplicates = [
         p for p in participants if (p.id != participant.id and p.status == "working")
     ]
@@ -1658,7 +1767,15 @@ def check_for_duplicate_assignments(participant):
 @app.route("/worker_complete", methods=["POST"])
 @db.scoped_session_decorator
 def worker_complete():
-    """Complete worker."""
+    """Called when a participant completes their task.
+
+    1. Loads participant row, with a lock
+    2. Checks participant status, to avoid double-submits
+    3. Updates end_time of participant
+    4. Calls exp.participant_task_completed(participant)
+    5. Asks recruiter for new participant status and possible action to run
+    6. Runs any action requested by recruiter (synchronously)
+    """
     participant_id = request.values.get("participant_id")
     if not participant_id:
         return error_response(
@@ -1676,37 +1793,47 @@ def worker_complete():
 
 
 def _worker_complete(participant_id):
-    participant = models.Participant.query.get(participant_id)
+    # Lock the participant row, then check and update status to avoid
+    # double-submits:
+    exp = Experiment(session)
+    participant = (
+        session.query(models.Participant)
+        .populate_existing()
+        .with_for_update(of=models.Participant)
+        .get(participant_id)
+    )
     if participant is None:
         raise KeyError()
 
-    if participant.end_time is not None:  # Provide idempotence
+    # Provide idempotence
+    if participant.end_time is not None:
         return
 
     participant.end_time = datetime.now()
-    session.commit()
 
-    # Notify experiment that participant has been marked complete. Doing
-    # this here, rather than in the worker function, means that
-    # the experiment can request qualification assignment before the
-    # worker completes the HIT when using a recruiter like MTurk, where
-    # execution of the `worker_events.AssignmentSubmitted` command is
-    # deferred until they've submitted the HIT on the MTurk platform.
-    exp = Experiment(session)
+    # Immediately notify experiment that participant has been marked complete.
+    # This is done first so that the experiment can request qualification
+    # assignment before the worker submits their task on asynchronous
+    # recruitment platforms like MTurk. This prevents them from possibly being
+    # offered another task which they may no longer be qualified for.
     exp.participant_task_completed(participant)
 
-    event_type = participant.recruiter.submitted_event()
+    # The recruiter tells us which status to assign the participant,
+    # and synchronous recruiters also return the name of an event
+    # command to run immediately (and synchronously)
+    status_and_action = participant.recruiter.on_task_completion()
+    participant.status = status_and_action["new_status"]
+    # NB: commit releases lock.
+    session.commit()
 
-    if event_type is None:
-        return
-
-    # Currently we execute this function synchronously, regardless of the
-    # event type:
-    worker_function(
-        event_type=event_type,
-        assignment_id=participant.assignment_id,
-        participant_id=participant_id,
-    )
+    if "action" in status_and_action:
+        # Currently we execute this function synchronously, regardless of the
+        # event type:
+        worker_function(
+            event_type=status_and_action["action"],
+            assignment_id=participant.assignment_id,
+            participant_id=participant_id,
+        )
 
 
 @app.route("/worker_failed", methods=["GET"])
@@ -1732,7 +1859,7 @@ def worker_failed():
 
 
 def _worker_failed(participant_id):
-    participants = models.Participant.query.filter_by(id=participant_id).all()
+    participants = session.query(models.Participant).filter_by(id=participant_id).all()
     if not participants:
         raise KeyError()
 
