@@ -14,10 +14,11 @@ import traceback
 
 import psutil
 import six
+from cached_property import cached_property
 from six.moves import shlex_quote as quote
 
 from dallinger.config import SENSITIVE_KEY_NAMES
-from dallinger.utils import check_call, check_output
+from dallinger.utils import check_call, check_output, port_is_open
 
 
 def app_name(experiment_uuid):
@@ -57,11 +58,19 @@ def request_headers(auth_token):
 class HerokuCommandRunner(object):
     """Heroku command runner base class"""
 
-    def __init__(self, output=None, team=None):
+    def __init__(self, output=None, team=None, region=None):
         self.out = output
         self.team = team
+        self.region = region
         self.out_muted = open(os.devnull, "w")
 
+    def __del__(self):
+        # Make sure we close our filehandle when object deleted
+        try:
+            self.out_muted.close()
+        except Exception:
+            pass
+
     @property
     def sys_encoding(self):
         # Encoding of strings returned from subprocess calls. The Click
@@ -100,6 +109,8 @@ class HerokuInfo(HerokuCommandRunner):
         cmd = ["heroku", "apps", "--json"]
         if self.team:
             cmd.extend(["--team", self.team])
+        if self.region:
+            cmd.extend(["--region", self.region])
         return json.loads(self._result(cmd))
 
     def my_apps(self):
@@ -119,9 +130,9 @@ class HerokuInfo(HerokuCommandRunner):
 class HerokuApp(HerokuCommandRunner):
     """Representation of a Heroku app"""
 
-    def __init__(self, dallinger_uid, output=None, team=None):
+    def __init__(self, dallinger_uid, output=None, team=None, region=None):
         self.dallinger_uid = dallinger_uid
-        super(HerokuApp, self).__init__(output, team)
+        super(HerokuApp, self).__init__(output, team, region)
 
     def bootstrap(self, buildpack="heroku/python"):
         """Creates the heroku app and local git remote. Call this once you're
@@ -135,6 +146,10 @@ class HerokuApp(HerokuCommandRunner):
         if self.team:
             cmd.extend(["--team", self.team])
 
+        # If a region is specified, create the app in the specified region
+        if self.region:
+            cmd.extend(["--region", self.region])
+
         self._run(cmd)
         # Set HOST value
         self.set_multiple(
@@ -162,9 +177,18 @@ class HerokuApp(HerokuCommandRunner):
     def name(self):
         return app_name(self.dallinger_uid)
 
-    @property
+    @cached_property
     def url(self):
-        return "https://{}.herokuapp.com".format(self.name)
+        app_info = self._result(
+            [
+                "heroku",
+                "apps:info",
+                "--app",
+                self.name,
+                "--json",
+            ]
+        )
+        return json.loads(app_info)["app"]["web_url"].rstrip("/")
 
     def addon(self, name):
         """Set up an addon"""
@@ -215,7 +239,7 @@ class HerokuApp(HerokuCommandRunner):
         postgres://some-long-uid@ec2-52-7-232-59.compute-1.amazonaws.com:5432/d5fou154it1nvt
         """
         output = self.get("DATABASE", subcommand="pg:credentials:url")
-        match = re.search("(postgres://.*)$", output)
+        match = re.search("(postgres://.*)", output)
         if match is None:
             raise NameError(
                 "Could not retrieve the DB URI. Check for error output from "
@@ -400,7 +424,6 @@ class HerokuLocalWrapper(object):
     """
 
     shell_command = "heroku"
-    success_regex = r"^.*? \d+ workers$"
     # On Windows, use 'CTRL_C_EVENT', otherwise SIGINT
     int_signal = getattr(signal, "CTRL_C_EVENT", signal.SIGINT)
     MONITOR_STOP = object()
@@ -429,10 +452,9 @@ class HerokuLocalWrapper(object):
 
     def start(self, timeout_secs=60):
         """Start the heroku local subprocess group and verify that
-        it has started successfully.
+        it has started successfully by polling the relevant port.
 
-        The subprocess output is checked for a line matching 'success_regex'
-        to indicate success. If no match is seen after 'timeout_secs',
+        If the port is not available after 'timeout_secs',
         a HerokuTimeoutError is raised.
         """
 
@@ -474,11 +496,29 @@ class HerokuLocalWrapper(object):
             return
 
         try:
-            os.killpg(os.getpgid(self._process.pid), signal)
-            self.out.log("Local Heroku process terminated.")
-        except OSError:
-            self.out.log("Local Heroku was already terminated.")
-            self.out.log(traceback.format_exc())
+            # Try to kill the process group if still running
+            try:
+                os.killpg(os.getpgid(self._process.pid), signal)
+                self.out.log("Local Heroku process terminated.")
+            except OSError:
+                self.out.log("Local Heroku was already terminated.")
+            except Exception:
+                self.out.log("Unexpected error while terminating local Heroku.")
+                self.out.log(traceback.format_exc())
+
+            # Ensure the process is fully cleaned up by calling wait(), even if
+            # it has already terminated.
+            try:
+                self._process.wait(timeout=5)
+            except Exception:
+                self.out.log("Process did not terminate within timeout.")
+
+            # Close stdout to avoid ResourceWarning
+            try:
+                self._process.stdout.close()
+            except Exception:
+                pass
+
         finally:
             self._process = None
 
@@ -492,12 +532,13 @@ class HerokuLocalWrapper(object):
                 return
 
     def _verify_startup(self):
+        port = self.config.get("base_port")
         for line in self._stream():
             self._record.append(line)
             if self.verbose:
                 self.out.blather(line)
             line = line.strip()
-            if self._up_and_running(line):
+            if self._up_and_running(port):
                 return True
 
             if self._redis_not_running(line):
@@ -575,8 +616,8 @@ class HerokuLocalWrapper(object):
     def _stream(self):
         return iter(self._process.stdout.readline, self.STREAM_SENTINEL)
 
-    def _up_and_running(self, line):
-        return re.match(self.success_regex, line)
+    def _up_and_running(self, port):
+        return port_is_open(port)
 
     def _redis_not_running(self, line):
         return re.match(r"^.*? worker.1 .*? Connection refused.$", line)
