@@ -4,8 +4,8 @@ import shutil
 import sys
 import tempfile
 import time
+from unittest import mock
 
-import mock
 import pexpect
 import pytest
 from jinja2 import FileSystemLoader
@@ -142,6 +142,7 @@ def stub_config():
         "contact_email_on_error": "error_contact@test.com",
         "dallinger_email_address": "test@example.com",
         "database_size": "standard-0",
+        "disable_browser_autotranslate": True,
         "disable_when_duration_exceeded": True,
         "enable_global_experiment_registry": False,
         "redis_size": "premium-0",
@@ -158,12 +159,15 @@ def stub_config():
         "id": "TEST_EXPERIMENT_UID",  # This is a significant value; change with caution.
         "keywords": "kw1, kw2, kw3",
         "lifetime": 1,
+        "lock_table_when_creating_participant": True,
         "logfile": "-",
         "loglevel": 0,
+        "loglevel_worker": 2,
         "mode": "debug",
         "num_dynos_web": 1,
         "num_dynos_worker": 1,
         "organization_name": "Monsters University",
+        "publish_experiment": True,
         "sentry": True,
         "smtp_host": "smtp.fakehost.com:587",
         "smtp_username": "fake email username",
@@ -183,7 +187,9 @@ def stub_config():
         config.register(*key)
     config.extend(defaults.copy())
     # Patch load() so we don't update any key/value pairs from actual files:
-    config.load = mock.Mock(side_effect=lambda: setattr(config, "ready", True))
+    config.load = mock.Mock(
+        side_effect=lambda strict=True: setattr(config, "ready", True)
+    )
     config.ready = True
 
     return config
@@ -245,13 +251,15 @@ def webapp_admin(csrf_token, webapp):
 def db_session():
     import dallinger.db
 
-    # The drop_all call can hang without this; see:
-    # https://stackoverflow.com/questions/13882407/sqlalchemy-blocked-on-dropping-tables
-    dallinger.db.session.close()
-    session = dallinger.db.init_db(drop_all=True)
-    yield session
-    session.rollback()
-    session.close()
+    try:
+        # Initialize the database and drop all tables
+        session = dallinger.db.init_db(drop_all=True)
+        session.commit()
+        yield session
+    finally:
+        # No need to rollback here, since the test may have committed
+        # Close and return the session to the pool
+        session.remove()
 
 
 @pytest.fixture
@@ -395,7 +403,7 @@ def a(db_session):
 
         def _insert(self, thing):
             db_session.add(thing)
-            db_session.flush()  # This gets us an ID and sets relationships
+            db_session.commit()
 
     return ModelFactory(db_session)
 
@@ -419,9 +427,8 @@ def webapp(active_config, reset_sys_modules, env):
     not caching the Flask template search path, and clearing out sys.modules
     before loading the Flask app.
     """
-    from dallinger.experiment_server import sockets
+    from dallinger.experiment_server.experiment_server import app
 
-    app = sockets.app
     # look in the cwd for test's templates, and make sure the template loader
     # uses that directory to search for them.
     app.root_path = os.getcwd()
@@ -554,7 +561,7 @@ def selenium_recruits(request, recruitment_loop):
 
                 chrome_options = Options()
                 chrome_options.add_argument("--headless")
-                kwargs = {"chrome_options": chrome_options}
+                kwargs = {"options": chrome_options}
             driver = driver_class(**kwargs)
             driver.get(url)
             try:
@@ -655,3 +662,13 @@ def wait_for_text(driver, el_id, value, removed=False, timeout=10):
             return el
 
     raise AttributeError
+
+
+@pytest.fixture
+def redis_conn():
+    from dallinger.db import redis_conn as _redis
+
+    yield _redis
+
+    for key in _redis.keys():
+        _redis.delete(key)
