@@ -3,38 +3,103 @@ from __future__ import unicode_literals
 import functools
 import io
 import locale
+import logging
 import os
 import random
 import re
 import shutil
+import socket
 import string
 import subprocess
 import sys
 import tempfile
+import warnings
 import webbrowser
+from datetime import datetime
 from hashlib import md5
+from importlib.metadata import files as files_metadata
+from importlib.util import find_spec
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from unicodedata import normalize
 
-import pkg_resources
+import redis
 import requests
 from faker import Faker
 from flask import request
-from pkg_resources import get_distribution
-
-try:
-    from importlib.metadata import files as files_metadata
-except ImportError:
-    from importlib_metadata import files as files_metadata
+from pythonjsonlogger import jsonlogger
+from sqlalchemy import exc as sa_exc
 
 from dallinger import db
 from dallinger.compat import is_command
 from dallinger.config import get_config
+from dallinger.models import Participant
 from dallinger.version import __version__
 
+local_warning_cache = {}
+
+
+def show_warnings_once(warning, category, filename, lineno, file=None, line=None):
+    redis_conn = db.redis_conn
+
+    if filename is not None and lineno is not None:
+        # The same Warning can be raised using different messages, e.g. if it contains variables
+        redis_key = f"{filename}:{lineno}"
+    else:
+        redis_key = str(warning)
+    # Check if message in redis_conn if yes, fall back to the default behaviour
+    redis_available = True
+    try:
+        if redis_key in local_warning_cache or redis_conn.exists(redis_key):
+            # If the message is already logged, don't log it again
+            return
+    except redis.exceptions.ConnectionError:
+        redis_available = False
+
+    # Log the warning message to Redis
+    local_warning_cache[redis_key] = "Logged"
+    if redis_available:
+        redis_conn.set(redis_key, "Logged")
+
+    # Default behaviour
+    from warnings import formatwarning
+
+    print(formatwarning(warning, category, filename, lineno, line))
+
+
+def setup_warning_hooks():
+    warnings.showwarning = show_warnings_once
+    warnings.simplefilter("default", Warning)
+    warnings.simplefilter("ignore", category=sa_exc.SAWarning)
+
+
+try:
+    from pip._vendor import pkg_resources
+except ImportError:
+    pkg_resources = None
+
 fake = Faker()
 
+JSON_LOGFILE = "logs.jsonl"
+
+
+def attach_json_logger(log):
+    fmt = jsonlogger.JsonFormatter(
+        "%(name)s %(asctime)s %(levelname)s %(filename)s %(lineno)s %(message)s"
+    )
+    handler = logging.FileHandler(JSON_LOGFILE)
+    handler.setFormatter(fmt)
+    log.addHandler(handler)
+
+
+def get_exp_klass():
+    from dallinger import experiment
+
+    try:
+        return experiment.load()
+    except ImportError:
+        return None  # pragma: no cover
+
 
 def get_base_url():
     """Returns the base url for the experiment.
@@ -74,10 +139,7 @@ def dallinger_package_path():
     >>> utils.dallinger_package_location()
     '/Users/janedoe/projects/Dallinger3/dallinger'
     """
-    dist = get_distribution("dallinger")
-    src_base = os.path.join(dist.location, dist.project_name)
-
-    return src_base
+    return os.path.dirname(find_spec("dallinger").origin)
 
 
 def generate_random_id(size=6, chars=string.ascii_uppercase + string.digits):
@@ -321,7 +383,7 @@ def _new_webbrowser_profile():
 
 
 def struct_to_html(data):
-    parts = ["<ul>"]
+    parts = []
     if isinstance(data, (list, tuple)):
         for i in data:
             parts.append(struct_to_html(i))
@@ -335,21 +397,22 @@ def struct_to_html(data):
                 data["count"], data["failed"], failed_percentage
             )
             if failed_percentage == 100:
-                value = '<span class="all-failures">{}</span>'.format(value)
+                value = f'<span class="statistics-value all-failures">{value}</span>'
             elif failed_percentage > 0:
-                value = '<span class="some-failures">{}</span>'.format(value)
+                value = f'<span class="statistics-value some-failures">{value}</span>'
             elif data["count"]:
-                value = '<span class="no-failures">{}</span>'.format(value)
+                value = f'<span class="statistics-value no-failures">{value}</span>'
             return value
 
         for k in data:
             item = struct_to_html(data[k])
-            parts.append("<li>{}: {}</li>".format(k, item))
+            parts.append(
+                f'<span class="nowrap"><span class="statistics-key">{k}</span>: <span class="statistics-value">{item}</span></span>'
+            )
     else:
         return str(data)
 
-    parts.append("</ul>")
-    return "\n".join(parts)
+    return "<br>".join(parts)
 
 
 def abspath_from_egg(egg, path):
@@ -380,14 +443,13 @@ def get_editable_dallinger_path():
 def check_local_db_connection(log):
     """Verify that the local Postgres server is running."""
     try:
-        log("Checking your local Postgres database connection...")
         db.check_connection()
     except Exception:
-        log("There was a problem connecting!")
+        log("There was a problem connecting to the local Postgres server!")
         raise
 
 
-def check_experiment_dependencies(requirement_file):
+def check_experiment_dependencies(requirements_file):
     """Verify that the dependencies defined in a requirements file are
     in fact installed.
     If the environment variable SKIP_DEPENDENCY_CHECK is set, no check
@@ -396,11 +458,23 @@ def check_experiment_dependencies(requirement_file):
     if os.environ.get("SKIP_DEPENDENCY_CHECK"):
         return
     try:
-        with open(requirement_file, "r") as f:
-            dependencies = [r for r in f.readlines() if r[:3] != "-e "]
+        with open(requirements_file, "r") as f:
+            dependencies = [
+                re.split("@|\\ |>|<|=|\\[", line)[0].strip()
+                for line in f.readlines()
+                if line[:3] != "-e " and line[0].strip() not in ["#", ""]
+            ]
     except (OSError, IOError):
         dependencies = []
-    pkg_resources.require(dependencies)
+
+    for dep in dependencies:
+        if find_spec(dep) is None:
+            try:
+                pkg_resources.get_distribution(dep)
+            except (pkg_resources.DistributionNotFound, AttributeError):
+                raise ValueError(
+                    f"Please install the '{dep}' package to run this experiment."
+                )
 
 
 def develop_target_path(config):
@@ -456,7 +530,6 @@ def bootstrap_development_session(exp_config, experiment_path, log):
     source_path = Path(dallinger_package_path()) / "dev_server"
     destination_path = develop_target_path(config)
 
-    log("Wiping develop directory and re-writing it...")
     ensure_directory(destination_path)
     expunge_directory(destination_path)
     collate_experiment_files(
@@ -594,7 +667,6 @@ cp requirements.txt constraints.txt"""
             check_output(
                 [
                     "pip-compile",
-                    "--resolver=backtracking",
                     "-v",
                     str(tmpfile),
                     "-o",
@@ -720,7 +792,9 @@ class FileSource(object):
     @property
     def size(self):
         """Combined size of all files, accounting for exclusions."""
-        return sum([os.path.getsize(path) for path in self.files])
+        return sum(
+            [os.path.getsize(path) for path in self.files if not os.path.islink(path)]
+        )
 
     def apply_to(self, destination, copy_func=copy_file):
         """Copy files based iterable of source and destination tuples.
@@ -731,6 +805,12 @@ class FileSource(object):
             ensure_directory(target_folder)
             if os.path.exists(to_path):
                 continue
+            if is_broken_symlink(from_path):
+                raise OSError(
+                    f"Cannot copy broken symlink: {from_path}. "
+                    "If this file comes from a virtual environment directory (e.g. .venv), we recommend adding that "
+                    "virtual environment directory to your .gitignore file."
+                )
             copy_func(from_path, to_path)
 
     def map_locations_to(self, destination):
@@ -741,6 +821,14 @@ class FileSource(object):
         raise NotImplementedError()
 
 
+def is_broken_symlink(path: str) -> bool:
+    """Check if the given path is a broken symlink.
+    Relies on the documented behavior of ``os.path.exists``:
+    "Return True if path refers to an existing path. Returns False for broken symbolic links".
+    """
+    return os.path.islink(path) and not os.path.exists(path)
+
+
 class DallingerFileSource(FileSource):
     """The core Dallinger framework is a source for files to be used in an
     experiment run. These include the /frontend directory contents,
@@ -880,6 +968,7 @@ def exclusion_policy():
             "config.txt",
             "*.db",
             "*.dmg",
+            "local_only",
             "node_modules",
             "snapshots",
             "data",
@@ -911,15 +1000,118 @@ def build_and_place(source: str, destination: str) -> str:
     return package_path.name
 
 
-def deferred_route_decorator(route, registered_routes):
+def route_name_from_func_name(func_name: str) -> str:
+    return func_name
+
+
+def deferred_route_decorator(
+    route, registered_routes, rename_route_name=route_name_from_func_name
+):
     def new_func(func):
         # Check `__func__` in case we have a classmethod or staticmethod
         base_func = getattr(func, "__func__", func)
-        name = getattr(base_func, "__name__", None)
-        if name is not None:
-            route["func_name"] = name
+        func_name = getattr(base_func, "__name__", None)
+        if func_name is not None:
+            route["func_name"] = func_name
+            route["name"] = rename_route_name(func_name)
             if route not in registered_routes:
                 registered_routes.append(route)
         return func
 
     return new_func
+
+
+def get_from_config(key):
+    config = get_config()
+    if not config.ready:
+        config.load()
+    return config.get(key)
+
+
+class ClassPropertyDescriptor(object):
+    def __init__(self, fget, fset=None):
+        self.fget = fget
+        self.fset = fset
+
+    def __get__(self, obj, cls=None):
+        if cls is None:
+            cls = type(obj)
+        return self.fget.__get__(obj, cls)()
+
+    def __set__(self, obj, value):
+        if not self.fset:
+            raise AttributeError("can't set attribute")
+        type_ = type(obj)
+        return self.fset.__get__(obj, type_)(value)
+
+    def setter(self, func):
+        if not isinstance(func, (classmethod, staticmethod)):
+            func = classmethod(func)
+        self.fset = func
+        return self
+
+
+def classproperty(func):
+    """
+    Defines an analogous version of @property but for classes,
+    after https://stackoverflow.com/questions/5189699/how-to-make-a-class-property.
+    """
+    if not isinstance(func, (classmethod, staticmethod)):
+        func = classmethod(func)
+
+    return ClassPropertyDescriptor(func)
+
+
+def median_time_spent_in_hours(participants: list[Participant]) -> float:
+    """
+    Parameters:
+        participants: list[Participant]
+            A list of participants to calculate the median time spent in hours for.
+
+    Returns:
+        float
+            The median time spent in hours up to the current time by participants from the provided list of participants.
+    """
+    from statistics import median
+
+    if not participants:
+        return None
+
+    times = [datetime.now() - participant.creation_time for participant in participants]
+
+    median_time = median(times)
+
+    return median_time.total_seconds() / 3600
+
+
+# Terminal color constants
+RED = "\033[31m"
+END = "\033[0m"
+GREEN = "\033[32m"
+BLUE = "\033[34m"
+BOLD = "\033[1m"
+
+
+def print_bold(message):
+    """Print with bold formatting."""
+    print(f"{BOLD}{message}{END}")
+
+
+def port_is_open(port, host="127.0.0.1"):
+    """
+    Check if a TCP port is open on a given host.
+
+    Args:
+        port (int): The port number to check.
+        host (str, optional): The host to check (default is '127.0.0.1').
+
+    Returns:
+        bool: True if the port is open, False otherwise.
+    """
+    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
+        s.settimeout(1)
+        try:
+            s.connect((host, port))
+            return True
+        except (ConnectionRefusedError, OSError):
+            return False
