@@ -1,4 +1,3 @@
-"""Heroku web worker."""
 listen = ["high", "default", "low"]
 
 
@@ -16,15 +15,32 @@ def main():
 
     from gevent.queue import LifoQueue
     from redis import BlockingConnectionPool, StrictRedis
-    from rq import Connection, Queue
+    from rq import Queue
     from six.moves.urllib.parse import urlparse
 
-    from dallinger.config import initialize_experiment_package
+    from dallinger.config import get_config, initialize_experiment_package
     from dallinger.heroku.rq_gevent_worker import GeventWorker as Worker
+    from dallinger.utils import attach_json_logger
 
     initialize_experiment_package(os.getcwd())
 
-    logging.basicConfig(format="%(asctime)s %(message)s", level=logging.DEBUG)
+    config = get_config()
+    if not config.ready:
+        config.load()
+
+    LOG_LEVELS = [
+        logging.DEBUG,
+        logging.INFO,
+        logging.WARNING,
+        logging.ERROR,
+        logging.CRITICAL,
+    ]
+    LOG_LEVEL = LOG_LEVELS[config.get("loglevel_worker")]
+    logging.basicConfig(
+        format="%(asctime)s %(message)s",
+        level=LOG_LEVEL,
+    )
+    attach_json_logger(logging.getLogger())
     redis_url = os.getenv("REDIS_URL", "redis://localhost:6379")
     # Specify queue class for improved performance with gevent.
     # see http://carsonip.me/posts/10x-faster-python-gevent-redis-connection-pool/
@@ -40,9 +56,16 @@ def main():
     redis_pool = BlockingConnectionPool.from_url(**connection_args)
     redis_conn = StrictRedis(connection_pool=redis_pool)
 
-    with Connection(redis_conn):
-        worker = Worker(list(map(Queue, listen)))
-        worker.work()
+    worker = Worker(
+        [Queue(name, connection=redis_conn) for name in listen],
+        connection=redis_conn,
+        log_job_description=False,
+    )
+    worker.log_result_lifespan = False
+    # Default to log.warn because rq logs extremely verbosely at the info
+    # level
+    worker.log.info = worker.log.debug
+    worker.work(logging_level=LOG_LEVEL)
 
 
 if __name__ == "__main__":  # pragma: nocover
