@@ -4,10 +4,10 @@ import os
 import re
 import subprocess
 from time import sleep
+from unittest import mock
 from uuid import UUID
 
 import click
-import mock
 import pytest
 import six
 from click.testing import CliRunner
@@ -111,8 +111,8 @@ class TestVerify(object):
 @pytest.mark.slow
 class TestCommandLine(object):
     def test_dallinger_no_args(self):
-        output = subprocess.check_output(["dallinger"])
-        assert b"Usage: dallinger [OPTIONS] COMMAND [ARGS]" in output
+        result = subprocess.run(["dallinger"], capture_output=True, text=True)
+        assert "Usage: dallinger [OPTIONS] COMMAND [ARGS]" in result.stderr
 
     def test_log_empty(self):
         id = "dlgr-3b9c2aeb"
@@ -207,7 +207,6 @@ class TestDevelopCommand(object):
         result = CliRunner().invoke(develop, ["bootstrap"])
 
         assert result.exit_code == 0
-        assert "Preparing your pristine development environment" in result.output
         assert found_in("app.py", develop_directory)
         assert found_in("run.sh", develop_directory)
         assert found_in("experiment.py", develop_directory)
@@ -286,9 +285,9 @@ class TestSandboxAndDeploy(object):
         assert "directory is not a valid Dallinger experiment" in result.output
 
     def test_uses_specified_app_id(self, sandbox, dsss):
-        CliRunner().invoke(sandbox, ["--verbose", "--app", "some app id"])
+        CliRunner().invoke(sandbox, ["--verbose", "--app", "some-app-id"])
         dsss.assert_called_once_with(
-            app="some app id", verbose=True, log=mock.ANY, prelaunch_actions=[]
+            app="some-app-id", verbose=True, log=mock.ANY, prelaunch_actions=[]
         )
 
     def test_works_with_no_app_id(self, sandbox, dsss):
@@ -310,11 +309,22 @@ class TestSandboxAndDeploy(object):
         assert active_config.get("logfile") == "-"
 
     def test_rejects_invalid_app_id(self, sandbox, dsss):
-        result = CliRunner().invoke(sandbox, ["--verbose", "--app", "dlgr-some app id"])
+        result = CliRunner().invoke(sandbox, ["--verbose", "--app", "dlgr-some-app-id"])
         dsss.assert_not_called()
         assert result.exit_code == 2
         assert "The --app parameter requires the full UUID" in result.output
 
+    def test_rejects_invalid_symbols_in_app_id(self, sandbox, dsss):
+        result = CliRunner().invoke(
+            sandbox, ["--verbose", "--app", "some-app-id_with_underscores"]
+        )
+        dsss.assert_not_called()
+        assert result.exit_code == 2
+        assert (
+            "The --app parameter contains invalid characters. The only characters allowed are: a-z, 0-9, and '-'."
+            in result.output
+        )
+
     def test_accepts_valid_archive_path(self, sandbox, tempdir, dsss):
         CliRunner().invoke(sandbox, ["--verbose", "--archive", tempdir])
 
@@ -341,9 +351,9 @@ class TestLoad(object):
             yield dep
 
     def test_load_with_app_id(self, load, deployment):
-        CliRunner().invoke(load, ["--app", "some app id", "--replay", "--verbose"])
+        CliRunner().invoke(load, ["--app", "some-app-id", "--replay", "--verbose"])
         deployment.assert_called_once_with(
-            "some app id", mock.ANY, True, {"replay": True}
+            "some-app-id", mock.ANY, True, {"replay": True}
         )
 
 
@@ -370,7 +380,8 @@ class TestSummary(object):
             yield req
 
     def test_summary(self, summary, patched_summary_route):
-        result = CliRunner().invoke(summary, ["--app", "some app id"])
+        with mock.patch("dallinger.heroku.tools.HerokuApp.url"):
+            result = CliRunner().invoke(summary, ["--app", "some-app-id"])
         assert "Yield: 50.00%" in result.output
 
 
@@ -400,12 +411,13 @@ class TestBot(object):
         assert isinstance(bot, BotBase)
 
     def test_bot_no_debug_url(self, bot_command, mock_bot):
-        CliRunner().invoke(bot_command, ["--app", "some app id"])
+        with mock.patch("dallinger.heroku.tools.HerokuApp.url"):
+            CliRunner().invoke(bot_command, ["--app", "some-app-id"])
 
         assert mock_bot.run_experiment.called
 
     def test_bot_with_debug_url(self, bot_command, mock_bot):
-        CliRunner().invoke(bot_command, ["--app", "some app id", "--debug", "some url"])
+        CliRunner().invoke(bot_command, ["--app", "some-app-id", "--debug", "some url"])
 
         assert mock_bot.run_experiment.called
 
@@ -985,8 +997,8 @@ class TestDestroy(object):
         )
         heroku.destroy.assert_called_once()
         mturk_instance = mturk.return_value
-        mturk_instance.get_hits.not_called()
-        mturk_instance.expire_hit.not_called()
+        mturk_instance.get_hits.assert_not_called()
+        mturk_instance.expire_hit.assert_not_called()
 
     def test_requires_confirmation(self, destroy, heroku):
         CliRunner().invoke(destroy, ["--app", "some-app-uid"])
