@@ -2,8 +2,8 @@
 from __future__ import unicode_literals
 
 import codecs
+from unittest import mock
 
-import mock
 import pytest
 
 from dallinger.experiment_server.dashboard import DashboardTab
@@ -24,81 +24,92 @@ class TestDashboardTabs(object):
     def dashboard_tabs(self):
         from dallinger.experiment_server.dashboard import DashboardTabs
 
-        return DashboardTabs([DashboardTab("Home", "dashboard.index")])
+        return DashboardTabs([DashboardTab("Home", "dashboard.dashboard_index")])
 
     def test_dashboard_iter(self):
         from dallinger.experiment_server.dashboard import DashboardTabs
 
         dashboard_tabs = DashboardTabs(
-            [DashboardTab("Home", "index"), DashboardTab("Second", "dashboard.second")]
+            [
+                DashboardTab("Home", "dashboard.dashboard_index"),
+                DashboardTab("Second", "dashboard.dashboard_second"),
+            ]
         )
         tab_list = list(dashboard_tabs)
         assert len(tab_list) == 2
         assert tab_list[0].title == "Home"
-        assert tab_list[0].route_name == "dashboard.index"
+        assert tab_list[0].route_name == "dashboard.dashboard_index"
         assert tab_list[1].title == "Second"
-        assert tab_list[1].route_name == "dashboard.second"
+        assert tab_list[1].route_name == "dashboard.dashboard_second"
 
     def test_dashboard_insert(self, dashboard_tabs):
-        dashboard_tabs.insert("Next", "next")
+        dashboard_tabs.insert("Next", "dashboard.dashboard_next")
         assert list(dashboard_tabs) == [
-            DashboardTab("Home", "dashboard.index"),
-            DashboardTab("Next", "dashboard.next"),
+            DashboardTab("Home", "dashboard.dashboard_index"),
+            DashboardTab("Next", "dashboard.dashboard_next"),
         ]
 
-        dashboard_tabs.insert("Previous", "dashboard.previous", 1)
+        dashboard_tabs.insert("Previous", "dashboard.dashboard_previous", 1)
         assert list(dashboard_tabs) == [
-            DashboardTab("Home", "dashboard.index"),
-            DashboardTab("Previous", "dashboard.previous"),
-            DashboardTab("Next", "dashboard.next"),
+            DashboardTab("Home", "dashboard.dashboard_index"),
+            DashboardTab("Previous", "dashboard.dashboard_previous"),
+            DashboardTab("Next", "dashboard.dashboard_next"),
         ]
 
     def test_dashboard_insert_before(self, dashboard_tabs):
-        dashboard_tabs.insert_before_route("First", "first", "index")
+        dashboard_tabs.insert_before_route(
+            "First", "dashboard.dashboard_first", "dashboard.dashboard_index"
+        )
         assert list(dashboard_tabs) == [
-            DashboardTab("First", "dashboard.first"),
-            DashboardTab("Home", "dashboard.index"),
+            DashboardTab("First", "dashboard.dashboard_first"),
+            DashboardTab("Home", "dashboard.dashboard_index"),
         ]
 
         dashboard_tabs.insert_before_route(
-            "Second", "dashboard.second", "dashboard.index"
+            "Second", "dashboard.dashboard_second", "dashboard.dashboard_index"
         )
         assert list(dashboard_tabs) == [
-            DashboardTab("First", "dashboard.first"),
-            DashboardTab("Second", "dashboard.second"),
-            DashboardTab("Home", "dashboard.index"),
+            DashboardTab("First", "dashboard.dashboard_first"),
+            DashboardTab("Second", "dashboard.dashboard_second"),
+            DashboardTab("Home", "dashboard.dashboard_index"),
         ]
 
     def test_dashboard_insert_after(self, dashboard_tabs):
-        dashboard_tabs.insert_after_route("Last", "last", "index")
+        dashboard_tabs.insert_after_route(
+            "Last", "dashboard.dashboard_last", "dashboard.dashboard_index"
+        )
         assert list(dashboard_tabs) == [
-            DashboardTab("Home", "dashboard.index"),
-            DashboardTab("Last", "dashboard.last"),
+            DashboardTab("Home", "dashboard.dashboard_index"),
+            DashboardTab("Last", "dashboard.dashboard_last"),
         ]
 
         dashboard_tabs.insert_after_route(
-            "Second", "dashboard.second", "dashboard.index"
+            "Second", "dashboard.dashboard_second", "dashboard.dashboard_index"
         )
         assert list(dashboard_tabs) == [
-            DashboardTab("Home", "dashboard.index"),
-            DashboardTab("Second", "dashboard.second"),
-            DashboardTab("Last", "dashboard.last"),
+            DashboardTab("Home", "dashboard.dashboard_index"),
+            DashboardTab("Second", "dashboard.dashboard_second"),
+            DashboardTab("Last", "dashboard.dashboard_last"),
         ]
 
     def test_dashboard_remove(self, dashboard_tabs):
-        dashboard_tabs.insert("Last", "last")
+        dashboard_tabs.insert("Last", "dashboard.dashboard_last")
         assert len(list(dashboard_tabs)) == 2
 
-        dashboard_tabs.remove("last")
-        assert list(dashboard_tabs) == [DashboardTab("Home", "dashboard.index")]
+        dashboard_tabs.remove("dashboard.dashboard_last")
+        assert list(dashboard_tabs) == [
+            DashboardTab("Home", "dashboard.dashboard_index")
+        ]
 
-        dashboard_tabs.insert("Last", "last")
+        dashboard_tabs.insert("Last", "dashboard.dashboard_last")
         assert len(list(dashboard_tabs)) == 2
 
-        dashboard_tabs.remove("dashboard.last")
-        assert list(dashboard_tabs) == [DashboardTab("Home", "dashboard.index")]
+        dashboard_tabs.remove("dashboard.dashboard_last")
+        assert list(dashboard_tabs) == [
+            DashboardTab("Home", "dashboard.dashboard_index")
+        ]
 
-        dashboard_tabs.remove("index")
+        dashboard_tabs.remove("dashboard.dashboard_index")
         assert len(list(dashboard_tabs)) == 0
 
     def test_deferred_tab_decorator(self, cleared_tab_routes):
@@ -124,6 +135,7 @@ class TestDashboardTabs(object):
             "tab": None,
             "kwargs": (("methods", ["POST", "GET"]),),
             "func_name": "fake_route",
+            "name": "fake_route",
         }
 
 
@@ -441,27 +453,32 @@ class TestDashboardMonitorRoute(object):
 
         assert resp.status_code == 200
         resp_text = resp.data.decode("utf8")
-        assert '<h5 class="card-title">Participants</h5>' in resp_text
-        assert "<li>working: 0</li>" in resp_text
+        for title in ["Participants", "Networks", "Nodes", "Infos"]:
+            assert f'<h5 class="card-title">{title}</h5>' in resp_text
 
-    def test_statistics_show_working(self, webapp_admin, db_session):
+    def test_statistics_show_status_and_count(self, webapp_admin, db_session):
         from dallinger.models import Participant
 
-        participant = Participant(
-            recruiter_id="hotair",
-            worker_id="1",
-            hit_id="1",
-            assignment_id="1",
-            mode="test",
-        )
-        db_session.add(participant)
+        for i in range(2):
+            _id = str(i + 1)
+            participant = Participant(
+                recruiter_id="hotair",
+                worker_id=_id,
+                hit_id=_id,
+                assignment_id=_id,
+                mode="test",
+            )
+            db_session.add(participant)
 
         resp = webapp_admin.get("/dashboard/monitoring")
 
         assert resp.status_code == 200
         resp_text = resp.data.decode("utf8")
         assert '<h5 class="card-title">Participants</h5>' in resp_text
-        assert "<li>working: 1</li>" in resp_text
+        assert (
+            '<span class="statistics-key">working</span>: <span class="statistics-value">2</span>'
+            in resp_text
+        )
 
     def test_custom_vis_options(self, webapp_admin):
         # The HTML is customized using a property on the model class
@@ -501,6 +518,7 @@ class TestDashboardNetworkInfo(object):
         info4 = a.info(origin=source2, contents="contents3")
         a.transformation(info_in=info1, info_out=info2)
         a.transformation(info_in=info3, info_out=info4)
+        db_session.commit()
         yield exp
 
     def test_network_structure(self, a, db_session):
@@ -681,10 +699,10 @@ class TestDashboardDatabase(object):
         assert webapp.get("/dashboard/database").status_code == 401
 
     def test_render(self, active_config, webapp_admin):
-        resp = webapp_admin.get("/dashboard/database?model_type=Network")
+        resp = webapp_admin.get("/dashboard/database?table=network")
 
         assert resp.status_code == 200
-        assert "<h1>Database View: Networks</h1>" in resp.data.decode("utf8")
+        assert "<h1>Database View: Network</h1>" in resp.data.decode("utf8")
 
     def test_table_data(self, a, db_session):
         from dallinger.experiment_server.experiment_server import Experiment
@@ -694,7 +712,7 @@ class TestDashboardDatabase(object):
 
         network = Network.query.all()[0]
 
-        table = exp.table_data(model_type=["Network"])
+        table = exp.table_data(table="network")
         assert len(table["data"]) == 1
         assert table["data"][0]["id"] == network.id
         assert table["data"][0]["role"] == network.role
@@ -708,7 +726,7 @@ class TestDashboardDatabase(object):
 
         source = a.source(network=network)
 
-        table = exp.table_data(model_type="Node")
+        table = exp.table_data(table="node")
         assert len(table["data"]) == 1
         assert table["data"][0]["id"] == source.id
         assert table["data"][0]["type"] == source.type
@@ -721,7 +739,7 @@ class TestDashboardDatabase(object):
             raise KeyError("'id' not in Node columns")
 
         p = a.participant()
-        table = exp.table_data(model_type="Participant")
+        table = exp.table_data(table="participant")
         assert len(table["data"]) == 1
         assert table["data"][0]["id"] == p.id
         assert table["data"][0]["type"] == p.type
@@ -882,7 +900,9 @@ class TestDashboardDatabase(object):
 
         webapp, renderer = mock_renderer
         p = a.participant()
-        webapp.get("/dashboard/database?model_type=Participant")
+        participant_id = p.id
+        assignment_id = p.assignment_id
+        webapp.get("/dashboard/database?table=participant")
         renderer.assert_called_once()
         render_args = renderer.call_args[1]
         dt_options = json.loads(render_args["datatables_options"])
@@ -907,19 +927,19 @@ class TestDashboardDatabase(object):
         data = dt_options["data"]
         assert len(data) == 1
         participant_data = data[0]
-        assert participant_data["id"] == p.id
+        assert participant_data["id"] == participant_id
         assert participant_data["object_type"] == "Participant"
-        assert participant_data["assignment_id"] == p.assignment_id
+        assert participant_data["assignment_id"] == assignment_id
 
     def test_actions_with_mturk(self, a, active_config, mock_renderer):
         webapp, renderer = mock_renderer
         active_config.extend({"mode": "live", "recruiter": "mturk"})
-        webapp.get("/dashboard/database?model_type=Participant")
+        webapp.get("/dashboard/database?table=participant")
         render_args = renderer.call_args[1]
         assert render_args["is_sandbox"] is False
 
         active_config.extend({"mode": "sandbox", "recruiter": "mturk"})
-        webapp.get("/dashboard/database?model_type=Participant")
+        webapp.get("/dashboard/database?table=participant")
         render_args = renderer.call_args[1]
         assert render_args["is_sandbox"] is True
 
@@ -933,7 +953,7 @@ class TestDashboardDatabase(object):
             actions.return_value = [
                 {"name": "special_action", "title": "Special Action"}
             ]
-            webapp.get("/dashboard/database?model_type=Participant")
+            webapp.get("/dashboard/database?table=participant")
             render_args = renderer.call_args[1]
             dt_options = json.loads(render_args["datatables_options"])
             actions = dt_options["buttons"][1]
