@@ -10,10 +10,11 @@ import tempfile
 import textwrap
 import uuid
 from pathlib import Path
+from unittest import mock
 
-import mock
 import pexpect
 import pytest
+import requests
 import six
 from pytest import raises
 from six.moves import configparser
@@ -57,7 +58,7 @@ def faster(tempdir, active_config):
 
 @pytest.fixture
 def launch():
-    with mock.patch("dallinger.deployment._handle_launch_data") as hld:
+    with mock.patch("dallinger.deployment.handle_launch_data") as hld:
         hld.return_value = {"recruitment_msg": "fake\nrecruitment\nlist"}
         yield hld
 
@@ -370,14 +371,14 @@ class TestSetupExperiment(object):
         assert copy == orig
 
     def test_setup_uses_specified_python_version(self, active_config, setup_experiment):
-        active_config.extend({"heroku_python_version": "3.8.7"})
+        active_config.extend({"heroku_python_version": "3.13.1"})
 
         exp_id, dst = setup_experiment(log=mock.Mock())
 
         with open(os.path.join(dst, "runtime.txt"), "r") as file:
             version = file.read()
 
-        assert version == "python-3.8.7"
+        assert version == "python-3.13.1"
 
     def test_setup_copies_docker_script(self, setup_experiment):
         exp_id, dst = setup_experiment(log=mock.Mock())
@@ -488,7 +489,7 @@ class TestSetupExperiment(object):
             log = mock.Mock()
             tmp_dir = assemble_experiment_temp_dir(log, active_config)
 
-        assert "dallinger==" in (Path(tmp_dir) / "requirements.txt").read_text()
+        assert "dallinger" in (Path(tmp_dir) / "requirements.txt").read_text()
 
     @pytest.mark.slow
     def test_build_egg_if_in_development(self, active_config):
@@ -733,13 +734,12 @@ class TestDeploySandboxSharedSetupFullSystem(object):
 
 
 @pytest.mark.usefixtures("bartlett_dir")
-@pytest.mark.slow
-class Test_handle_launch_data(object):
+class Testhandle_launch_data(object):
     @pytest.fixture
     def handler(self):
-        from dallinger.deployment import _handle_launch_data
+        from dallinger.deployment import handle_launch_data
 
-        return _handle_launch_data
+        return handle_launch_data
 
     def test_success(self, handler):
         log = mock.Mock()
@@ -750,19 +750,17 @@ class Test_handle_launch_data(object):
             mock_post.return_value = result
             assert handler("/some-launch-url", error=log) == {"message": "msg!"}
 
-    def test_failure(self, handler):
-        from requests.exceptions import HTTPError
-
+    def test_failure_mock(self, handler):
         log = mock.Mock()
         with mock.patch("dallinger.deployment.requests.post") as mock_post:
             mock_post.return_value = mock.Mock(
                 ok=False,
                 json=mock.Mock(return_value={"message": "msg!"}),
-                raise_for_status=mock.Mock(side_effect=HTTPError),
+                raise_for_status=mock.Mock(side_effect=requests.exceptions.HTTPError),
                 status_code=500,
                 text="Failure",
             )
-            with pytest.raises(HTTPError):
+            with pytest.raises(requests.exceptions.HTTPError):
                 handler("/some-launch-url", error=log, delay=0.05, attempts=3)
 
         log.assert_has_calls(
@@ -776,11 +774,37 @@ class Test_handle_launch_data(object):
                     "Experiment launch failed. Trying again (attempt 3 of 3) in 0.2 seconds ..."
                 ),
                 mock.call("Error accessing /some-launch-url (500):\nFailure"),
-                mock.call("Experiment launch failed, check web dyno logs for details."),
+                mock.call("Experiment launch failed after multiple attempts."),
                 mock.call("msg!"),
             ]
         )
 
+    def test_failure_real(self, handler):
+        log = mock.Mock()
+
+        try:
+            handler("https://httpbin.org/status/500", log, attempts=1)
+        except requests.exceptions.HTTPError:
+            pass
+        log.assert_has_calls(
+            [
+                mock.call(
+                    "Error parsing response from https://httpbin.org/status/500, check server logs for details.\n"
+                ),
+                mock.call("Experiment launch failed after multiple attempts."),
+            ]
+        )
+
+        log.reset_mock()
+        try:
+            handler("https://nonexistent.example.com/", log, attempts=1)
+        except requests.exceptions.ConnectionError:
+            pass
+        assert (
+            "Error accessing https://nonexistent.example.com/"
+            in log.call_args_list[0][0][0]
+        )
+
     def test_non_json_response_error(self, handler):
         log = mock.Mock()
         with mock.patch("dallinger.deployment.requests.post") as mock_post:
@@ -791,10 +815,57 @@ class Test_handle_launch_data(object):
                 handler("/some-launch-url", error=log)
 
         log.assert_called_once_with(
-            "Error parsing response from /some-launch-url, check web dyno logs for details: "
+            "Error parsing response from /some-launch-url, check server logs for details.\n\n"
             "Big, unexpected problem."
         )
 
+    def test_error_log_messages(self, handler):
+        log = mock.Mock()
+        with (
+            mock.patch("dallinger.deployment.requests.post") as mock_post,
+            mock.patch("dallinger.deployment.print_bold") as mock_print,
+            mock.patch("dallinger.deployment.time.sleep"),
+        ):
+            mock_response = mock.Mock(
+                ok=False,
+                json=mock.Mock(return_value={"message": "msg!"}),
+                status_code=500,
+                text="Failure",
+            )
+            mock_response.raise_for_status = mock.Mock(
+                side_effect=requests.exceptions.HTTPError
+            )
+            mock_post.return_value = mock_response
+
+            # Test Heroku context
+            with pytest.raises(requests.exceptions.HTTPError):
+                handler(
+                    "https://example.com/some-launch-url", error=log, context="heroku"
+                )
+            mock_print.assert_called_once_with(
+                "For detailed server logs, visit the Papertrail add-on in your Heroku dashboard"
+            )
+
+            # Test SSH context with Dozzle
+            mock_print.reset_mock()
+            with pytest.raises(requests.exceptions.HTTPError):
+                handler(
+                    "https://example.com/some-launch-url",
+                    error=log,
+                    context="ssh",
+                    dns_host="example.com",
+                    dozzle_password="secret",
+                )
+            mock_print.assert_called_once_with(
+                "Check the detailed server logs at https://logs.example.com (user = dallinger, password = secret)"
+            )
+
+            # Test local context
+            mock_print.reset_mock()
+            with pytest.raises(requests.exceptions.HTTPError):
+                handler("/some-launch-url", error=log, context="local")
+            mock_print.assert_not_called()
+
 
 @pytest.mark.usefixtures("bartlett_dir", "clear_workers", "env")
 @pytest.mark.slow
@@ -922,23 +993,18 @@ class TestDebugServer(object):
                 pass
 
     def test_failure(self, debugger):
-        from requests.exceptions import HTTPError
-
         with mock.patch("dallinger.deployment.HerokuLocalDeployment.WRAPPER_CLASS"):
-            # For some reason this test alone triggers a failure in the line in
-            # deployment.py that invokes `pkg_resources.require`.
-            # Monkey patching it here makes it work.
-            # It would be interesting to get to the root cause.
-            with mock.patch("dallinger.utils.pkg_resources"):
-                with mock.patch("dallinger.deployment.requests.post") as mock_post:
-                    mock_post.return_value = mock.Mock(
-                        ok=False,
-                        json=mock.Mock(return_value={"message": "msg!"}),
-                        raise_for_status=mock.Mock(side_effect=HTTPError),
-                        status_code=500,
-                        text="Failure",
-                    )
-                    debugger.run()
+            with mock.patch("dallinger.deployment.requests.post") as mock_post:
+                mock_post.return_value = mock.Mock(
+                    ok=False,
+                    json=mock.Mock(return_value={"message": "msg!"}),
+                    raise_for_status=mock.Mock(
+                        side_effect=requests.exceptions.HTTPError
+                    ),
+                    status_code=500,
+                    text="Failure",
+                )
+                debugger.run()
 
         # Only one launch attempt should be made in debug mode
         debugger.out.error.assert_has_calls(
@@ -946,7 +1012,7 @@ class TestDebugServer(object):
                 mock.call(
                     "Error accessing http://localhost:5000/launch (500):\nFailure"
                 ),
-                mock.call("Experiment launch failed, check web dyno logs for details."),
+                mock.call("Experiment launch failed after multiple attempts."),
                 mock.call("msg!"),
             ]
         )
@@ -986,7 +1052,6 @@ class TestDockerServer(object):
             p.expect_exact("Recruitment is complete", timeout=180)
             p.expect_exact("'status': 'success'", timeout=120)
             p.expect_exact("Experiment completed", timeout=10)
-            p.expect_exact("Container bartlett1932-web-1         Stopped", timeout=30)
             p.expect(pexpect.EOF)
         finally:
             try:
@@ -1018,7 +1083,6 @@ class TestDockerServer(object):
             p.expect_exact("Recruitment is complete", timeout=240)
             p.expect_exact("'status': 'success'", timeout=120)
             p.expect_exact("Experiment completed", timeout=20)
-            p.expect_exact("Container bartlett1932-web-1         Stopped", timeout=20)
             p.expect(pexpect.EOF)
         finally:
             try:
@@ -1137,7 +1201,10 @@ class TestConstraints(object):
             constraints_file = Path(tmp_path) / "constraints.txt"
             # If not present a `constraints.txt` file will be generated
             assert constraints_file.exists()
-            assert "toml" in constraints_file.read_text()
+            if sys.version_info >= (3, 11):
+                assert "toml" not in constraints_file.read_text()
+            else:
+                assert "toml" in constraints_file.read_text()
 
             # An existing file will be left untouched
             constraints_file.write_text("foobar")
