@@ -1,7 +1,7 @@
 import json
 from datetime import datetime
+from unittest import mock
 
-import mock
 import pytest
 
 from dallinger import models
@@ -56,7 +56,7 @@ class TestAppConfiguration(object):
                 "dallinger.experiment_server.gunicorn.ProxyFix"
             ) as ProxyFix:
                 StandaloneServer().load()
-                ProxyFix.called_once_with(webapp.application)
+                ProxyFix.assert_called_once_with(webapp.application)
 
     def test_load_sets_flask_secret_from_env(self, webapp, active_config, env):
         webapp.application.debug = False
@@ -115,7 +115,9 @@ class TestAppConfiguration(object):
     ):
         active_config.set("protected_routes", '["/robots.txt"]')
 
-        assert webapp_admin.get("/robots.txt").status == "200 OK"
+        resp = webapp_admin.get("/robots.txt")
+        assert resp.status == "200 OK"
+        resp.close()
 
 
 @pytest.mark.usefixtures("experiment_dir")
@@ -290,11 +292,26 @@ class TestWorkerComplete(object):
     def test_sets_end_time(self, a, webapp, db_session):
         participant = a.participant()
         webapp.post("/worker_complete", data={"participant_id": participant.id})
-        assert db_session.merge(participant).end_time is not None
+        participant = db_session.merge(participant)
+        assert participant.end_time is not None
+
+    def test_immune_to_double_submission(self, a, webapp, db_session):
+        participant = a.participant()
+        webapp.post("/worker_complete", data={"participant_id": participant.id})
+        participant = db_session.merge(participant)
+        end_time_after_first_submission = participant.end_time
+
+        # re-submit
+        webapp.post("/worker_complete", data={"participant_id": participant.id})
+
+        participant = db_session.merge(participant)
+        assert participant.end_time == end_time_after_first_submission
 
     def test_records_notification_if_debug_mode(self, a, webapp):
         webapp.post("/worker_complete", data={"participant_id": a.participant().id})
-        assert models.Notification.query.one().event_type == "AssignmentSubmitted"
+        assert (
+            models.Notification.query.one().event_type == "RecruiterSubmissionComplete"
+        )
 
     def test_records_notification_if_bot_recruiter(self, a, webapp, active_config):
         webapp.post(
@@ -302,7 +319,10 @@ class TestWorkerComplete(object):
             data={"participant_id": a.participant(recruiter_id="bots").id},
         )
 
-        assert models.Notification.query.one().event_type == "BotAssignmentSubmitted"
+        assert (
+            models.Notification.query.one().event_type
+            == "BotRecruiterSubmissionComplete"
+        )
 
     def test_records_notification_for_non_mturk_recruiter(
         self, a, webapp, active_config
@@ -313,7 +333,9 @@ class TestWorkerComplete(object):
             data={"participant_id": a.participant(recruiter_id="cli").id},
         )
 
-        assert models.Notification.query.one().event_type == "AssignmentSubmitted"
+        assert (
+            models.Notification.query.one().event_type == "RecruiterSubmissionComplete"
+        )
 
     def test_records_no_notification_mturk_recruiter_and_nondebug(
         self, a, webapp, active_config
@@ -381,7 +403,7 @@ class TestRecruiterExit(object):
         participant = a.participant(assignment_id="some distinctive ID")
         resp = webapp.get("/recruiter-exit?participant_id={}".format(participant.id))
 
-        assert participant.assignment_id in str(resp.data)
+        assert "some distinctive ID" in str(resp.data)
 
     def test_mturk_recruiter_renders_hit_submission_form(
         self, a, webapp, active_config
@@ -416,7 +438,7 @@ class TestHandleError(object):
         assert resp.status_code == 200
         notifications = models.Notification.query.all()
         assert len(notifications) == 2
-        assert notifications[0].event_type == "AssignmentSubmitted"
+        assert notifications[0].event_type == "RecruiterSubmissionComplete"
         assert notifications[1].event_type == "ExperimentError"
 
     def test_saves_error_without_participant(self, a, webapp):
@@ -441,7 +463,7 @@ class TestHandleError(object):
 
         notifications = models.Notification.query.all()
         assert len(notifications) == 2
-        assert notifications[0].event_type == "AssignmentSubmitted"
+        assert notifications[0].event_type == "RecruiterSubmissionComplete"
         assert notifications[1].event_type == "ExperimentError"
         assert notifications[1].assignment_id == assignment_id
         assert (
@@ -456,7 +478,7 @@ class TestHandleError(object):
 
         notifications = models.Notification.query.all()
         assert len(notifications) == 2
-        assert notifications[0].event_type == "AssignmentSubmitted"
+        assert notifications[0].event_type == "RecruiterSubmissionComplete"
         assert notifications[1].event_type == "ExperimentError"
         assert notifications[1].assignment_id == assignment_id
         assert (
@@ -484,7 +506,7 @@ class TestHandleError(object):
 
         notifications = models.Notification.query.all()
         assert len(notifications) == 2
-        assert notifications[0].event_type == "AssignmentSubmitted"
+        assert notifications[0].event_type == "RecruiterSubmissionComplete"
         assert notifications[1].event_type == "ExperimentError"
         assert notifications[1].assignment_id == assignment_id
         assert (
@@ -517,7 +539,7 @@ class TestHandleError(object):
 
         notifications = models.Notification.query.all()
         assert len(notifications) == 2
-        assert notifications[0].event_type == "AssignmentSubmitted"
+        assert notifications[0].event_type == "RecruiterSubmissionComplete"
         assert notifications[1].event_type == "ExperimentError"
         assert notifications[1].assignment_id == assignment_id
         assert (
@@ -588,10 +610,12 @@ class TestSimpleGETRoutes(object):
         resp = webapp.get("/favicon.ico")
         assert resp.content_type == "image/x-icon"
         assert resp.content_length > 0
+        resp.close()
 
     def test_robots(self, webapp):
         resp = webapp.get("/robots.txt")
         assert b"User-agent" in resp.data
+        resp.close()
 
     def test_consent(self, webapp):
         resp = webapp.get(
@@ -666,12 +690,13 @@ class TestParticipantGetRoute(object):
 class TestParticipantByAssignmentRoute(object):
     def test_load_participant_calls_experiment_method(self, a, webapp):
         p = a.participant()
+        assignment_id = p.assignment_id
         with mock.patch(
             "dallinger.experiment.Experiment.load_participant"
         ) as load_participant:
-            load_participant.side_effect = lambda *args: p
-            webapp.post("/load-participant", data={"assignment_id": p.assignment_id})
-            load_participant.assert_called_once_with(p.assignment_id)
+            load_participant.side_effect = lambda *args: None
+            webapp.post("/load-participant", data={"assignment_id": assignment_id})
+            load_participant.assert_called_once_with(assignment_id)
 
     def test_load_participant(self, a, webapp):
         p = a.participant()
@@ -699,13 +724,16 @@ class TestParticipantByAssignmentRoute(object):
         self, a, db_session, webapp
     ):
         p = a.participant()
+        assignment_id = p.assignment_id
+        worker_id = p.worker_id
+        hit_id = p.hit_id
         with mock.patch(
             "dallinger.experiment.Experiment.normalize_entry_information"
         ) as normalizer:
             normalizer.side_effect = lambda *args: {
-                "assignment_id": p.assignment_id,
-                "worker_id": p.worker_id,
-                "hit_id": p.hit_id,
+                "assignment_id": assignment_id,
+                "worker_id": worker_id,
+                "hit_id": hit_id,
                 "entry_information": args[-1],
             }
             resp = webapp.post("/load-participant", data={"random_info": "123"})
@@ -718,6 +746,13 @@ class TestParticipantByAssignmentRoute(object):
 @pytest.mark.usefixtures("experiment_dir", "db_session")
 @pytest.mark.slow
 class TestParticipantCreateRoute(object):
+
+    def create_participant(self, a, **kw):
+        if "recruiter_name" in kw:
+            kw["recruiter_id"] = kw["recruiter_name"]
+        del kw["recruiter_name"]
+        return a.participant(**kw)
+
     @pytest.fixture
     def overrecruited(self, a):
         with mock.patch(
@@ -727,13 +762,14 @@ class TestParticipantCreateRoute(object):
             mock_exp.protected_routes = []
             mock_exp.is_overrecruited.return_value = True
             mock_exp.quorum = 50
-            mock_exp.create_participant.return_value = a.participant()
+            mock_exp.create_participant.side_effect = (
+                lambda **args: self.create_participant(a, **args)
+            )
             mock_class.return_value = mock_exp
 
             yield mock_class
 
     def test_create_participant_calls_experiment_method(self, a, webapp):
-        p = a.participant()
         with mock.patch(
             "dallinger.experiment_server.experiment_server.Experiment"
         ) as mock_class:
@@ -741,7 +777,9 @@ class TestParticipantCreateRoute(object):
             mock_exp.protected_routes = []
             mock_exp.is_overrecruited.return_value = False
             mock_exp.quorum = None
-            mock_exp.create_participant.side_effect = lambda **args: p
+            mock_exp.create_participant.side_effect = (
+                lambda **args: self.create_participant(a, **args)
+            )
             mock_class.return_value = mock_exp
 
             webapp.post("/participant/1/1/1/debug")
@@ -966,12 +1004,14 @@ class TestSummaryRoute(object):
 
     def test_summary_two_participants_with_different_status(self, a, webapp):
         p1 = a.participant()
-        p2 = a.participant()
+        p2 = a.participant(worker_id="2", hit_id="2", assignment_id="2")
         network = a.star()
         network.add_node(a.node(network=network, participant=p1))
         network.add_node(a.node(network=network, participant=p2))
         p1.status = "submitted"
         p2.status = "approved"
+        # Commit status changes so they can be read by the flask routes
+        a.db.commit()
 
         resp = webapp.get("/summary")
         data = json.loads(resp.data.decode("utf8"))
@@ -1356,11 +1396,14 @@ class TestNodeNeighbors(object):
         network = a.network()
         node1 = a.node(network=network)
         node2 = a.node(network=network)
+        node2_id = node2.id
         node1.connect(node2)
+        # Commit the connection so it can be read by the flask route
+        a.db.commit()
 
         resp = webapp.get("/node/{}/neighbors".format(node1.id))
         data = json.loads(resp.data.decode("utf8"))
-        assert data["nodes"][0]["id"] == node2.id
+        assert data["nodes"][0]["id"] == node2_id
 
     def test_pings_experiment(self, a, webapp):
         node = a.node()
@@ -1404,13 +1447,15 @@ class TestNodeReceivedInfos(object):
         receiver = a.node(network=net)
         sender.connect(direction="to", whom=receiver)
         info = a.info(origin=sender, contents="foo")
+        info_id = info.id
         sender.transmit(what=sender.infos()[0], to_whom=receiver)
         receiver.receive()
+        a.db.commit()
 
         resp = webapp.get("/node/{}/received_infos".format(receiver.id))
         data = json.loads(resp.data.decode("utf8"))
 
-        assert data["infos"][0]["id"] == info.id
+        assert data["infos"][0]["id"] == info_id
         assert data["infos"][0]["contents"] == "foo"
 
     def test_returns_empty_if_no_infos_received_by_node(self, a, webapp):
@@ -1462,6 +1507,7 @@ class TestTransformationGet(object):
         node = a.node()
         node_id = node.id  # save so we don't have to merge sessions
         node.replicate(a.info(origin=node))
+        a.db.commit()
 
         resp = webapp.get(
             "/node/{}/transformations?transformation_type=Replication".format(node_id)
@@ -1568,22 +1614,67 @@ class TestLaunchRoute(object):
         data = json.loads(resp.get_data())
         assert "recruitment_msg" in data
 
-    def test_launch_logging_fails(self, webapp):
+    def test_launch_with_recruitment(self, webapp, active_config):
         with mock.patch(
             "dallinger.experiment_server.experiment_server.Experiment"
         ) as mock_class:
-            bad_log = mock.Mock(side_effect=IOError)
-            mock_exp = mock.Mock(log=bad_log)
+            mock_exp = mock.Mock()
             mock_exp.protected_routes = []
+            mock_exp.background_tasks = []
+            mock_exp.channel = None
+            mock_exp.recruiter.open_recruitment.return_value = {
+                "items": ["item"],
+                "message": "a message",
+            }
             mock_class.return_value = mock_exp
             resp = webapp.post("/launch", data={})
+        assert resp.status_code == 200
+        mock_exp.recruiter.open_recruitment.assert_called()
 
-        assert resp.status_code == 500
-        data = json.loads(resp.get_data())
-        assert data == {
-            "message": "IOError writing to experiment log: ",
-            "status": "error",
-        }
+    def test_launch_establishes_channel_subscription(self, webapp, active_config):
+        with mock.patch(
+            "dallinger.experiment_server.experiment_server.Experiment"
+        ) as mock_class:
+            mock_exp = mock.Mock()
+            mock_exp.protected_routes = []
+            mock_exp.background_tasks = []
+            mock_exp.channel = "special"
+            mock_exp.recruiter.open_recruitment.return_value = {
+                "items": ["item"],
+                "message": "a message",
+            }
+            mock_class.return_value = mock_exp
+            with mock.patch(
+                "dallinger.experiment_server.sockets.chat_backend"
+            ) as mock_chat:
+                webapp.post("/launch", data={})
+                # We should have subscribed the experiment to the specified
+                # channel and the experiment control channel.
+                assert mock_chat.subscribe.call_count == 2
+                assert mock_chat.subscribe.mock_calls[0].args == (mock_exp, "special")
+                assert mock_chat.subscribe.mock_calls[1].args == (
+                    mock_exp,
+                    "dallinger_control",
+                )
+
+    def test_launch_without_channel_gets_no_subscriptions(self, webapp, active_config):
+        with mock.patch(
+            "dallinger.experiment_server.experiment_server.Experiment"
+        ) as mock_class:
+            mock_exp = mock.Mock()
+            mock_exp.protected_routes = []
+            mock_exp.background_tasks = []
+            mock_exp.channel = None
+            mock_exp.recruiter.open_recruitment.return_value = {
+                "items": ["item"],
+                "message": "a message",
+            }
+            mock_class.return_value = mock_exp
+            with mock.patch(
+                "dallinger.experiment_server.sockets.chat_backend"
+            ) as mock_chat:
+                webapp.post("/launch", data={})
+                assert mock_chat.subscribe.call_count == 0
 
 
 @pytest.mark.usefixtures("experiment_dir")
@@ -1645,6 +1736,8 @@ class TestWorkerFunctionIntegration(object):
         from dallinger.information import TrackingEvent
         from dallinger.models import Participant
 
+        exp = Experiment(db_session)
+
         participant = Participant(
             recruiter_id="hotair",
             worker_id="1",
@@ -1657,7 +1750,6 @@ class TestWorkerFunctionIntegration(object):
         db_session.commit()
         participant_id = participant.id
 
-        exp = Experiment(db_session)
         network = exp.get_network_for_participant(participant)
         node = exp.create_node(participant, network)
         exp.add_node_to_network(node, network)
@@ -1678,23 +1770,49 @@ class TestWorkerFunctionIntegration(object):
         assert event.origin.participant_id == participant_id
         assert event.details["test"] is True
 
+    def test_converts_timestamp_and_sets_time(self, a, worker_func):
+        participant = a.participant()
+        receive_time = datetime.now()
+
+        with mock.patch(self.dispatcher) as mock_baseclass:
+            runner = mock.Mock()
+            mock_baseclass.for_name = mock.Mock(return_value=runner)
+            worker_func(
+                event_type="MockEvent",
+                assignment_id=None,
+                participant_id=participant.id,
+                receive_timestamp=receive_time.timestamp(),
+            )
+            mock_baseclass.for_name.assert_called_once_with("MockEvent")
+            assert runner.call_args[1]["receive_time"] == receive_time
+            assert isinstance(runner.call_args[1]["now"], datetime)
+
 
 class TestWorkerEvents(object):
     def test_dispatch(self):
         from dallinger.experiment_server.worker_events import (
-            AssignmentSubmitted,
+            RecruiterSubmissionComplete,
             WorkerEvent,
         )
 
-        cls = WorkerEvent.for_name("AssignmentSubmitted")
+        cls = WorkerEvent.for_name("RecruiterSubmissionComplete")
 
-        assert cls is AssignmentSubmitted
+        assert cls is RecruiterSubmissionComplete
 
     def test_dispatch_with_unsupported_event_type(self):
         from dallinger.experiment_server.worker_events import WorkerEvent
 
         assert WorkerEvent.for_name("nonsense") is None
 
+    def test_event_subclass_registered(self):
+        from dallinger.experiment_server.worker_events import WorkerEvent
+
+        class MyCustomEventClass(WorkerEvent):
+            pass
+
+        # The meta class automatically registers new classes by name
+        assert WorkerEvent.for_name("MyCustomEventClass") is MyCustomEventClass
+
 
 end_time = datetime(2000, 1, 1)
 
@@ -1716,6 +1834,7 @@ def standard_args(experiment):
 
     participant = mock.Mock(
         spec_set=Participant,
+        id="42",
         status="working",
         worker_id="123",
         assignment_id="some assignment id",
@@ -1728,13 +1847,16 @@ def standard_args(experiment):
         "session": mock.Mock(spec_set=scoped_session),
         "config": {},
         "now": end_time,
+        "receive_time": end_time,
     }.copy()
 
 
-class TestAssignmentSubmitted(object):
+class TestRecruiterSubmissionComplete(object):
     @pytest.fixture
     def runner(self, standard_args):
-        from dallinger.experiment_server.worker_events import AssignmentSubmitted
+        from dallinger.experiment_server.worker_events import (
+            RecruiterSubmissionComplete,
+        )
 
         experiment = standard_args["experiment"]
         experiment.attention_check.return_value = True
@@ -1743,108 +1865,21 @@ class TestAssignmentSubmitted(object):
         experiment.bonus_reason.return_value = "You rock."
         standard_args["config"].update({"base_payment": 1.00})
 
-        return AssignmentSubmitted(**standard_args)
+        return RecruiterSubmissionComplete(**standard_args)
 
-    def test_calls_reward_bonus_if_experiment_returns_bonus_more_than_one_cent(
-        self, runner
-    ):
-        runner.experiment.bonus.return_value = 0.02
+    def test_calls_on_recruiter_submission_complete(self, runner):
         runner()
-        runner.participant.recruiter.reward_bonus.assert_called_once_with(
-            runner.participant, 0.02, "You rock."
-        )
-
-    def test_no_reward_bonus_if_experiment_returns_bonus_less_than_one_cent(
-        self, runner
-    ):
-        runner()
-        runner.participant.recruiter.reward_bonus.assert_not_called()
-        assert "NOT paying bonus" in str(runner.experiment.log.call_args_list)
-
-    def test_sets_participant_bonus_regardless(self, runner):
-        runner.experiment.bonus.return_value = 0.005
-        runner()
-        assert runner.participant.bonus == 0.005
-
-    def test_approve_hit_called_on_recruiter(self, runner):
-        runner()
-        runner.participant.recruiter.approve_hit.assert_called_once_with(
-            "some assignment id"
-        )
-
-    def test_participant_base_pay_set(self, runner):
-        runner()
-        assert runner.participant.base_pay == 1.0
-
-    def test_participant_status_set(self, runner):
-        runner()
-        assert runner.participant.status == "approved"
-
-    def test_approves_overrecruited_participants(self, runner):
-        runner.participant.status = "overrecruited"
-        runner()
-        assert runner.participant.status == "approved"
-
-    def test_participant_end_time_set(self, runner):
-        runner()
-        assert runner.participant.end_time == end_time
-
-    def test_submission_successful_called_on_experiment(self, runner):
-        runner()
-        runner.experiment.submission_successful.assert_called_once_with(
-            participant=runner.participant
-        )
-
-    def test_recruit_called_on_experiment(self, runner):
-        runner()
-        runner.experiment.recruit.assert_called_once()
-
-    def test_does_nothing_if_already_approved_worker(self, runner):
-        runner.participant.status = "approved"
-        runner()
-        runner.session.assert_not_called()
-
-    def test_sets_participant_status_on_failed_data_check(self, runner):
-        runner.experiment.data_check.return_value = False
-        runner()
-        assert runner.participant.status == "bad_data"
-
-    def test_calls_data_check_failed_on_failed_data_check(self, runner):
-        runner.experiment.data_check.return_value = False
-        runner()
-        runner.experiment.data_check_failed.assert_called_once_with(
-            participant=runner.participant
-        )
-
-    def test_recruits_another_on_failed_data_check(self, runner):
-        runner.experiment.data_check.return_value = False
-        runner()
-        runner.experiment.recruiter.recruit.assert_called_once_with(n=1)
-
-    def test_no_bonus_inquiry_on_failed_data_check(self, runner):
-        runner.experiment.data_check.return_value = False
-        runner()
-        runner.experiment.bonus.assert_not_called()
-
-    def test_sets_participant_status_on_failed_attention_check(self, runner):
-        runner.experiment.attention_check.return_value = False
-        runner()
-        assert runner.participant.status == "did_not_attend"
-
-    def test_calls_attention_check_failed_on_failed_attention_check(self, runner):
-        runner.experiment.attention_check.return_value = False
-        runner()
-        runner.experiment.attention_check_failed.assert_called_once_with(
-            participant=runner.participant
-        )
+        runner.experiment.on_recruiter_submission_complete.assert_called_once()
 
 
-class TestBotAssignmentSubmitted(object):
+class TestBotRecruiterSubmissionComplete(object):
     @pytest.fixture
     def runner(self, standard_args):
-        from dallinger.experiment_server.worker_events import BotAssignmentSubmitted
+        from dallinger.experiment_server.worker_events import (
+            BotRecruiterSubmissionComplete,
+        )
 
-        return BotAssignmentSubmitted(**standard_args)
+        return BotRecruiterSubmissionComplete(**standard_args)
 
     def test_participant_status_set(self, runner):
         runner()
@@ -2004,3 +2039,22 @@ class TestNotificationMissing(object):
         runner.participant.end_time = marker
         runner()
         assert runner.participant.end_time is marker
+
+
+class TestWebSocketMessage(object):
+    @pytest.fixture
+    def runner(self, standard_args):
+        from dallinger.experiment_server.worker_events import WebSocketMessage
+
+        return WebSocketMessage(**standard_args)
+
+    def test_calls_experiment_recieve(self, runner):
+        runner.details = {"message": '{"key":"value"}', "channel_name": "exp_channel"}
+        runner()
+        runner.experiment.receive_message.assert_called_once_with(
+            '{"key":"value"}',
+            channel_name="exp_channel",
+            participant=runner.participant,
+            node=runner.node,
+            receive_time=end_time,
+        )
