@@ -4,8 +4,8 @@ import datetime
 import os
 import signal
 import time
+from unittest import mock
 
-import mock
 import pytest
 
 from dallinger.config import get_config
@@ -31,9 +31,11 @@ def check_output():
 
 
 class TestClockScheduler(object):
+    @pytest.fixture
     def setup(self):
         """Set up the environment by moving to the demos directory."""
-        os.chdir("tests/experiment")
+        root_dir = os.path.dirname(__file__)
+        os.chdir(f"{root_dir}/experiment")
         config = get_config()
         config.ready = False
         from dallinger.heroku import clock
@@ -41,7 +43,7 @@ class TestClockScheduler(object):
         self.clock = clock
 
     @pytest.fixture
-    def patched_scheduler(self):
+    def patched_scheduler(self, setup):
         from dallinger import heroku
 
         with mock.patch("apscheduler.schedulers.blocking.BlockingScheduler.start"):
@@ -50,25 +52,17 @@ class TestClockScheduler(object):
     def teardown(self):
         os.chdir("../..")
 
-    def test_scheduler_has_job(self):
-        jobs = self.clock.scheduler.get_jobs()
-        assert len(jobs) == 1
-        assert (
-            jobs[0].func_ref
-            == "dallinger.heroku.clock:check_db_for_missing_notifications"
-        )
+    def test_scheduler_has_job(self, setup):
+        assert len(self.clock.scheduler.get_jobs()) > 0
 
     def test_launch_loads_config(self, patched_scheduler):
         self.clock.launch()
-        assert patched_scheduler.start.called_once()
+        patched_scheduler.start.assert_called_once()
         assert get_config().ready
 
     def test_launch_registers_additional_tasks(
         self, patched_scheduler, tasks_with_cleanup
     ):
-        jobs = patched_scheduler.get_jobs()
-        assert len(jobs) == 1
-
         tasks_with_cleanup.append(
             {
                 "func_name": "test_task",
@@ -79,10 +73,11 @@ class TestClockScheduler(object):
 
         self.clock.launch()
         jobs = patched_scheduler.get_jobs()
-        assert len(jobs) == 2
+        func_names = [job.func_ref for job in jobs]
+
         assert (
-            jobs[1].func_ref
-            == "dallinger_experiment.dallinger_experiment:TestExperiment.test_task"
+            "dallinger_experiment.dallinger_experiment:TestExperiment.test_task"
+            in func_names
         )
 
 
@@ -217,8 +212,20 @@ class TestHerokuApp(object):
     def test_name(self, app):
         assert app.name == "dlgr-fake-uid"
 
-    def test_url(self, app):
+    def test_url(self, app, check_call, check_output):
+        check_output.return_value = (
+            '{"app": {"web_url": "https://dlgr-fake-uid.herokuapp.com"}}'
+        )
         assert app.url == "https://dlgr-fake-uid.herokuapp.com"
+        check_output.assert_called_once_with(
+            [
+                "heroku",
+                "apps:info",
+                "--app",
+                app.name,
+                "--json",
+            ]
+        )
 
     def test_config_url(self, app):
         assert app.config_url == "https://api.heroku.com/apps/dlgr-fake-uid/config-vars"
@@ -233,6 +240,9 @@ class TestHerokuApp(object):
         )
 
     def test_bootstrap_creates_app_with_team(self, app, check_call, check_output):
+        check_output.return_value = (
+            '{"app": {"web_url": "https://dlgr-fake-uid.herokuapp.com"}}'
+        )
         app.team = "some-team"
         app.bootstrap()
         check_call.assert_has_calls(
@@ -253,6 +263,9 @@ class TestHerokuApp(object):
         )
 
     def test_bootstrap_sets_variables(self, app, check_call, check_output):
+        check_output.return_value = (
+            '{"app": {"web_url": "https://dlgr-fake-uid.herokuapp.com"}}'
+        )
         app.team = "some-team"
         app.bootstrap()
         check_call.assert_called_with(
@@ -527,12 +540,7 @@ class TestHerokuLocalWrapper(object):
 
         wrapper = HerokuLocalWrapper(config, output, env=env)
         yield wrapper
-        try:
-            print("Calling stop() on {}".format(wrapper))
-            print(wrapper._record[-1])
-            wrapper.stop(signal.SIGKILL)
-        except (IndexError, TypeError):
-            pass
+        wrapper.stop()
 
     def test_start(self, heroku):
         assert heroku.start()
@@ -562,15 +570,15 @@ class TestHerokuLocalWrapper(object):
         with pytest.raises(HerokuStartupError):
             heroku.start()
 
-    def test_start_fails_if_stream_ends_without_matching_success_regex(self, heroku):
+    def test_start_fails_if_port_never_opens(self, heroku):
         from dallinger.heroku.tools import HerokuStartupError
 
         heroku._stream = mock.Mock(
             return_value=["apple", "orange", heroku.STREAM_SENTINEL]
         )
-        heroku.success_regex = "not going to match anything"
-        with pytest.raises(HerokuStartupError):
-            heroku.start()
+        with mock.patch("dallinger.utils.port_is_open", return_value=False):
+            with pytest.raises(HerokuStartupError):
+                heroku.start()
         assert not heroku.is_running
 
     def test_error_flushes_logs(self, heroku):
@@ -579,17 +587,17 @@ class TestHerokuLocalWrapper(object):
         heroku._stream = mock.Mock(
             return_value=["apple", "orange", heroku.STREAM_SENTINEL]
         )
-        heroku.success_regex = "not going to match anything"
         heroku._log_failure = mock.Mock()
-        with pytest.raises(HerokuStartupError):
-            heroku.start()
+        with mock.patch("dallinger.utils.port_is_open", return_value=False):
+            with pytest.raises(HerokuStartupError):
+                heroku.start()
         heroku._log_failure.assert_called_once()
 
     def test_failure_logs_until_process_end(self, heroku):
         heroku._stream = mock.Mock(
             return_value=["real", "stopped", heroku.STREAM_SENTINEL]
         )
-        heroku._process = mock.Mock()
+        heroku._process = mock.Mock(pid=12345)
         heroku._process.poll = mock.Mock(return_value=1)
         heroku._log_failure()
         heroku._process.poll.assert_called_once()
@@ -605,7 +613,7 @@ class TestHerokuLocalWrapper(object):
                 heroku.STREAM_SENTINEL,
             ]
         )
-        heroku._process = mock.Mock()
+        heroku._process = mock.Mock(pid=12345)
         heroku._process.poll = mock.Mock(return_value=None)
         heroku._log_failure()
         assert heroku._record == ["real", "more"]
@@ -619,7 +627,7 @@ class TestHerokuLocalWrapper(object):
             yield heroku.STREAM_SENTINEL
 
         heroku._stream = mock.Mock(return_value=timeout_stream())
-        heroku._process = mock.Mock()
+        heroku._process = mock.Mock(pid=12345)
         heroku._process.poll = mock.Mock(return_value=None)
         start_time = time.time()
         heroku._log_failure()
@@ -635,7 +643,11 @@ class TestHerokuLocalWrapper(object):
         heroku.start()
         heroku._process.terminate()
         heroku.stop()
-        mock.call("Local Heroku was already terminated.") in heroku.out.log.mock_calls
+        log_calls = [call.args[0] for call in heroku.out.log.mock_calls]
+        assert (
+            "Local Heroku was already terminated." in log_calls
+            or "Local Heroku process terminated." in log_calls
+        )
 
     def test_start_when_shell_command_fails(self, heroku):
         heroku.shell_command = "nonsense"
