@@ -9,6 +9,7 @@ import six
 from pytest import mark, raises
 
 from dallinger import models, nodes
+from dallinger.db import Base, get_all_mapped_classes, get_polymorphic_mapping
 from dallinger.information import Gene
 from dallinger.nodes import Agent, Source
 from dallinger.transformations import Mutation
@@ -108,6 +109,11 @@ class TestModels(object):
             "property4": None,
             "property5": None,
             "details": {},
+            "n_alive_nodes": 3,
+            "n_failed_nodes": 0,
+            "n_completed_infos": 2,
+            "n_pending_infos": 0,
+            "n_failed_infos": 0,
             "object_type": "Network",
         }
 
@@ -150,7 +156,7 @@ class TestModels(object):
         assert len(net.transformations(failed="all", type=models.Transformation)) == 1
 
         for t in net.transformations(failed="all"):
-            assert type(t.node) == Agent
+            assert isinstance(t.node, Agent)
 
         # test latest_transmission_recipient
         assert net.latest_transmission_recipient() == agent
@@ -755,3 +761,60 @@ class TestModels(object):
         # make sure private data is not in there
         assert "unique_id" not in participant_json
         assert "worker_id" not in participant_json
+
+    def test_get_all_mapped_classes(self, db_session):
+        net = models.Network()
+        participant = models.Participant(
+            recruiter_id="hotair",
+            worker_id=str(1),
+            hit_id=str(1),
+            assignment_id=str(1),
+            mode="test",
+        )
+
+        db_session.add(net)
+        db_session.add(participant)
+        db_session.commit()
+
+        node = models.Node(network=net)
+        agent = Agent(network=net, participant=participant)
+        source = Source(network=net)
+
+        db_session.add(node)
+        db_session.add(agent)
+        db_session.add(source)
+        db_session.commit()
+
+        classes = get_all_mapped_classes()
+
+        assert classes["Participant"] == {
+            "cls": models.Participant,
+            "table": "participant",
+            "polymorphic_identity": "participant",
+        }
+
+        assert classes["Source"] == {
+            "cls": Source,
+            "table": "node",
+            "polymorphic_identity": "generic_source",
+        }
+
+        assert classes["Agent"] == {
+            "cls": Agent,
+            "table": "node",
+            "polymorphic_identity": "agent",
+        }
+
+        assert classes["Node"] == {
+            "cls": models.Node,
+            "table": "node",
+            "polymorphic_identity": "node",
+        }
+
+    def test_get_polymorphic_mapping(self, db_session):
+        table = Base.metadata.tables["node"]
+        mappers = get_polymorphic_mapping(table)
+
+        assert mappers["generic_source"] == Source
+        assert mappers["agent"] == Agent
+        assert mappers["node"] == models.Node
