@@ -1,13 +1,15 @@
 import json
 import os
 from datetime import datetime
+from unittest import mock
 
-import mock
 import pytest
 
 from dallinger.experiment import Experiment
 from dallinger.models import Participant
 from dallinger.mturk import MTurkQualificationRequirements, MTurkQuestions
+from dallinger.prolific import ProlificScreenOutDenied
+from dallinger.recruiters import MTurkRecruiterException
 
 
 class TestModuleFunctions(object):
@@ -17,11 +19,6 @@ class TestModuleFunctions(object):
 
         return recruiters
 
-    def test__get_queue(self, mod):
-        from rq import Queue
-
-        assert isinstance(mod._get_queue(), Queue)
-
     def test_for_experiment(self, mod):
         mock_exp = mock.MagicMock(spec=Experiment)
         mock_exp.recruiter = mock.sentinel.some_object
@@ -33,6 +30,16 @@ class TestModuleFunctions(object):
     def test_by_name_with_valid_nickname(self, mod):
         assert isinstance(mod.by_name("bots"), mod.BotRecruiter)
 
+    def test_by_name_with_custom_recruiter_valid_name(self, mod):
+        class CustomProlificRecruiter(mod.ProlificRecruiter):
+            def custom_method(self):
+                return "return value"
+
+        assert isinstance(mod.by_name("prolific"), CustomProlificRecruiter)
+        with pytest.raises(AttributeError) as e:
+            mod.ProlificRecruiter().custom_method()
+        assert e.match("'ProlificRecruiter' object has no attribute 'custom_method'")
+
     def test_by_name_with_invalid_name(self, mod):
         assert mod.by_name("blah") is None
 
@@ -72,6 +79,16 @@ class TestModuleFunctions(object):
         with pytest.raises(NotImplementedError):
             mod.from_config(stub_config)
 
+    @pytest.mark.usefixtures("db_session")
+    def test_run_status_check_calls_recruiters_with_their_participants(self, a, mod):
+        mock_recruiter = mock.Mock(spec=mod.Recruiter)
+        participant = a.participant()
+
+        with mock.patch("dallinger.recruiters.by_name") as mock_by_name:
+            mock_by_name.return_value = mock_recruiter
+            mod.run_status_check()
+            mock_recruiter.verify_status_of.assert_called_once_with([participant])
+
 
 class TestRecruiter(object):
     @pytest.fixture
@@ -100,6 +117,10 @@ class TestRecruiter(object):
         with pytest.raises(NotImplementedError):
             recruiter.reward_bonus(None, 0.01, "You're great!")
 
+    def test_verify_status_of(self, recruiter):
+        with pytest.raises(NotImplementedError):
+            recruiter.verify_status_of([])
+
     def test_external_submission_url(self, recruiter):
         assert recruiter.external_submission_url is None
 
@@ -187,8 +208,16 @@ class TestCLIRecruiter(object):
         result = recruiter.open_recruitment()
         assert "mode=new_mode" in result["items"][0]
 
-    def test_returns_standard_submission_event_type(self, recruiter):
-        assert recruiter.submitted_event() == "AssignmentSubmitted"
+    def test_on_task_completion__returns_event_type_and_new_status(self, recruiter):
+        assert recruiter.on_task_completion() == {
+            "new_status": "submitted",
+            "action": "RecruiterSubmissionComplete",
+        }
+
+    def test_verify_status_of_is_harmless_noop(self, a, recruiter):
+        p = a.participant()
+        p.recruiter_id = "cli"
+        recruiter.verify_status_of([p])
 
 
 @pytest.mark.usefixtures("active_config")
@@ -239,8 +268,16 @@ class TestHotAirRecruiter(object):
         result = recruiter.open_recruitment()
         assert "mode=debug" in result["items"][0]
 
-    def test_returns_standard_submission_event_type(self, recruiter):
-        assert recruiter.submitted_event() == "AssignmentSubmitted"
+    def test_on_task_completion__returns_event_type_and_new_status(self, recruiter):
+        assert recruiter.on_task_completion() == {
+            "new_status": "submitted",
+            "action": "RecruiterSubmissionComplete",
+        }
+
+    def test_verify_status_of_is_harmless_noop(self, a, recruiter):
+        p = a.participant()
+        p.recruiter_id = "hotair"
+        recruiter.verify_status_of([p])
 
 
 class TestSimulatedRecruiter(object):
@@ -262,12 +299,20 @@ class TestSimulatedRecruiter(object):
     def test_open_recruitment_multiple_returns_empty_result(self, recruiter):
         assert recruiter.open_recruitment(n=3)["items"] == []
 
-    def test_returns_standard_submission_event_type(self, recruiter):
-        assert recruiter.submitted_event() == "AssignmentSubmitted"
+    def test_on_task_completion__returns_event_type_and_new_status(self, recruiter):
+        assert recruiter.on_task_completion() == {
+            "new_status": "submitted",
+            "action": "RecruiterSubmissionComplete",
+        }
 
     def test_close_recruitment(self, recruiter):
         assert recruiter.close_recruitment() is None
 
+    def test_verify_status_of_is_harmless_noop(self, a, recruiter):
+        p = a.participant()
+        p.recruiter_id = "sim"
+        recruiter.verify_status_of([p])
+
 
 class TestBotRecruiter(object):
     @pytest.fixture
@@ -275,7 +320,7 @@ class TestBotRecruiter(object):
         from dallinger.recruiters import BotRecruiter
 
         with mock.patch.multiple(
-            "dallinger.recruiters", _get_queue=mock.DEFAULT, get_base_url=mock.DEFAULT
+            "dallinger.recruiters", get_queue=mock.DEFAULT, get_base_url=mock.DEFAULT
         ) as mocks:
             mocks["get_base_url"].return_value = "fake_base_url"
             r = BotRecruiter()
@@ -311,8 +356,11 @@ class TestBotRecruiter(object):
     def test_reward_bonus(self, a, recruiter):
         recruiter.reward_bonus(a.participant(), 0.01, "You're great!")
 
-    def test_returns_specific_submission_event_type(self, recruiter):
-        assert recruiter.submitted_event() == "BotAssignmentSubmitted"
+    def test_on_task_completion__returns_event_type_and_new_status(self, recruiter):
+        assert recruiter.on_task_completion() == {
+            "new_status": "submitted",
+            "action": "BotRecruiterSubmissionComplete",
+        }
 
     def test_notify_duration_exceeded_rejects_participants(self, a, recruiter):
         bot = a.participant(recruiter_id="bots")
@@ -321,6 +369,11 @@ class TestBotRecruiter(object):
 
         assert bot.status == "rejected"
 
+    def test_verify_status_of_is_harmless_noop(self, a, recruiter):
+        p = a.participant()
+        p.recruiter_id = "bots"
+        recruiter.verify_status_of([p])
+
 
 @pytest.fixture
 def notifies_admin():
@@ -344,10 +397,13 @@ def prolific_config(active_config):
         "prolific_api_token": "fake Prolific API token",
         "prolific_api_version": "v1",
         "prolific_estimated_completion_minutes": 5,
-        "prolific_reward_cents": 10,
+        "prolific_is_custom_screening": True,
+        "prolific_project": "My project",
         "prolific_recruitment_config": json.dumps(
             {"peripheral_requirements": ["audio", "microphone"]}
         ),
+        "prolific_workspace": "My workspace",
+        "publish_experiment": True,
     }
     active_config.extend(prolific_extensions)
 
@@ -364,7 +420,7 @@ def prolificservice(prolific_config, fake_parsed_prolific_study):
         api_version=prolific_config.get("prolific_api_version"),
     )
 
-    service.published_study.return_value = fake_parsed_prolific_study
+    service.create_study.return_value = fake_parsed_prolific_study
     service.add_participants_to_study.return_value = fake_parsed_prolific_study
 
     return service
@@ -373,7 +429,16 @@ def prolificservice(prolific_config, fake_parsed_prolific_study):
 @pytest.mark.usefixtures("prolific_config")
 class TestProlificRecruiter(object):
     @pytest.fixture
-    def recruiter(self, mailer, notifies_admin, prolificservice, hit_id_store):
+    def recruiter(
+        self,
+        request,
+        mailer,
+        notifies_admin,
+        prolificservice,
+        hit_id_store,
+        active_config,
+    ):
+        active_config.extend({"debug_recruiter": ""})
         from dallinger.recruiters import ProlificRecruiter
 
         with mock.patch.multiple(
@@ -384,13 +449,40 @@ class TestProlificRecruiter(object):
             r = ProlificRecruiter(store=hit_id_store)
             r.notifies_admin = notifies_admin
             r.mailer = mailer
-            r.prolificservice = prolificservice
+
+            # We don't want to mock prolificservice in some tests
+            if not hasattr(request, "param"):
+                r.prolificservice = prolificservice
 
             return r
 
+    @pytest.mark.parametrize("recruiter", [], indirect=True)
+    def test_open_recruitment_and_publish(self, recruiter):
+        recruiter.config["mode"] = "live"
+        recruiter.config["publish_experiment"] = True
+        with mock.patch.multiple(
+            "dallinger.prolific.ProlificService",
+            draft_study=mock.DEFAULT,
+            publish_study=mock.DEFAULT,
+        ) as mocks:
+            recruiter.open_recruitment(n=1)
+        mocks["publish_study"].assert_called()
+
+    @pytest.mark.parametrize("recruiter", [], indirect=True)
+    def test_open_recruitment_but_do_not_publish(self, recruiter):
+        recruiter.config["mode"] = "live"
+        recruiter.config["publish_experiment"] = False
+        with mock.patch.multiple(
+            "dallinger.prolific.ProlificService",
+            draft_study=mock.DEFAULT,
+            publish_study=mock.DEFAULT,
+        ) as mocks:
+            recruiter.open_recruitment(n=1)
+        mocks["publish_study"].assert_not_called()
+
     def test_open_recruitment_with_valid_request(self, recruiter):
         result = recruiter.open_recruitment(n=5)
-        assert result["message"] == "Study now published on Prolific"
+        assert result["message"] == "Study created on Prolific"
 
     def test_open_recruitment_raises_if_study_already_in_progress(self, recruiter):
         from dallinger.recruiters import ProlificRecruiterException
@@ -424,8 +516,10 @@ class TestProlificRecruiter(object):
             "entry_information": prolific_format,
         }
 
-    def test_defers_assignment_submission_via_null_submitted_event(self, recruiter):
-        assert recruiter.submitted_event() is None
+    def test_on_task_completion__returns_no_event_and_new_status(self, recruiter):
+        assert recruiter.on_task_completion() == {
+            "new_status": "recruiter_submission_started",
+        }
 
     @pytest.mark.usefixtures("experiment_dir_merged")
     def test_exit_page_includes_submission_prolific_button(self, a, webapp, recruiter):
@@ -468,14 +562,14 @@ class TestProlificRecruiter(object):
         fake_id = "fake assignment id"
         recruiter.approve_hit(fake_id)
 
-        recruiter.prolificservice.approve_participant_session.assert_called_once_with(
-            session_id=fake_id
+        recruiter.prolificservice.approve_participant_submission.assert_called_once_with(
+            submission_id=fake_id
         )
 
     def test_approve_hit_logs_exception(self, recruiter):
         from dallinger.prolific import ProlificServiceException
 
-        recruiter.prolificservice.approve_participant_session.side_effect = (
+        recruiter.prolificservice.approve_participant_submission.side_effect = (
             ProlificServiceException("Boom!")
         )
         with mock.patch("dallinger.recruiters.logger") as mock_logger:
@@ -491,12 +585,14 @@ class TestProlificRecruiter(object):
             study_id="abcdefghijklmnopqrstuvwx", number_to_add=1
         )
 
-    def test_submission_listener_enqueues_assignment_submitted_notification(
-        self, queue, webapp
+    def test_submission_listener_updates_status_and_enqueues_notification(
+        self, a, queue, webapp, db_session
     ):
+        participant = a.participant(assignment_id="some assignment ID")
+        participant_id = participant.id
         exit_form_submission = {
-            "assignmentId": "some assignment ID",
-            "participantId": "some participant ID",
+            "assignmentId": participant.assignment_id,
+            "participantId": f"{participant_id}",
             "somethingElse": "blah... whatever",
         }
 
@@ -504,10 +600,36 @@ class TestProlificRecruiter(object):
             "/prolific-submission-listener", data=exit_form_submission
         )
 
+        participant = db_session.merge(participant)
         assert response.status_code == 200
+        assert participant.status == "submitted"
         queue.enqueue.assert_called_once_with(
-            mock.ANY, "AssignmentSubmitted", "some assignment ID", "some participant ID"
-        ),
+            mock.ANY,
+            "RecruiterSubmissionComplete",
+            "some assignment ID",
+            f"{participant_id}",
+        )
+
+    def test_submission_listener_prevents_double_processing(self, a, queue, webapp):
+        participant = a.participant(assignment_id="some assignment ID")
+        participant_id = participant.id
+        exit_form_submission = {
+            "assignmentId": "some assignment ID",
+            "participantId": f"{participant_id}",
+            "somethingElse": "blah... whatever",
+        }
+
+        # Call twice
+        webapp.post("/prolific-submission-listener", data=exit_form_submission)
+        webapp.post("/prolific-submission-listener", data=exit_form_submission)
+
+        # Just one job added to queue
+        queue.enqueue.assert_called_once_with(
+            mock.ANY,
+            "RecruiterSubmissionComplete",
+            "some assignment ID",
+            f"{participant_id}",
+        )
 
     def test_clean_qualification_attributes(self, recruiter):
         json_path = os.path.join(
@@ -576,6 +698,161 @@ class TestProlificRecruiter(object):
             },
         ]
 
+    def test_verify_status_triggers_corrections(self, a, recruiter, queue):
+        p1 = a.participant(assignment_id="aaa111", recruiter_id="prolific")
+        p2 = a.participant(assignment_id="bbb222", recruiter_id="prolific")
+
+        # Set up mock response from Prolific regarding these participants:
+        recruiter.prolificservice.get_assignments_for_study.return_value = {
+            p1.assignment_id: {
+                "participant_id": p1.assignment_id,
+                "hit_id": "some-study-id",
+                "worker_id": "some-prolific-worker-id-1",
+                "started_at": "2021-05-20T11:23:00.457Z",
+                "status": "RETURNED",
+            },
+            p2.assignment_id: {
+                "participant_id": p2.assignment_id,
+                "hit_id": "some-study-id",
+                "worker_id": "some-prolific-worker-id-2",
+                "started_at": "2021-05-20T11:24:00.457Z",
+                "status": "TIMED-OUT",
+            },
+        }
+
+        recruiter.verify_status_of([p1, p2])
+
+        queue.enqueue.assert_has_calls(
+            [
+                mock.call(mock.ANY, "AssignmentReturned", "aaa111", p1.id),
+                mock.call(mock.ANY, "AssignmentAbandoned", "bbb222", p2.id),
+            ]
+        )
+
+    def test_verify_status_copes_with_assignments_not_in_prolific(
+        self, a, recruiter, queue
+    ):
+        p1 = a.participant(assignment_id="aaa111", recruiter_id="prolific")
+        p2 = a.participant(assignment_id="bbb222", recruiter_id="prolific")
+
+        # Set up mock response from Prolific where only the first participant is included:
+        recruiter.prolificservice.get_assignments_for_study.return_value = {
+            p1.assignment_id: {
+                "participant_id": p1.assignment_id,
+                "hit_id": "some-study-id",
+                "worker_id": "some-prolific-worker-id-1",
+                "started_at": "2021-05-20T11:23:00.457Z",
+                "status": "RETURNED",
+            },
+        }
+
+        recruiter.verify_status_of([p1, p2])
+
+        queue.enqueue.assert_has_calls(
+            [
+                mock.call(mock.ANY, "AssignmentReturned", "aaa111", p1.id),
+            ]
+        )
+
+    def test_notify_duration_exceeded_triggers_worker_events(self, a, recruiter, queue):
+        p1 = a.participant(assignment_id="aaa111", recruiter_id="prolific")
+
+        # Set up mock response from Prolific where only the first participant is included:
+        recruiter.prolificservice.get_participant_submission.return_value = {
+            "participant_id": p1.assignment_id,
+            "hit_id": "some-study-id",
+            "worker_id": "some-prolific-worker-id-1",
+            "started_at": "2021-05-20T11:23:00.457Z",
+            "status": "RETURNED",
+        }
+        reference_time = datetime.now()
+
+        recruiter.notify_duration_exceeded([p1], reference_time)
+
+        queue.enqueue.assert_has_calls(
+            [
+                mock.call(mock.ANY, "AssignmentReturned", "aaa111", p1.id),
+            ]
+        )
+
+    def test_validate_config_assert_publishing(self, a, recruiter):
+        recruiter.config["publish_experiment"] = True
+        recruiter.validate_config()
+
+    def test_validate_config_assert_not_publishing(self, a, recruiter):
+        recruiter.config["publish_experiment"] = False
+        recruiter.validate_config()
+
+    def test_screen_out_calls_service_with_correct_args(self, a, recruiter):
+        participant = a.participant()
+        participant.assignment_id = "test_submission_id"
+        participant.base_payment = 2.00
+        participant.bonus = 1.50
+
+        mock_response = {
+            "status": 204,
+            "message": "The request to bulk screen out has been made successfully.",
+            "payment_per_participant": {"amount": 3.50, "currency": "GBP"},
+        }
+
+        recruiter.prolificservice = mock.Mock()
+        recruiter.prolificservice.get_study = mock.Mock(
+            return_value={"is_custom_screening": True}
+        )
+        recruiter.prolificservice.screen_out = mock.Mock()
+        recruiter.prolificservice.screen_out.return_value = mock_response
+
+        recruiter._record_current_study_id("test_study_id")
+
+        result = recruiter.screen_out(participant, bonus=3.50)
+
+        recruiter.prolificservice.screen_out.assert_called_once_with(
+            study_id="test_study_id",
+            submission_ids=[participant.assignment_id],
+            bonus_per_submission=3.50,
+            increase_places=recruiter.config.get("auto_recruit", False),
+        )
+
+        assert participant.status == "screened_out"
+        assert result == mock_response
+
+    def test_screen_out_handles_rejection(self, a, recruiter):
+        participant = a.participant()
+        participant.assignment_id = "test_submission_id"
+        participant.base_payment = 2.00
+        participant.bonus = 1.50
+
+        error_response = dict(
+            {
+                "status": 400,
+                "message": "Screen out request denied",
+                "error": "Payment too low",
+            }
+        )
+
+        recruiter.prolificservice = mock.Mock()
+        recruiter.prolificservice.get_study = mock.Mock(
+            return_value={"is_custom_screening": True}
+        )
+        recruiter.prolificservice.screen_out = mock.Mock(return_value=error_response)
+
+        recruiter._record_current_study_id("test_study_id")
+
+        with pytest.raises(ProlificScreenOutDenied) as exc_info:
+            recruiter.screen_out(participant, bonus=3.50)
+
+        expected_error = f"Prolific denied screen-out request for participant {participant.id}: {error_response['message']}"
+        assert str(exc_info.value) == expected_error
+
+        recruiter.prolificservice.screen_out.assert_called_once_with(
+            study_id="test_study_id",
+            submission_ids=[participant.assignment_id],
+            bonus_per_submission=3.50,
+            increase_places=recruiter.config.get("auto_recruit", False),
+        )
+
+        assert participant.status != "screened_out"
+
 
 class TestMTurkRecruiterMessages(object):
     @pytest.fixture
@@ -721,7 +998,7 @@ def queue():
     from rq import Queue
 
     instance = mock.Mock(spec=Queue)
-    with mock.patch("dallinger.recruiters._get_queue") as mock_q:
+    with mock.patch("dallinger.recruiters.get_queue") as mock_q:
         mock_q.return_value = instance
 
         yield instance
@@ -790,6 +1067,7 @@ class TestMTurkRecruiter(object):
             mocks["get_base_url"].return_value = "http://fake-domain"
             mocks["os"].getenv.return_value = "fake-host-domain"
             active_config.extend({"mode": "sandbox"})
+            active_config.extend({"publish_experiment": True})
             r = MTurkRecruiter(store=hit_id_store)
             r.notifies_admin = notifies_admin
             r.mailer = mailer
@@ -797,14 +1075,6 @@ class TestMTurkRecruiter(object):
 
             return r
 
-    def test_instantiation_fails_with_invalid_mode(self, active_config):
-        from dallinger.recruiters import MTurkRecruiter, MTurkRecruiterException
-
-        active_config.extend({"mode": "nonsense"})
-        with pytest.raises(MTurkRecruiterException) as ex_info:
-            MTurkRecruiter()
-        assert ex_info.match('"nonsense" is not a valid mode')
-
     def test_config_passed_to_constructor_sandbox(self, recruiter):
         assert recruiter.config.get("title") == "fake experiment title"
 
@@ -830,8 +1100,6 @@ class TestMTurkRecruiter(object):
     def test_open_recruitment_raises_if_no_external_hit_domain_configured(
         self, recruiter
     ):
-        from dallinger.recruiters import MTurkRecruiterException
-
         recruiter.hit_domain = None
         with pytest.raises(MTurkRecruiterException):
             recruiter.open_recruitment(n=1)
@@ -950,14 +1218,14 @@ class TestMTurkRecruiter(object):
     def test_open_recruitment_raises_error_if_hit_already_in_progress(
         self, fake_parsed_hit, recruiter
     ):
-        from dallinger.recruiters import MTurkRecruiterException
-
         recruiter.open_recruitment()
         with pytest.raises(MTurkRecruiterException):
             recruiter.open_recruitment()
 
-    def test_supresses_assignment_submitted(self, recruiter):
-        assert recruiter.submitted_event() is None
+    def test_on_task_completion__returns_no_event_and_new_status(self, recruiter):
+        assert recruiter.on_task_completion() == {
+            "new_status": "recruiter_submission_started",
+        }
 
     def test_current_hit_id_with_active_experiment(self, recruiter, fake_parsed_hit):
         recruiter.open_recruitment()
@@ -1176,7 +1444,10 @@ class TestMTurkRecruiter(object):
         recruiter.notify_duration_exceeded(participants, datetime.now())
 
         queue.enqueue.assert_called_once_with(
-            worker_function, "AssignmentSubmitted", participants[0].assignment_id, None
+            worker_function,
+            "RecruiterSubmissionComplete",
+            participants[0].assignment_id,
+            None,
         )
         recruiter.notifies_admin.send.assert_called_once()
 
@@ -1253,6 +1524,31 @@ class TestMTurkRecruiter(object):
 
         recruiter.mturkservice.expire_hit.assert_not_called()
 
+    def test_validate_config_with_invalid_mode(self, a, recruiter):
+        recruiter.config["mode"] = "nonsense"
+
+        with pytest.raises(MTurkRecruiterException) as ex_info:
+            recruiter.validate_config()
+        assert ex_info.match('"nonsense" is not a valid mode')
+
+    def test_validate_config_assert_publishing(self, a, recruiter):
+        recruiter.config["publish_experiment"] = True
+        recruiter.validate_config()
+
+    def test_validate_config_assert_not_publishing(self, a, recruiter):
+        recruiter.config["publish_experiment"] = False
+        with pytest.raises(AssertionError) as ex_info:
+            recruiter.validate_config()
+        ex_info.match(
+            "MTurkRecruiter does not support delayed experiment publishing. Set `publish_experiment=true` in your experiment config!"
+        )
+
+    def test_validate_config_deploy(self, a, recruiter):
+        recruiter.validate_config(mode="live")
+
+    def test_validate_config_sandbox(self, a, recruiter):
+        recruiter.validate_config(mode="sandbox")
+
 
 class TestRedisTally(object):
     @pytest.fixture
@@ -1301,8 +1597,6 @@ class TestMTurkLargeRecruiter(object):
     def test_open_recruitment_raises_error_if_experiment_in_progress(
         self, fake_parsed_hit, recruiter
     ):
-        from dallinger.recruiters import MTurkRecruiterException
-
         recruiter.open_recruitment()
         with pytest.raises(MTurkRecruiterException):
             recruiter.open_recruitment()
